## Project settings

VERSION = 1.0.0

QT += core gui widgets

TARGET = Grinder
TEMPLATE = app

CONFIG += c++14
DEFINES += QT_DEPRECATED_WARNINGS

## Output directories

Debug:DESTDIR = ../bin-Debug
Release:DESTDIR = ../bin-Release

## Libraries

include(Libraries.pro)

## Platform specifics

win32 {
	RC_FILE = res/Grinder.rc
}

## Project files

SOURCES += \
	main.cpp \
	ui/mainwnd/GrinderWindow.cpp \
	core/GrinderApplication.cpp \
	Version.cpp \
	util/StringUtils.cpp \
	util/StringConv.cpp \
	pipeline/PipelineItem.cpp \
	pipeline/PipelineExceptions.cpp \
	pipeline/Block.cpp \
	pipeline/BlockType.cpp \
	pipeline/Port.cpp \
	pipeline/PortType.cpp \
	pipeline/Connection.cpp \
	pipeline/ConnectionVector.cpp \
	pipeline/PortVector.cpp \
	pipeline/BlockVector.cpp \
	pipeline/Pipeline.cpp \
	pipeline/PipelineManager.cpp \
	pipeline/PipelineVector.cpp \
	ui/graph/GraphView.cpp \
	ui/graph/GraphScene.cpp \
	ui/graph/GraphStyle.cpp \
	ui/graph/GraphNode.cpp \
	ui/graph/GraphBlockNode.cpp \
	ui/graph/GraphNodeFactory.cpp \
	ui/graph/GraphPortNode.cpp \
	ui/graph/GraphConnectionNode.cpp \
	ui/graph/GraphConnectionBase.cpp \
	ui/graph/GraphConnectionBlueprint.cpp \
	ui/graph/GraphConnectionMessage.cpp \
	pipeline/BlockCatalog.cpp \
	pipeline/BlockCategory.cpp \
	util/UIUtils.cpp \
	ui/StyleSheet.cpp \
	controller/PipelineController.cpp \
	ui/graph/GraphBlockList.cpp \
	ui/graph/GraphWidget.cpp \
	project/Label.cpp \
	project/LabelVector.cpp \
	project/Project.cpp \
	project/ProjectExceptions.cpp \
	controller/ProjectController.cpp \
	ui/widgets/ControlBar.cpp \
	ui/mainwnd/LabelsListItem.cpp \
	ui/mainwnd/LabelsListWidget.cpp \
	controller/GenericController.cpp \
	project/ImageReferenceVector.cpp \
	project/ImageReference.cpp \
	project/ProjectItem.cpp \
	ui/mainwnd/ImageReferencesListItem.cpp \
	ui/mainwnd/ImageReferencesListWidget.cpp \
	util/ImageUtils.cpp \
	util/FileUtils.cpp \
	ui/TemporaryStatusMessage.cpp \
	engine/data/DataDescriptor.cpp \
	util/DataUtils.cpp \
	core/GrinderSettings.cpp \
	engine/data/DataBlob.cpp \
	util/CVUtils.cpp \
	engine/data/DataExceptions.cpp \
	pipeline/BlockHierarchy.cpp \
	pipeline/blocks/BinaryThresholdBlock.cpp \
	engine/Engine.cpp \
	engine/EngineExceptions.cpp \
	controller/EngineController.cpp \
	engine/EngineExecutionContext.cpp \
	engine/processors/BinaryThresholdProcessor.cpp \
	engine/ProcessorBase.cpp \
	pipeline/blocks/ConvertToGrayscaleBlock.cpp \
	engine/processors/ConvertToGrayscaleProcessor.cpp \
	ui/graph/GraphLayout.cpp \
	core/GrinderExceptions.cpp \
	common/serialization/SettingsContainer.cpp \
	common/serialization/SerializationExceptions.cpp \
	common/serialization/SettingsCodec.cpp \
	common/serialization/JsonSettingsCodec.cpp \
	project/ProjectSerializer.cpp \
	common/serialization/SerializationContext.cpp \
	common/serialization/DeserializationContext.cpp \
	util/SerializationUtils.cpp \
	common/MRUStringList.cpp \
	ui/mainwnd/RecentProjectsMenu.cpp \
	ui/dlg/OptionsDialog.cpp \
	ui/properties/PropertyTreeWidget.cpp \
	ui/properties/PropertyTreeItem.cpp \
	ui/properties/BlockPropertyTreeItem.cpp \
	ui/properties/PropertyTreeItemDelegate.cpp \
	ui/properties/ValuePropertyTreeItem.cpp \
	ui/properties/editors/BoolPropertyEditor.cpp \
	ui/properties/editors/TextPropertyEditor.cpp \
	pipeline/blocks/OutputBlock.cpp \
	pipeline/blocks/InputBlock.cpp \
	image/ImageBuild.cpp \
	image/ImageBuildVector.cpp \
	image/ImageBuildPool.cpp \
	image/ImageExceptions.cpp \
	engine/processors/InputProcessor.cpp \
	engine/processors/OutputProcessor.cpp \
	ui/image/ImageEditorManager.cpp \
	ui/image/ImageEditorDockWidget.cpp \
	ui/visscene/VisualSceneView.cpp \
	ui/visscene/VisualSceneStyle.cpp \
	ui/image/ImageEditorView.cpp \
	ui/image/ImageEditorScene.cpp \
	controller/ImageEditorController.cpp \
	ui/image/ImageEditorStyle.cpp \
	image/ImageBuildItem.cpp \
	image/Layer.cpp \
	image/LayerVector.cpp \
	ui/image/LayersListWidget.cpp \
	ui/image/LayersListItem.cpp \
	common/properties/types/BoolProperty.cpp \
	common/properties/types/IntProperty.cpp \
	common/properties/types/RealProperty.cpp \
	common/properties/types/StringProperty.cpp \
	common/properties/types/UIntProperty.cpp \
	common/properties/PropertyBase.cpp \
	common/properties/PropertyExceptions.cpp \
	common/properties/PropertyID.cpp \
	common/properties/PropertyVector.cpp \
	image/DraftItemType.cpp \
	image/DraftItem.cpp \
	common/properties/types/PointProperty.cpp \
	common/properties/types/ColorProperty.cpp \
	image/DraftItemVector.cpp \
	image/DraftItemCatalog.cpp \
	image/draftitems/BoxDraftItem.cpp \
	image/draftitems/LineDraftItem.cpp \
	image/renderers/BoxDraftItemRenderer.cpp \
	image/renderers/LineDraftItemRenderer.cpp \
	common/properties/types/SizeProperty.cpp \
	ui/visscene/VisualNode.cpp \
	ui/image/ImageEditorNode.cpp \
	ui/image/DraftItemNode.cpp \
	ui/image/DraftItemNodeFactory.cpp \
	ui/image/ImageEditorTool.cpp \
	ui/image/tools/DraftItemTool.cpp \
	ui/image/tools/DefaultImageEditorTool.cpp \
	ui/image/tools/BoxDraftItemTool.cpp \
	ui/image/tools/LineDraftItemTool.cpp \
	ui/image/ImageEditorToolList.cpp \
	ui/image/draftitems/BoxDraftItemNode.cpp \
	ui/image/draftitems/LineDraftItemNode.cpp \
	ui/image/ImageEditorPropertyWidget.cpp \
	ui/properties/editors/PointPropertyEditor.cpp \
	ui/properties/editors/SizePropertyEditor.cpp \
	ui/widgets/ColorWidget.cpp \
	ui/image/tools/ColorPickerTool.cpp \
	ui/widgets/GrinderDockWidget.cpp \
	common/properties/types/AngleProperty.cpp \
	ui/properties/editors/AnglePropertyEditor.cpp \
	ui/image/ImageEditorEnvironment.cpp \
	util/MathUtils.cpp \
	ui/image/ImageEditorWidget.cpp \
	ui/image/ImageEditor.cpp \
	ui/image/ImageEditorComponent.cpp \
	ui/image/InPlaceEditorDragHandle.cpp \
	ui/image/InPlaceEditor.cpp \
	ui/image/editors/LinearInPlaceEditor.cpp \
	common/properties/PropertyObject.cpp \
	ui/image/editors/RectangularInPlaceEditor.cpp \
	ui/visscene/VisualSceneInputHandler.cpp \
	ui/image/ColorPresetsWidget.cpp \
	common/serialization/SerializationBase.cpp \
	core/ClipboardManager.cpp \
	project/properties/ImageReferenceProperty.cpp \
	ui/properties/editors/ImageReferencePropertyEditor.cpp \
	image/ImageTag.cpp \
	image/ImageTagVector.cpp \
	pipeline/blocks/ImageTagsBlock.cpp \
	ui/image/ImageTagsListItem.cpp \
	ui/image/ImageTagDialog.cpp \
	ui/properties/PropertyEditorBase.cpp \
	ui/dlg/CheckListDialog.cpp \
	ui/widgets/BaseListWidget.cpp \
	ui/image/BaseImageTagsListWidget.cpp \
	image/ImageTags.cpp \
	image/properties/ImageTagsProperty.cpp \
	ui/image/ImageTagsDialog.cpp \
	ui/properties/editors/ImageTagsPropertyEditor.cpp \
	ui/image/EditableImageTagsListWidget.cpp \
	ui/image/ImageTagsListWidget.cpp \
	image/ImageTagsAllotment.cpp \
	image/properties/ImageTagsAllotmentProperty.cpp \
	ui/properties/editors/ImageTagsAllotmentPropertyEditor.cpp \
	ui/DragInfo.cpp \
	image/LayerPixelsData.cpp \
	common/CommonOperators.cpp \
	image/LayerPixels.cpp \
	image/ItemRenderer.cpp \
	image/renderers/LayerPixelsRenderer.cpp \
	ui/image/LayerPixelsNode.cpp \
	ui/image/tools/PaintbrushTool.cpp \
	ui/image/tools/EraserTool.cpp \
	ui/LongOperationIndicator.cpp \
	ui/properties/editors/IntPropertyEditor.cpp \
	image/draftitems/PixelsDraftItem.cpp \
	image/renderers/PixelsDraftItemRenderer.cpp \
	ui/image/draftitems/PixelsDraftItemNode.cpp \
	ui/visscene/RubberBandNode.cpp \
	ui/image/tools/ZoomTool.cpp \
	ui/properties/editors/EnumPropertyEditor.cpp \
	common/properties/types/EnumPropertyBase.cpp \
	pipeline/properties/BinaryThresholdTypeProperty.cpp \
	pipeline/properties/BinaryThresholdModeProperty.cpp \
	pipeline/blocks/AdaptiveThresholdBlock.cpp \
	engine/processors/AdaptiveThresholdProcessor.cpp \
	pipeline/properties/AdaptiveThresholdTypeProperty.cpp \
	pipeline/properties/AdaptiveThresholdMethodProperty.cpp \
	ui/properties/editors/RealPropertyEditor.cpp \
	engine/processors/AlphaBlendingProcessor.cpp \
	pipeline/blocks/AlphaBlendingBlock.cpp \
    engine/processors/ConvertToColorProcessor.cpp \
    pipeline/blocks/ConvertToColorBlock.cpp \
    engine/processors/NormalizeProcessor.cpp \
    pipeline/blocks/NormalizeBlock.cpp \
    cv/ColorConv.cpp \
    pipeline/properties/ColorSpaceProperty.cpp \
    pipeline/blocks/MergeChannelsBlock.cpp \
    pipeline/blocks/SplitChannelsBlock.cpp \
    engine/processors/SplitChannelsProcessor.cpp \
    engine/processors/MergeChannelsProcessor.cpp \
    ui/image/LayerItemNode.cpp \
    ui/image/LayerBackgroundNode.cpp \
    ui/widgets/SliderValueWidget.cpp \
    cv/MatrixCompat.cpp \
    engine/processors/ReplaceColorProcessor.cpp \
    pipeline/blocks/ReplaceColorBlock.cpp \
    cv/Pixel.cpp \
    ui/properties/editors/ColorPropertyEditor.cpp

HEADERS += \
	ui/mainwnd/GrinderWindow.h \
	core/GrinderApplication.h \
	Grinder.h \
	Version.h \
	util/StringUtils.h \
	util/StringConv.h \
	pipeline/PipelineItem.h \
	pipeline/PipelineExceptions.h \
	pipeline/Block.h \
	pipeline/BlockType.h \
	pipeline/Port.h \
	pipeline/PortType.h \
	pipeline/Connection.h \
	pipeline/ConnectionVector.h \
	pipeline/PortVector.h \
	pipeline/BlockVector.h \
	pipeline/Pipeline.h \
	pipeline/PipelineManager.h \
	pipeline/PipelineVector.h \
	ui/graph/GraphView.h \
	ui/graph/GraphScene.h \
	ui/graph/GraphStyle.h \
	ui/graph/GraphNode.h \
	ui/graph/GraphBlockNode.h \
	ui/graph/GraphNodeFactory.h \
	ui/graph/GraphPortNode.h \
	ui/graph/GraphConnectionNode.h \
	ui/graph/GraphConnectionBase.h \
	ui/graph/GraphConnectionBlueprint.h \
	ui/graph/GraphConnectionMessage.h \
	pipeline/BlockCatalog.h \
	pipeline/BlockCategory.h \
	util/UIUtils.h \
	ui/StyleSheet.h \
	controller/PipelineController.h \
	ui/graph/GraphBlockList.h \
	ui/graph/GraphWidget.h \
	common/ObjectVector.h \
	common/ObjectVector.impl.h \
	project/Label.h \
	project/LabelVector.h \
	project/Project.h \
	project/ProjectExceptions.h \
	controller/ProjectController.h \
	ui/widgets/ControlBar.h \
	ui/mainwnd/LabelsListItem.h \
	ui/mainwnd/LabelsListWidget.h \
	controller/GenericController.h \
	controller/GenericController.impl.h \
	project/ImageReference.h \
	project/ImageReferenceVector.h \
	project/ProjectItem.h \
	ui/widgets/ObjectListItem.h \
	ui/widgets/ObjectListItem.impl.h \
	ui/widgets/ObjectListWidget.h \
	ui/widgets/ObjectListWidget.impl.h \
	ui/mainwnd/ImageReferencesListItem.h \
	ui/mainwnd/ImageReferencesListWidget.h \
	util/ImageUtils.h \
	util/FileUtils.h \
	ui/TemporaryStatusMessage.h \
	engine/data/DataDescriptor.h \
	util/DataUtils.h \
	core/GrinderSettings.h \
	engine/data/DataBlob.h \
	engine/data/DataBlob.impl.h \
	util/CVUtils.h \
	util/CVUtils.impl.h \
	engine/data/DataExceptions.h \
	engine/data/DataDescriptor.impl.h \
	pipeline/BlockHierarchy.h \
	pipeline/blocks/BinaryThresholdBlock.h \
	engine/Engine.h \
	engine/EngineExceptions.h \
	controller/EngineController.h \
	engine/EngineExecutionContext.h \
	engine/Processor.h \
	engine/processors/BinaryThresholdProcessor.h \
	engine/Processor.impl.h \
	engine/ProcessorBase.h \
	pipeline/blocks/ConvertToGrayscaleBlock.h \
	engine/processors/ConvertToGrayscaleProcessor.h \
	ui/graph/GraphLayout.h \
	core/GrinderExceptions.h \
	common/serialization/SettingsContainer.h \
	common/serialization/SerializationExceptions.h \
	common/serialization/SettingsContainer.impl.h \
	common/serialization/SettingsCodec.h \
	common/serialization/JsonSettingsCodec.h \
	project/ProjectSerializer.h \
	common/serialization/SerializationContext.h \
	common/serialization/DeserializationContext.h \
	util/SerializationUtils.h \
	util/SerializationUtils.impl.h \
	common/MRUStringList.h \
	common/MRUStringList.impl.h \
	ui/mainwnd/RecentProjectsMenu.h \
	ui/dlg/OptionsDialog.h \
	util/StringUtils.impl.h \
	ui/properties/PropertyTreeWidget.h \
	ui/properties/PropertyTreeItem.h \
	ui/properties/BlockPropertyTreeItem.h \
	ui/properties/PropertyTreeItemDelegate.h \
	ui/properties/ValuePropertyTreeItem.h \
	ui/properties/PropertyEditor.h \
	ui/properties/PropertyEditor.impl.h \
	ui/properties/editors/BoolPropertyEditor.h \
	ui/properties/editors/TextPropertyEditor.h \
	res/Resources.h \
	pipeline/blocks/OutputBlock.h \
	pipeline/blocks/InputBlock.h \
	image/ImageBuild.h \
	common/serialization/SerializationContext.impl.h \
	common/serialization/DeserializationContext.impl.h \
	image/ImageBuildVector.h \
	image/ImageBuildPool.h \
	image/ImageExceptions.h \
	engine/processors/InputProcessor.h \
	engine/processors/OutputProcessor.h \
	ui/image/ImageEditorManager.h \
	ui/image/ImageEditorDockWidget.h \
	common/AdjacentRange.h \
	ui/visscene/VisualScene.h \
	ui/visscene/VisualScene.impl.h \
	ui/visscene/VisualSceneView.h \
	ui/visscene/VisualSceneStyle.h \
	ui/widgets/MetaWidget.h \
	ui/widgets/MetaWidget.impl.h \
	ui/image/ImageEditorView.h \
	ui/image/ImageEditorScene.h \
	controller/ImageEditorController.h \
	ui/image/ImageEditorStyle.h \
	image/ImageBuildItem.h \
	image/Layer.h \
	image/LayerVector.h \
	ui/image/LayersListWidget.h \
	ui/image/LayersListItem.h \
	common/properties/types/BoolProperty.h \
	common/properties/types/IntProperty.h \
	common/properties/constraints/RangeConstraint.h \
	common/properties/constraints/RangeConstraint.impl.h \
	common/properties/types/RealProperty.h \
	common/properties/StandardProperties.h \
	common/properties/types/StringProperty.h \
	common/properties/types/UIntProperty.h \
	common/properties/Property.h \
	common/properties/Property.impl.h \
	common/properties/PropertyBase.h \
	common/properties/PropertyConstraint.h \
	common/properties/PropertyConstraint.impl.h \
	common/properties/PropertyExceptions.h \
	common/properties/PropertyID.h \
	common/properties/PropertyVector.h \
	common/properties/PropertyVector.impl.h \
	image/DraftItemType.h \
	image/DraftItem.h \
	common/properties/types/PointProperty.h \
	common/properties/types/ColorProperty.h \
	image/DraftItemVector.h \
	image/DraftItemCatalog.h \
	image/draftitems/BoxDraftItem.h \
	image/draftitems/LineDraftItem.h \
	image/DraftItemRenderer.h \
	image/renderers/BoxDraftItemRenderer.h \
	image/renderers/LineDraftItemRenderer.h \
	image/DraftItemRenderer.impl.h \
	common/properties/types/SizeProperty.h \
	ui/visscene/VisualNode.h \
	ui/image/ImageEditorNode.h \
	ui/visscene/VisualNodeFactory.h \
	ui/visscene/VisualNodeFactory.impl.h \
	ui/image/DraftItemNode.h \
	ui/image/DraftItemNodeFactory.h \
	ui/image/ImageEditorScene.impl.h \
	ui/image/ImageEditorTool.h \
	ui/image/tools/DraftItemTool.h \
	ui/image/tools/DefaultImageEditorTool.h \
	ui/image/tools/BoxDraftItemTool.h \
	ui/image/tools/LineDraftItemTool.h \
	ui/image/ImageEditorToolList.h \
	ui/image/ImageEditorToolList.impl.h \
	ui/image/draftitems/BoxDraftItemNode.h \
	ui/image/draftitems/LineDraftItemNode.h \
	ui/image/ImageEditorPropertyWidget.h \
	ui/properties/editors/PointPropertyEditor.h \
	ui/properties/editors/SizePropertyEditor.h \
	ui/widgets/ColorWidget.h \
	ui/image/tools/ColorPickerTool.h \
	ui/widgets/GrinderDockWidget.h \
	common/properties/types/AngleProperty.h \
	ui/properties/editors/AnglePropertyEditor.h \
	ui/image/ImageEditorEnvironment.h \
	util/MathUtils.h \
	ui/image/ImageEditorWidget.h \
	ui/image/ImageEditor.h \
	ui/image/ImageEditorComponent.h \
	ui/image/InPlaceEditorDragHandle.h \
	ui/image/InPlaceEditor.h \
	ui/image/editors/LinearInPlaceEditor.h \
	common/properties/PropertyObject.h \
	common/properties/PropertyObject.impl.h \
	ui/image/editors/RectangularInPlaceEditor.h \
	ui/visscene/VisualSceneInputHandler.h \
	ui/visscene/VisualSceneInputHandler.impl.h \
	ui/image/ColorPresetsWidget.h \
	common/serialization/SerializationBase.h \
	core/ClipboardManager.h \
	core/ClipboardManager.impl.h \
	project/properties/ImageReferenceProperty.h \
	ui/properties/editors/ImageReferencePropertyEditor.h \
	image/ImageTag.h \
	image/ImageTagVector.h \
	pipeline/blocks/ImageTagsBlock.h \
	common/properties/ObjectProperty.h \
	common/properties/ObjectProperty.impl.h \
	ui/properties/editors/DialogPropertyEditor.h \
	ui/properties/editors/DialogPropertyEditor.impl.h \
	ui/image/ImageTagsListItem.h \
	ui/image/ImageTagDialog.h \
	ui/properties/PropertyEditorBase.h \
	engine/ProcessorBase.impl.h \
	ui/widgets/ActiveObjectListItem.h \
	ui/widgets/ActiveObjectListItem.impl.h \
	ui/widgets/ActiveObjectListWidget.h \
	ui/widgets/ActiveObjectListWidget.impl.h \
	ui/widgets/CheckListWidget.h \
	ui/widgets/CheckListWidget.impl.h \
	ui/dlg/CheckListDialog.h \
	ui/dlg/CheckListDialog.impl.h \
	ui/widgets/BaseListWidget.h \
	ui/image/BaseImageTagsListWidget.h \
	pipeline/Block.impl.h \
	image/ImageTags.h \
	image/properties/ImageTagsProperty.h \
	ui/image/ImageTagsDialog.h \
	ui/properties/editors/ImageTagsPropertyEditor.h \
	ui/image/EditableImageTagsListWidget.h \
	ui/image/ImageTagsListWidget.h \
	image/ImageTagsAllotment.h \
	image/properties/ImageTagsAllotmentProperty.h \
	ui/properties/editors/ImageTagsAllotmentPropertyEditor.h \
	ui/DragInfo.h \
	common/RangeVector.h \
	common/RangeVector.impl.h \
	common/Range.h \
	common/Range.impl.h \
	common/RangeMap.h \
	common/RangeMap.impl.h \
	image/LayerPixelsData.h \
	common/CommonOperators.h \
	common/NestedIterator.h \
	common/NestedIterator.impl.h \
	image/LayerPixels.h \
	image/ItemRenderer.h \
	image/LayerItem.h \
	image/LayerItem.impl.h \
	image/renderers/LayerPixelsRenderer.h \
	ui/image/LayerPixelsNode.h \
	ui/image/tools/PaintbrushTool.h \
	ui/image/tools/EraserTool.h \
	ui/LongOperationIndicator.h \
	ui/properties/editors/IntPropertyEditor.h \
	ui/properties/editors/DualIntPropertyEditor.h \
	ui/properties/editors/DualIntPropertyEditor.impl.h \
	ui/widgets/AutoFocusWidget.h \
	image/draftitems/PixelsDraftItem.h \
	image/renderers/PixelsDraftItemRenderer.h \
	ui/image/draftitems/PixelsDraftItemNode.h \
	ui/visscene/RubberBandNode.h \
	ui/image/tools/ZoomTool.h \
	common/properties/types/EnumProperty.h \
	common/properties/types/EnumProperty.impl.h \
	ui/properties/editors/EnumPropertyEditor.h \
	common/properties/types/EnumPropertyBase.h \
	pipeline/properties/BinaryThresholdTypeProperty.h \
	pipeline/properties/BinaryThresholdModeProperty.h \
	pipeline/blocks/AdaptiveThresholdBlock.h \
	engine/processors/AdaptiveThresholdProcessor.h \
	pipeline/properties/AdaptiveThresholdTypeProperty.h \
	pipeline/properties/AdaptiveThresholdMethodProperty.h \
	common/properties/constraints/EvenOddConstraint.h \
	common/properties/constraints/EvenOddConstraint.impl.h \
	ui/properties/editors/RealPropertyEditor.h \
	engine/processors/AlphaBlendingProcessor.h \
	pipeline/blocks/AlphaBlendingBlock.h \
    engine/processors/ConvertToColorProcessor.h \
    pipeline/blocks/ConvertToColorBlock.h \
	cv/CVTypes.h \
    engine/processors/NormalizeProcessor.h \
    pipeline/blocks/NormalizeBlock.h \
    cv/ColorConv.h \
    pipeline/properties/ColorSpaceProperty.h \
    pipeline/blocks/MergeChannelsBlock.h \
    pipeline/blocks/SplitChannelsBlock.h \
    engine/processors/SplitChannelsProcessor.h \
    engine/processors/MergeChannelsProcessor.h \
    ui/image/LayerItemNode.h \
    ui/image/LayerBackgroundNode.h \
    ui/widgets/SliderValueWidget.h \
    cv/MatrixCompat.h \
    engine/processors/ReplaceColorProcessor.h \
    pipeline/blocks/ReplaceColorBlock.h \
    cv/Pixel.h \
    ui/properties/editors/ColorPropertyEditor.h

FORMS += \
	ui/mainwnd/GrinderWindow.ui \
	ui/dlg/OptionsDialog.ui \
	ui/image/ImageEditorWidget.ui \
	ui/image/ImageTagDialog.ui \
	ui/dlg/CheckListDialog.ui \
	ui/image/ImageTagsDialog.ui

RESOURCES += \
	res/Grinder.qrc

OTHER_FILES += \
	res/Grinder.rc
