/******************************************************************************
 * File: MRUStringList.cpp
 * Date: 01.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "MRUStringList.h"

MRUStringList::MRUStringList(bool caseSensitive, bool pathList, unsigned int entryLimit) :
	_isCaseSensitive{caseSensitive}, _isPathList{pathList}, _entryLimit{entryLimit}
{

}

void MRUStringList::add(QString entry)
{
	auto it = find(entry);

	if (it != _entries.begin() || empty())	// The entry is not at the front of the list
	{
		// Remove the item if it already exists
		if (it != _entries.end())
			_entries.erase(it);

		_entries.push_front(entry);
		purgeList();
	}
}

void MRUStringList::remove(QString entry)
{
	std::remove(_entries.begin(), _entries.end(), entry);
}

QStringList::const_iterator MRUStringList::find(QString entry) const
{
	// Remove constness to be able to call find; safe since we will return a const_iterator
	MRUStringList* list = const_cast<MRUStringList*>(this);
	return list->find(entry, _entries.cbegin(), _entries.cend());
}

void MRUStringList::purgeList()
{
	// Remove excessive entries
	if (_entryLimit > 0)
	{
		while (_entries.size() > static_cast<int>(_entryLimit))
			_entries.pop_back();
	}
}
