/******************************************************************************
 * File: NestedIterator.impl.h
 * Date: 17.5.2018
 *****************************************************************************/

#include "Grinder.h"
#include "NestedIterator.h"

template <typename OuterIt, bool OuterIsMap, bool InnerIsContainer, bool IsConst>
NestedIterator<OuterIt, OuterIsMap, InnerIsContainer, IsConst>::NestedIterator(outer_iterator beginOuter, outer_iterator endOuter) :
	_currentOuter{beginOuter}, _endOuter{endOuter}
{
	if (beginOuter != endOuter)	// Heed empty containers
		restartInnerIterator();
}

template <typename OuterIt, bool OuterIsMap, bool InnerIsContainer, bool IsConst>
NestedIterator<OuterIt, OuterIsMap, InnerIsContainer, IsConst>& NestedIterator<OuterIt, OuterIsMap, InnerIsContainer, IsConst>::operator ++()
{
	if (_currentInner)
	{
		if (++(*_currentInner) == *_endInner)	// Exceeded our current inner iterator, go to the next outer one
		{
			if (++_currentOuter != _endOuter)
				restartInnerIterator();
		}
	}

	return *this;
}

template <typename OuterIt, bool OuterIsMap, bool InnerIsContainer, bool IsConst>
template<bool isMap, bool isConst>
std::enable_if_t<isMap && !isConst, void> NestedIterator<OuterIt, OuterIsMap, InnerIsContainer, IsConst>::restartInnerIterator()
{
	_currentInner = std::make_unique<inner_iterator>(_currentOuter->second.begin());
	_endInner = std::make_unique<inner_iterator>(_currentOuter->second.end());
}

template <typename OuterIt, bool OuterIsMap, bool InnerIsContainer, bool IsConst>
template<bool isMap, bool isConst>
std::enable_if_t<!isMap && !isConst, void> NestedIterator<OuterIt, OuterIsMap, InnerIsContainer, IsConst>::restartInnerIterator()
{
	_currentInner = std::make_unique<inner_iterator>(_currentOuter->begin());
	_endInner = std::make_unique<inner_iterator>(_currentOuter->end());
}

template <typename OuterIt, bool OuterIsMap, bool InnerIsContainer, bool IsConst>
template<bool isMap, bool isConst>
std::enable_if_t<isMap && isConst, void> NestedIterator<OuterIt, OuterIsMap, InnerIsContainer, IsConst>::restartInnerIterator()
{
	_currentInner = std::make_unique<inner_iterator>(_currentOuter->second.cbegin());
	_endInner = std::make_unique<inner_iterator>(_currentOuter->second.cend());
}

template <typename OuterIt, bool OuterIsMap, bool InnerIsContainer, bool IsConst>
template<bool isMap, bool isConst>
std::enable_if_t<!isMap && isConst, void> NestedIterator<OuterIt, OuterIsMap, InnerIsContainer, IsConst>::restartInnerIterator()
{
	_currentInner = std::make_unique<inner_iterator>(_currentOuter->cbegin());
	_endInner = std::make_unique<inner_iterator>(_currentOuter->cend());
}
