/******************************************************************************
 * File: ObjectVector.h
 * Date: 14.1.2018
 *****************************************************************************/

#ifndef OBJECTVECTOR_H
#define OBJECTVECTOR_H

#include <vector>
#include <memory>

#include "common/serialization/SerializationContext.h"
#include "common/serialization/DeserializationContext.h"

namespace grndr
{
	template<typename ObjType>
	class ObjectVector : public std::vector<std::shared_ptr<ObjType>>
	{
	public:
		using vector_type = ObjectVector<ObjType>;
		using object_type = ObjType;
		using object_vec_type = std::vector<ObjType>;
		using pointer_type = std::shared_ptr<ObjType>;
		using pointer_vec_type = std::vector<pointer_type>;
		using size_type = typename std::vector<std::shared_ptr<ObjType>>::size_type;

	public:
		ObjectVector() { }
		ObjectVector(const vector_type& src) = default;
		ObjectVector(vector_type&& src) = default;

		vector_type& operator =(const vector_type& src) = default;
		vector_type& operator =(vector_type&& src) = default;

		template<typename T = vector_type>
		void deepCopy(const std::enable_if_t<std::is_copy_constructible<ObjType>::value, T>& src);

	public:
		using std::vector<std::shared_ptr<ObjType>>::erase;
		void erase(const object_type* obj);

	public:
		vector_type sorted() const;

		pointer_type selectFirst(std::function<bool(const pointer_type&)> pred) const;
		pointer_vec_type select(std::function<bool(const pointer_type&)> pred) const;

		auto find(const pointer_type& obj) const;
		auto find(const object_type* obj) const;
		auto find(std::function<bool(const pointer_type&)> pred) const;
		auto find(std::function<bool(const object_type*)> pred) const;

		auto indexOf(const pointer_type& obj) const;
		auto indexOf(const object_type* obj) const;
		bool contains(const pointer_type& obj) const { return indexOf(obj) != -1; }
		bool contains(const object_type* obj) const { return indexOf(obj) != -1; }

	public:
		void serialize(QString elemName, SerializationContext& ctx, std::function<bool(const pointer_type&)> predicate = nullptr) const;
		void deserialize(QString elemName, DeserializationContext& ctx, std::function<pointer_type(const SettingsContainer&)> objCreator);
	};
}

#include "ObjectVector.impl.h"

#endif
