/******************************************************************************
 * File: Property.h
 * Date: 12.1.2018
 *****************************************************************************/

#ifndef PROPERTY_H
#define PROPERTY_H

#include <memory>

#include "PropertyBase.h"
#include "PropertyConstraint.h"

namespace grndr
{	
	template<typename ValType>
	class Property : public PropertyBase
	{
	public:
		using property_type = Property<ValType>;
		using value_type = ValType;

	public:
		Property(PropertyID id, QString name, value_type defValue = value_type{}, Flags flags = Flag::None);
		Property(const property_type& src) = default;
		Property(property_type&& src) = default;
		Property(const value_type& val);
		Property(value_type&& val);

		Property& operator =(const property_type& src) = default;
		Property& operator =(property_type&& src) = default;
		Property& operator =(const value_type& val);
		Property& operator =(value_type&& val);

		operator const value_type&() const { return _value; }

		bool operator ==(const property_type& other) const;
		bool operator ==(const value_type& val) const;

	public:
		const value_type& getValue() const { return _value; }
		void setValue(const value_type& val);
		void setValue(value_type&& val);

	public:
		template<template<typename> class ConstraintType, typename... Args>
		void createConstraint(Args... args);
		void removeConstraint(const PropertyConstraints::value_type& constraint);

		const PropertyConstraints& constraints() { return _constraints; }	

	public:
		virtual void copyValue(const PropertyBase* property) override;

		virtual QString toString() const override;
		virtual void fromString(const QString& data) override;

		virtual void serialize(SerializationContext& ctx) const override;
		virtual void deserialize(DeserializationContext& ctx) override;	

	private:
		template<typename V, typename Setter>
		void _setValue(V val, Setter setter);

		void applyConstraints();

	protected:
		value_type _value{};

		PropertyConstraints _constraints;
	};
}

#include "Property.impl.h"

#endif
