/******************************************************************************
 * File: PropertyBase.h
 * Date: 13.1.2018
 *****************************************************************************/

#ifndef PROPERTYBASE_H
#define PROPERTYBASE_H

#include "PropertyID.h"
#include "common/serialization/SerializationContext.h"
#include "common/serialization/DeserializationContext.h"

namespace grndr
{
	class PropertyBase : public QObject
	{
		Q_OBJECT

	public:
		static const char* Serialization_Value_ID;
		static const char* Serialization_Value_Name;
		static const char* Serialization_Value_Value;

		enum class Flag : unsigned int
		{
			None = 0x0000,
			ReadOnly = 0x0001,
			Hidden = 0x0002,
		};

		Q_DECLARE_FLAGS(Flags, Flag)		

	public:
		PropertyBase(PropertyID id, QString name, Flags flags = Flag::None);
		PropertyBase(const PropertyBase& src) = default;
		PropertyBase(PropertyBase&& src) = default;

		PropertyBase& operator =(const PropertyBase& src) = default;
		PropertyBase& operator =(PropertyBase&& src) = default;

		bool operator <(const PropertyBase& prop) { return _name < prop._name; }

	public:
		virtual void initProperty() { }

	public:
		PropertyID getID() const { return _id; }
		void setID(PropertyID id) { _id = id; }

		QString getName() const { return _name; }
		void setName(QString name) { _name = name; }
		QString getDescription() const { return _description; }
		void setDescription(QString desc) { _description = desc; }

		bool hasFlag(Flag flag) const { return _flags.testFlag(flag); }
		void setFlag(Flag flag) { _flags |= flag; }
		Flags getFlags() const { return _flags; }
		void setFlags(Flags flags) { _flags = flags; }

	public:
		virtual QWidget* createEditor(QWidget* parent = nullptr) { Q_UNUSED(parent); return nullptr; }

	public:
		virtual void copyValue(const PropertyBase* property) = 0;

		virtual QString toString() const = 0;
		virtual void fromString(const QString& data) = 0;

		virtual void serialize(SerializationContext& ctx) const;
		virtual void deserialize(DeserializationContext& ctx);

	signals:
		void valueChanged() const;

	protected:
		PropertyID _id{PropertyID::None};
		QString _name{""};
		QString _description{""};

		Flags _flags{Flag::None};
	};	
}

Q_DECLARE_OPERATORS_FOR_FLAGS(grndr::PropertyBase::Flags)

#endif
