/******************************************************************************
 * File: DeserializationContext.h
 * Date: 28.2.2018
 *****************************************************************************/

#ifndef DESERIALIZATIONCONTEXT_H
#define DESERIALIZATIONCONTEXT_H

#include <stack>

#include "SerializationBase.h"
#include "SettingsContainer.h"

namespace grndr
{
	class Block;
	class Port;
	class ImageReference;

	class DeserializationContext : public SerializationBase
	{
	public:
		DeserializationContext(const SettingsContainer& settings, Mode mode);

	public:
		SettingsContainer& settings(bool rootSettings = false) { if (!rootSettings && !_settingsStack.empty()) return *_settingsStack.top(); else return _settings; }
		const SettingsContainer& settings(bool rootSettings = false) const { if (!rootSettings && !_settingsStack.empty()) return *_settingsStack.top(); else return _settings; }

	public:
		bool beginGroup(QString name);
		void beginGroup(SettingsContainer* settings) { _settingsStack.push(settings); }
		void endGroup() { _settingsStack.pop(); }

	public:
		void addBlock(int index, Block* block) { _blocks[index] = block; }
		Block* getBlock(int index) const { return getObject(index, _blocks); }

		void addPort(int index, Port* port) { _ports[index] = port; }
		Port* getPort(int index) const { return getObject(index, _ports); }

		void addImageReference(int index, ImageReference* imageRef) { _imageReferences[index] = imageRef; }
		ImageReference* getImageReference(int index) const { return getObject(index, _imageReferences); }

		void addImageSearchPaths(const QStringList& searchPaths);
		const QStringList& imageSearchPaths() const { return _imageSearchPaths; }

	private:
		template<typename ObjType>
		ObjType* getObject(int index, const std::map<int, ObjType*>& vec) const;

	private:
		SettingsContainer _settings;
		std::stack<SettingsContainer*> _settingsStack;

		std::map<int, Block*> _blocks;
		std::map<int, Port*> _ports;
		std::map<int, ImageReference*> _imageReferences;

		QStringList _imageSearchPaths;
	};
}

#include "DeserializationContext.impl.h"

#endif
