/******************************************************************************
 * File: JsonSettingsCodec.h
 * Date: 27.2.2018
 *****************************************************************************/

#ifndef JSONSETTINGSCODEC_H
#define JSONSETTINGSCODEC_H

#include <stack>

#include "SettingsCodec.h"

namespace grndr
{
	class JsonSettingsCodec : public SettingsCodec
	{
	public:
		using SettingsCodec::SettingsCodec;

	public:
		virtual void encodeSettings(const SettingsContainer& settings, QTextStream& stream) override;
		virtual void decodeSettings(SettingsContainer& settings, QTextStream& stream) override;

	private:
		void encode(const SettingsContainer& settings, QTextStream& stream, int level, bool lastEntry) const;
		void decode(SettingsContainer& settings, QTextStream& stream) const;

	private:
		void writeJsonValues(const SettingsContainer& settings, QTextStream& stream, int level) const;
		void writeJsonValue(QString valueName, const QVariant& value, QTextStream& stream, int level, bool lastEntry) const;
		void writeJsonLine(const QString& text, QTextStream& stream, int level) const;

	private:
		enum class TokenType
		{
			Undefined,
			String,
			ObjectBegin,
			ObjectEnd,
			ArrayBegin,
			ArrayEnd,
			Colon,
			Comma,
			Value,
		};

		struct Token
		{
			TokenType type{TokenType::Undefined};
			QString value{""};
		};

		struct ParseContext
		{
			std::stack<SettingsContainer*> settingsStack;
			unsigned int currentToken{0};
			QString identifier{""};

			void pushChildSettings(SettingsContainer& settings, bool isArray = false);
			void popChildSettings() { settingsStack.pop(); }

			void setSettingsValue(const QVariant& value);
		};

		std::vector<Token> readJsonDocument(QTextStream& stream) const;

		void parseJsonDocument(SettingsContainer& settings, const std::vector<Token>& tokens) const;
		void parseJsonObject(SettingsContainer& settings, const std::vector<Token>& tokens, ParseContext& parseContext) const;
		void parseJsonObjectItems(SettingsContainer& settings, const std::vector<Token>& tokens, ParseContext& parseContext) const;
		void parseJsonObjectItem(SettingsContainer& settings, const std::vector<Token>& tokens, ParseContext& parseContext) const;
		void parseJsonArray(SettingsContainer& settings, const std::vector<Token>& tokens, ParseContext& parseContext) const;

		bool rdpAccept(TokenType token, const std::vector<Token>& tokens, ParseContext& parseContext, std::function<void(QString)> attributor = nullptr, bool peek = false) const;
		void rdpExpect(TokenType token, const std::vector<Token>& tokens, ParseContext& parseContext, std::function<void(QString)> attributor = nullptr) const;

	private:
		static void removeEnclosingQuotes(QString& text);
	};
}

#endif
