/******************************************************************************
 * File: SettingsContainer.cpp
 * Date: 26.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "SettingsContainer.h"

SettingsContainer::SettingsContainer(QString name, bool isArray) :
	_name{name}, _isArray{isArray}
{

}

bool SettingsContainer::operator ==(const SettingsContainer& container)
{
	// Compare basic settings
	if (_name != container._name || _isArray != container._isArray)
		return false;

	// Compare all stored values
	if (_values != container._values)
		return false;

	// Compare children
	auto compare = [](const auto& child1, const auto& child2) { return *child1 == *child2; };

	if (_isArray)	// Arrays need to have the same order to be considered equal
	{
		if (!std::equal(container._childContainers.cbegin(), container._childContainers.cend(), _childContainers.cbegin(), _childContainers.cend(), compare))
			return false;
	}
	else
	{
		if (!std::is_permutation(container._childContainers.cbegin(), container._childContainers.cend(), _childContainers.cbegin(), _childContainers.cend(), compare))
			return false;
	}

	return true;
}

SettingsContainer& SettingsContainer::operator <<(const SettingsContainer& child)
{
	auto container = std::make_shared<SettingsContainer>(child);
	_childContainers.push_back(container);
	return *this;
}

SettingsContainer* SettingsContainer::createChild(QString name, bool isArray)
{
	auto container = std::make_shared<SettingsContainer>(name, isArray);
	_childContainers.push_back(container);
	return container.get();
}

SettingsContainer& SettingsContainer::operator <<(SettingsContainer&& child)
{
	auto container = std::make_shared<SettingsContainer>(std::move(child));
	_childContainers.push_back(container);
	return *this;
}

void SettingsContainer::removeChildren(QString name)
{
	if (name.isEmpty())
		throw std::invalid_argument{_EXCPT("name may not be empty")};

	std::remove_if(_childContainers.begin(), _childContainers.end(), [name](const auto& child) { return child->_name.compare(name, Qt::CaseInsensitive) == 0; });
}

QVariant& SettingsContainer::value(QString name, QVariant defaultValue)
{
	if (!contains(name))
		_values[name] = defaultValue;

	return _values[name];
}
