/******************************************************************************
 * File: SettingsContainer.h
 * Date: 26.2.2018
 *****************************************************************************/

#ifndef SETTINGSCONTAINER_H
#define SETTINGSCONTAINER_H

#include <QString>
#include <QVariant>
#include <memory>

namespace grndr
{
	class SettingsContainer
	{
	public:
		SettingsContainer(QString name = "", bool isArray = false);
		SettingsContainer(const SettingsContainer& container) = default;
		SettingsContainer(SettingsContainer&& container) = default;

		SettingsContainer& operator =(const SettingsContainer& container) = default;
		SettingsContainer& operator =(SettingsContainer&& container) = default;	

	public:
		QString getName() const { return _name; }
		bool isArray() const { return _isArray; }

		bool isEmpty() const { return _childContainers.empty() && _values.isEmpty(); }

		void clear() { clearChildren(); clearValues(); }

		bool operator ==(const SettingsContainer& container);
		bool operator !=(const SettingsContainer& container) { return !(*this == container); }

	public:
		SettingsContainer& operator <<(const SettingsContainer& child);
		SettingsContainer& operator <<(SettingsContainer&& child);

		SettingsContainer* createChild(QString name, bool isArray = false);

		SettingsContainer* child(QString name) { return _child<SettingsContainer*>(name); }
		const SettingsContainer* child(QString name) const { return _child<const SettingsContainer*>(name); }
		const std::vector<SettingsContainer*> children() { return children(""); }
		const std::vector<const SettingsContainer*> children() const { return children(""); }
		const std::vector<SettingsContainer*> children(QString name) { return _children<SettingsContainer*>(name); }
		const std::vector<const SettingsContainer*> children(QString name) const { return _children<const SettingsContainer*>(name); }

		void removeChildren(QString name);
		void clearChildren() { _childContainers.clear(); }

	public:
		QVariant& value(QString name, QVariant defaultValue = QVariant{});
		QVariant value(QString name, QVariant defaultValue = QVariant{}) const { return _values.value(name, defaultValue); }

		QVariant& operator()(QString name, QVariant defaultValue = QVariant{}) { return value(name, defaultValue); }
		QVariant operator()(QString name, QVariant defaultValue = QVariant{}) const { return value(name, defaultValue); }

		const QVariantMap& values() const { return _values; }

		QStringList keys() const { return _values.keys(); }
		bool contains(QString name) const { return _values.find(name) != _values.cend(); }

		void removeValue(QString name) { _values.remove(name); }
		void clearValues() { _values.clear(); }

		auto begin() { return _values.begin(); }
		auto cbegin() const { return _values.cbegin(); }
		auto end() { return _values.end(); }
		auto cend() const { return _values.cend(); }

	private:
		template<typename DataType>
		DataType _child(QString name) const;
		template<typename DataType>
		const std::vector<DataType> _children(QString name) const;

	private:
		QString _name{""};
		bool _isArray{false};

		std::vector<std::shared_ptr<SettingsContainer>> _childContainers;
		QVariantMap _values;
	};
}

#include "SettingsContainer.impl.h"

#endif
