/******************************************************************************
 * File: EngineController.cpp
 * Date: 21.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "EngineController.h"
#include "core/GrinderApplication.h"
#include "project/Label.h"
#include "engine/Engine.h"
#include "engine/EngineExceptions.h"
#include "ui/LongOperationIndicator.h"
#include "util/UIUtils.h"

EngineController::EngineController(Engine* engine) : GenericController("Engine"),
	_engine{engine}
{
	if (!engine)
		throw std::invalid_argument{_EXCPT("engine may not be null")};
}

void EngineController::executeLabel(Label* label, const ImageReference* activeImageReference, Engine::ExecutionMode mode) const
{
	// If no label was provided, use the currently active one
	if (!label)
		label = grinder()->projectController().activeLabel();

	if (label)
	{		
		if (!activeImageReference)
			activeImageReference = grinder()->projectController().activeImageReference();

		callControllerFunction(QString{"Executing label '%1'"}.arg(label->getName()), [this](Label* label, const ImageReference* activeImageReference, Engine::ExecutionMode mode) {
			LongOperationIndicator longOperation{QString{"Executing label '%1'..."}.arg(label->getName())};

			_engine->executeLabel(label, activeImageReference, mode);
			return true;
		}, label, activeImageReference, mode);
	}
}

cv::Mat EngineController::executeLabelEx(Label* label, const Port* port, const ImageReference* activeImageReference, Engine::ExecutionMode mode) const
{
	// If no label was provided, use the currently active one
	if (!label)
		label = grinder()->projectController().activeLabel();

	if (label && port)
	{		
		if (!activeImageReference)
			activeImageReference = grinder()->projectController().activeImageReference();

		return callControllerFunction(QString{"Executing label '%1'"}.arg(label->getName()), [this](Label* label, const Port* port, const ImageReference* activeImageReference, Engine::ExecutionMode mode) {
			LongOperationIndicator longOperation{QString{"Executing label '%1'..."}.arg(label->getName())};

			// If the port is an in-port, get data from the connected out-port
			if (port->isIn())
			{
				auto cons = port->getConnections(Port::Direction::In);

				if (cons.size() == 1)
					port = cons.front()->sourcePort();
				else if (cons.size() > 1)
					throw EngineException{_engine, _EXCPT(QString{"Port '%1' has multiple inputs"}.arg(port->getFormattedName()))};
			}

			return _engine->executeLabelEx(label, port, activeImageReference, mode);
		}, label, port, activeImageReference, mode);
	}

	return cv::Mat{};
}
