/******************************************************************************
 * File: GenericController.impl.h
 * Date: 08.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "GenericController.h"

#include <opencv2/core.hpp>

template<typename Functor, typename... Args>
auto GenericController::callControllerFunction(QString action, Functor fnc, Args... args) const -> decltype(fnc(args...))
{
	if (canCallControllerFunction())
	{
		try {
			auto result = fnc(std::forward<Args>(args)...);
			controllerFunctionCalled();
			return result;
		} catch (std::exception& e) {
			showControllerError(action, e.what());
		}
	}

	using retType = decltype(fnc(args...));
	return retType{};
}
