/******************************************************************************
 * File: ImageEditorController.h
 * Date: 15.3.2018
 *****************************************************************************/

#ifndef IMAGEEDITORCONTROLLER_H
#define IMAGEEDITORCONTROLLER_H

#include "GenericController.h"
#include "ui/image/ImageEditorComponent.h"
#include "ui/image/ImageEditorScene.h"

namespace grndr
{
	class ImageEditor;
	class ImageEditorView;
	class ImageBuild;
	class Layer;
	class LayersListWidget;

	class ImageEditorController : public GenericController, public ImageEditorComponent
	{
		Q_OBJECT

	public:
		ImageEditorController(ImageEditor* imageEditor);

	public:
		void assignUiComponents(ImageEditorView* view, LayersListWidget* layersList);

	public:
		void showImageBuild(ImageBuild* build);
		void hideImageBuild() { showImageBuild(nullptr); }
		ImageBuild* activeImageBuild() { return _activeImageBuild; }
		const ImageBuild* activeImageBuild() const { return _activeImageBuild; }

		ImageEditorScene* activeScene() { return _activeScene.get(); }
		const ImageEditorScene* activeScene() const { return _activeScene.get(); }

		void switchLayer(Layer* layer);
		Layer* activeLayer() { return _activeLayer; }
		const Layer* activeLayer() const { return _activeLayer; }		

	public:
		void copyImageBuild(const ImageBuild* imageBuild = nullptr) const;
		void pasteImageBuild(ImageBuild* imageBuild = nullptr);		
		void duplicateImageBuild(const ImageBuild* sourceImageBuild = nullptr, ImageBuild* targetImageBuild = nullptr, bool switchToTargetImageBuild = true);
		void duplicateImageBuild(const ImageBuild* sourceImageBuild, std::vector<ImageReference*> targetImageReferences);

		std::shared_ptr<Layer> createLayer(QString name = "") const;
		void copyLayer(const Layer* layer) const;
		void pasteLayer();
		void cutLayer(const Layer* layer);
		void removeLayer(const Layer* layer, bool enforceRemoval = false);
		void removeAllLayers(bool enforceRemoval = false);

		std::shared_ptr<DraftItem> createDraftItem(DraftItemType type, Layer* layer = nullptr, bool autoCreateLayer = true);
		void removeDraftItem(const DraftItem* item) const;

		void setLayerVisibility(Layer* layer, bool visible) const;
		void setLayerAlpha(Layer* layer, float alpha) const;
		void moveLayer(Layer* layer, bool up) const;
		void renameLayer(Layer* layer, QString newName) const;
		void renameImageBuildItem(ImageBuildItem* item, QString newName) const;

		void setNodesPrimaryColor(QColor color, const std::vector<DraftItemNode*>& nodes) const;

		void convertSelectedNodesToPixels() const;
		void convertLayerPixelsToItems(Layer* layer);
		void paintPixels(Layer* layer, QColor color, const std::vector<QPoint>& pixels, bool autoCreateLayer = true);

		void copySelectedNodes() const;
		void pasteSelectedNodes();
		void cutSelectedNodes() const;
		void removeSelectedNodes() const;

		void assignImageTags(ImageTagsAllotmentProperty* imageTagsAllotment, const QStringList& imageTagNames, bool clearFirst = false) const;
		void assignImageTags(ImageTagsAllotmentProperty* imageTagsAllotment, const std::set<ImageTag*>& imageTags, bool clearFirst = false) const;

	signals:
		void imageBuildSwitching(ImageBuild*);
		void imageBuildSwitched(ImageBuild*);
		void layerSwitching(Layer*);
		void layerSwitched(Layer*);

	protected:
		virtual bool canCallControllerFunction() const override { return _activeImageBuild != nullptr; }
		virtual void controllerFunctionCalled() const override;

	private:
		void duplicateImageBuild(const ImageBuild* sourceImageBuild, std::vector<ImageBuild*> targetImageBuilds);
		void clearImageBuild(ImageBuild* imageBuild);

		Layer* createDefaultLayer();
		void validateLayerName(QString name, const Layer* layer = nullptr) const;

		void connectLayerSignals(ImageBuild* imageBuild, bool connectSignals = true) const;
		void connectLayerSignals(Layer* layer, bool connectSignals = true) const;

	private slots:
		void refreshEditorImage();

		void layerCreated(const std::shared_ptr<Layer>& layer) const;
		void layerRemoved(const std::shared_ptr<Layer>& layer) const;
		void layerMoved(const std::shared_ptr<Layer>& layer, int indexOld, int indexNew) const;
		void layerVisibilityChanged() const;
		void layerAlphaChanged(float alpha) const;

		void draftItemCreated(const std::shared_ptr<DraftItem>& item) const;
		void draftItemRemoved(const std::shared_ptr<DraftItem>& item) const;

	private:
		ImageEditorView* _view{nullptr};
		LayersListWidget* _layersList{nullptr};		

		ImageBuild* _activeImageBuild{nullptr};
		std::unique_ptr<ImageEditorScene> _activeScene;
		Layer* _activeLayer{nullptr};
	};
}

#endif
