/******************************************************************************
 * File: PipelineController.h
 * Date: 18.1.2018
 *****************************************************************************/

#ifndef PIPELINECONTROLLER_H
#define PIPELINECONTROLLER_H

#include <memory>

#include "GenericController.h"
#include "pipeline/BlockType.h"
#include "pipeline/BlockHierarchy.h"
#include "ui/graph/GraphScene.h"

namespace grndr
{
	class Pipeline;
	class PipelineItem;
	class Block;
	class Connection;
	class Port;
	class GraphView;
	class GraphScene;

	class PipelineController : public GenericController
	{
		Q_OBJECT

	public:
		PipelineController();

	public:
		void assignUiComponents(GraphView* view);

	public:
		void showPipeline(Pipeline* pipeline);
		void hidePipeline() { showPipeline(nullptr); }
		Pipeline* activePipeline() { return _activePipeline; }
		const Pipeline* activePipeline() const { return _activePipeline; }

		GraphScene* activeScene() { return _activeScene.get(); }
		const GraphScene* activeScene() const { return _activeScene.get(); }

	public:
		std::shared_ptr<Block> createBlock(BlockType type, QString name = "") const;
		void removeBlock(const Block* block) const;

		void disconnectPortFromAll(Port* port) const;

		void validateConnection(const Port* sourcePort, const Port* destPort) const;
		std::shared_ptr<Connection> createConnection(Port* sourcePort, Port* destPort) const;
		void removeConnection(const Connection* connection) const;

		void renameBlock(Block* block, QString newName) const;
		void renamePipelineItem(PipelineItem* item, QString newName) const;

		void removeSelectedNodes() const;

	public:
		BlockHierarchy createBlockHierarchy() const;

	signals:
		void pipelineSwitching(Pipeline*);
		void pipelineSwitched(Pipeline*);

	protected:
		virtual bool canCallControllerFunction() const override { return _activePipeline != nullptr; }
		virtual void controllerFunctionCalled() const override;

	private:
		void validateBlockName(QString name, const Block* block = nullptr) const;

	private slots:
		void blockCreated(const std::shared_ptr<Block>& block) const;
		void blockRemoved(const std::shared_ptr<Block>& block) const;
		void connectionCreated(const std::shared_ptr<Connection>& connection) const;
		void connectionRemoved(const std::shared_ptr<Connection>& connection) const;

	private:
		GraphView* _view{nullptr};

		Pipeline* _activePipeline{nullptr};
		std::unique_ptr<GraphScene> _activeScene;
	};
}

#endif
