/******************************************************************************
 * File: ClipboardManager.h
 * Date: 04.4.2018
 *****************************************************************************/

#ifndef CLIPBOARDMANAGER_H
#define CLIPBOARDMANAGER_H

#include <QObject>

#include "common/serialization/SettingsContainer.h"

namespace grndr
{
	class ClipboardManager : public QObject
	{
		Q_OBJECT

	public:
		using ClipboardData = std::pair<QString, std::vector<SettingsContainer>>;

	public:
		std::vector<SettingsContainer> getData(QString type) const;
		void setData(QString type, const std::vector<SettingsContainer>& data);
		void setData(QString type, std::vector<SettingsContainer>&& data);
		bool hasData(QString type) const;

	public:
		template<typename ObjType>
		void serialize(QString type, const std::vector<const ObjType*>& objects);
		template<typename ObjType>
		std::vector<ObjType*> deserialize(QString type, std::function<ObjType*(const SettingsContainer&)> objCreator) const;

	public:
		ClipboardData saveClipboard() const { return _data; }
		void restoreClipboard(const ClipboardData& data) { setData(data.first, data.second); }

	signals:
		void dataChanged();

	private:
		ClipboardData _data;
	};
}

#include "ClipboardManager.impl.h"

#endif
