/******************************************************************************
 * File: GrinderApplication.cpp
 * Date: 11.1.2018
 *****************************************************************************/

#include "Grinder.h"
#include "GrinderApplication.h"
#include "Version.h"
#include "pipeline/BlockCatalog.h"
#include "image/DraftItemCatalog.h"
#include "ui/StyleSheet.h"
#include "res/Resources.h"

GrinderApplication* GrinderApplication::s_appInstance = nullptr;

GrinderApplication::OperatingSystem GrinderApplication::operatingSystem()
{
#if defined(Q_OS_WIN)
	return OperatingSystem::Windows;
#elif defined(Q_OS_LINUX)
	return OperatingSystem::Linux;
#elif defined(Q_OS_OSX)
	return OperatingSystem::MacOSX;
#endif

	return OperatingSystem::Unknown;
}

QFont GrinderApplication::boldFont(QWidget* widget)
{
	QFont font;

	if (widget)
		font = widget->font();

	font.setBold(true);
	return font;
}

GrinderApplication::GrinderApplication(int& argc, char** argv, int flags) : QApplication(argc, argv, flags),
	_settings{GRNDR_INFO_COMPANY, GRNDR_INFO_TITLE}, _projectController{&_project}, _engineController{&_engine}, _imageEditorManager{&_project, &_projectController, &_pipelineManager}
{
	s_appInstance = this;

	// Basic app information
	setApplicationName(GRNDR_INFO_TITLE);
	setApplicationVersion(GetVersionString(true));
	setOrganizationName(GRNDR_INFO_COMPANY);
	setOrganizationDomain(GRNDR_INFO_WEBSITE);

	setWindowIcon(QIcon{FILE_ICON_MAIN});

	// Apply font quality fixes for some OSes
#if defined(Q_OS_LINUX)
	QFont fnt = font();
	fnt.setStyleStrategy(QFont::StyleStrategy(QFont::PreferAntialias|QFont::PreferQuality));
	setFont(fnt);
#else
	fixFontQuality("QMessageBox");
	fixFontQuality("QMenuBar");
	fixFontQuality("QMenu");
	fixFontQuality("QStatusBar");
#endif	

	// Apply global stylesheet
	setStyleSheet(StyleSheet::loadStyleSheet(FILE_STYLESHEET_GLOBAL));

	// Register standard types in all catalogs
	BlockCatalog::registerStandardBlocks();
	DraftItemCatalog::registerStandardDraftItems();
}

GrinderApplication::~GrinderApplication()
{
	s_appInstance = nullptr;
}

int GrinderApplication::run()
{
	_mainWindow = std::make_unique<GrinderWindow>();	
	_mainWindow->show();

	return exec();
}

void GrinderApplication::fixFontQuality(const char* ctrlClass)
{
	QFont fnt = font(ctrlClass);

	fnt.setStyleStrategy(QFont::StyleStrategy(QFont::PreferAntialias|QFont::PreferQuality));
	setFont(fnt, ctrlClass);
}
