/******************************************************************************
 * File: GrinderApplication.h
 * Date: 11.1.2018
 *****************************************************************************/

#ifndef GRINDERAPPLICATION_H
#define GRINDERAPPLICATION_H

#include <QApplication>

#include "GrinderSettings.h"
#include "ClipboardManager.h"
#include "pipeline/PipelineManager.h"
#include "project/Project.h"
#include "engine/Engine.h"
#include "controller/PipelineController.h"
#include "controller/ProjectController.h"
#include "controller/EngineController.h"
#include "ui/mainwnd/GrinderWindow.h"
#include "ui/image/ImageEditorManager.h"

namespace grndr
{
	class PipelineManager;

	class GrinderApplication : public QApplication
	{
		static GrinderApplication* s_appInstance;

	public:
		enum class OperatingSystem
		{
			Unknown,
			Windows,
			Linux,
			MacOSX,
		};

	public:
		static GrinderApplication* instance() { Q_ASSERT(s_appInstance != nullptr); return s_appInstance; }

	public:
		static OperatingSystem operatingSystem();
		static QFont boldFont(QWidget* widget = nullptr);

	public:
		GrinderApplication(int &argc, char **argv, int flags = ApplicationFlags);
		virtual ~GrinderApplication();

	public:
		int run();

	public:		
		GrinderSettings& settings() { return _settings; }
		const GrinderSettings& settings() const { return _settings; }
		ClipboardManager& clipboardManager() { return _clipboardManager; }
		const ClipboardManager& clipboardManager() const { return _clipboardManager; }

		PipelineManager& pipelineManager() { return _pipelineManager; }
		const PipelineManager& pipelineManager() const { return _pipelineManager; }
		PipelineController& pipelineController() { return _pipelineController; }
		const PipelineController& pipelineController() const { return _pipelineController; }

		Project& project() { return _project; }
		const Project& project() const { return _project; }
		ProjectController& projectController() { return _projectController; }
		const ProjectController& projectController() const { return _projectController; }

		Engine& engine() { return _engine; }
		const Engine& engine() const { return _engine; }
		EngineController& engineController() { return _engineController; }
		const EngineController& engineController() const { return _engineController; }

		GrinderWindow* mainWindow() { return _mainWindow.get(); }
		const GrinderWindow* mainWindow() const { return _mainWindow.get(); }

		ImageEditorManager& imageEditorManager() { return _imageEditorManager; }
		const ImageEditorManager& imageEditorManager() const { return _imageEditorManager; }

	private:
		void fixFontQuality(const char* ctrlClass);

	private:
		GrinderSettings _settings;
		ClipboardManager _clipboardManager;

		PipelineManager _pipelineManager;
		PipelineController _pipelineController;

		Project _project;
		ProjectController _projectController;

		Engine _engine;
		EngineController _engineController;

		std::unique_ptr<GrinderWindow> _mainWindow;

		ImageEditorManager _imageEditorManager;
	};

	inline GrinderApplication* grinder() { return GrinderApplication::instance(); }
}

#endif
