/******************************************************************************
 * File: ColorConv.cpp
 * Date: 12.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ColorConv.h"

void ColorConv::convertColorSpace(cv::Mat& matrix, ColorSpace colorSpaceFrom, ColorSpace colorSpaceTo)
{
	// Map the given color spaces to the proper OpenCV constant
	static const std::map<std::pair<ColorSpace, ColorSpace>, int> colorSpaceConversions = {
		{std::make_pair(ColorSpace::RGB, ColorSpace::HSV), cv::COLOR_BGR2HSV},
		{std::make_pair(ColorSpace::RGB, ColorSpace::HLS), cv::COLOR_BGR2HLS},
		{std::make_pair(ColorSpace::RGB, ColorSpace::CIELab), cv::COLOR_BGR2Lab},
		{std::make_pair(ColorSpace::RGB, ColorSpace::CIELuv), cv::COLOR_BGR2Luv},
		{std::make_pair(ColorSpace::RGB, ColorSpace::YUV), cv::COLOR_BGR2YUV},
		{std::make_pair(ColorSpace::HSV, ColorSpace::RGB), cv::COLOR_HSV2BGR},
		{std::make_pair(ColorSpace::HLS, ColorSpace::RGB), cv::COLOR_HLS2BGR},
		{std::make_pair(ColorSpace::CIELab, ColorSpace::RGB), cv::COLOR_Lab2BGR},
		{std::make_pair(ColorSpace::CIELuv, ColorSpace::RGB), cv::COLOR_Luv2BGR},
		{std::make_pair(ColorSpace::YUV, ColorSpace::RGB), cv::COLOR_YUV2BGR},
	};

	if (colorSpaceFrom == colorSpaceTo)
		return;

	auto convertColor = [&matrix](auto spaceFrom, auto spaceTo, const auto& conversions) {
		int colorCode = -1;

		for (const auto& conv : conversions)
		{
			if (conv.first.first == spaceFrom && conv.first.second == spaceTo)
			{
				colorCode = conv.second;
				break;
			}
		}

		if (colorCode != -1)
			cv::cvtColor(matrix, matrix, colorCode);
	};

	// Conversion always starts from RGB
	if (colorSpaceFrom != ColorSpace::RGB)
		convertColor(colorSpaceFrom, ColorSpace::RGB, colorSpaceConversions);

	if (colorSpaceTo != ColorSpace::RGB)
		convertColor(ColorSpace::RGB, colorSpaceTo, colorSpaceConversions);
}
