/******************************************************************************
 * File: MatrixCompat.cpp
 * Date: 16.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "MatrixCompat.h"
#include "engine/data/DataBlob.h"

bool MatrixCompat::checkCompatibility(const std::vector<cv::Mat>& matrices, CompatScope scope)
{
	// Perform a pairwise check of all matrix attributes
	for (unsigned int i = 0; i < matrices.size(); ++i)
	{
		for (unsigned int j = i + 1; j < matrices.size(); ++j)
		{
			auto& matrix1 = matrices[i];
			auto& matrix2 = matrices[j];

			if (scope.testFlag(CompatAttribute::Dimensions))
			{
				if (matrix1.rows != matrix2.rows || matrix1.cols != matrix2.cols)
					return false;
			}

			if (scope.testFlag(CompatAttribute::Channels))
			{
				if (matrix1.channels() != matrix2.channels())
					return false;
			}

			if (scope.testFlag(CompatAttribute::Type))
			{
				if (matrix1.type() != matrix2.type())
					return false;
			}
		}
	}

	// All matrices match the specified attributes
	return true;
}

bool MatrixCompat::checkCompatibility(std::initializer_list<cv::Mat> matrices, CompatScope scope)
{
	return checkCompatibility(std::vector<cv::Mat>{matrices}, scope);
}

bool MatrixCompat::checkCompatibility(const std::vector<const DataBlob*>& dataBlobs, CompatScope scope)
{
	std::vector<cv::Mat> matrices;

	for (auto dataBlob : dataBlobs)
		matrices.push_back(dataBlob->getMatrix());

	return checkCompatibility(matrices, scope);
}

bool MatrixCompat::checkCompatibility(std::initializer_list<const DataBlob*> dataBlobs, CompatScope scope)
{
	return checkCompatibility(std::vector<const DataBlob*>{dataBlobs}, scope);
}
