/******************************************************************************
 * File: MatrixCompat.h
 * Date: 16.6.2018
 *****************************************************************************/

#ifndef MATRIXCOMPAT_H
#define MATRIXCOMPAT_H

#include <QFlags>
#include <opencv2/core.hpp>

namespace grndr
{
	class DataBlob;

	class MatrixCompat final
	{
	public:
		enum class CompatAttribute : unsigned int
		{
			Dimensions = 0x0001,
			Channels = 0x0002,
			Type = 0x0004,

			All = 0xFFFF,
		};

		Q_DECLARE_FLAGS(CompatScope, CompatAttribute)

	public:
		static bool checkCompatibility(const std::vector<cv::Mat>& matrices, CompatScope scope);
		static bool checkCompatibility(std::initializer_list<cv::Mat> matrices, CompatScope scope);
		static bool checkCompatibility(const std::vector<const DataBlob*>& dataBlobs, CompatScope scope);
		static bool checkCompatibility(std::initializer_list<const DataBlob*> dataBlobs, CompatScope scope);

	private:
		MatrixCompat() { }
	};
}

Q_DECLARE_OPERATORS_FOR_FLAGS(grndr::MatrixCompat::CompatScope)

#endif
