/******************************************************************************
 * File: Pixel.cpp
 * Date: 19.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "Pixel.h"

QColor ColorPixel::getColor() const
{
	return QColor{_r, _g, _b};
}

void ColorPixel::setColor(const QColor& color)
{
	_r = color.red();
	_g = color.green();
	_b = color.blue();
}

bool ColorPixel::operator ==(const ColorPixel& other) const
{
	return _r == other._r && _g == other._g && _b == other._b;
}

QColor GrayscalePixel::getColor() const
{
	return QColor{_p, _p, _p};
}

void GrayscalePixel::setColor(const QColor& color)
{
	_p = std::lround((color.red() + color.green() + color.blue()) / 3.0f);
}

PixelRef::PixelRef(ColorPixel* pixel) :
	_pixel{pixel}, _pixelGetter{&PixelRef::getPixel}, _pixelSetter{&PixelRef::setPixel}
{

}

PixelRef::PixelRef(GrayscalePixel* grayscalePixel) :
	_grayscalePixel{grayscalePixel}, _pixelGetter{&PixelRef::getGrayscalePixel}, _pixelSetter{&PixelRef::setGrayscalePixel}
{

}

PixelRef PixelAccessor::at(int x, int y)
{
	if (_matrix.channels() == 3)	// Color
		return PixelRef{&_matrix.at<ColorPixel>(y, x)};
	else
		return PixelRef{&_matrix.at<GrayscalePixel>(y, x)};
}

void PixelAccessor::forEach(std::function<void (PixelRef, QPoint)> callback)
{
	if (_matrix.channels() == 3)	// Color
	{
		_matrix.forEach<ColorPixel>([callback](ColorPixel& pixel, const int pos[]) {
			callback(PixelRef{&pixel}, QPoint{pos[1], pos[0]});
		});
	}
	else	// Grayscale
	{
		_matrix.forEach<GrayscalePixel>([callback](GrayscalePixel& pixel, const int pos[]) {
			callback(PixelRef{&pixel}, QPoint{pos[1], pos[0]});
		});
	}
}

void PixelAccessor::verifyMatrix()
{
	if (CV_MAT_DEPTH(_matrix.type()) == CV_8U || CV_MAT_DEPTH(_matrix.type()) == CV_8S)
	{
		if (_matrix.channels() != 1 && _matrix.channels() != 3)
			throw std::invalid_argument{_EXCPT("The matrix must have 1 or 3 channels")};
	}
	else
		throw std::invalid_argument{_EXCPT("The matrix must have a depth of 8 bit")};
}
