/******************************************************************************
 * File: ProcessorBase.h
 * Date: 21.2.2018
 *****************************************************************************/

#ifndef PROCESSORBASE_H
#define PROCESSORBASE_H

#include "engine/EngineExecutionContext.h"
#include "common/properties/PropertyID.h"

namespace grndr
{
	class Engine;

	class ProcessorBase
	{
	public:
		ProcessorBase(const Block* block);

	public:
		const Engine* engine() const { return _engine; }
		const Block* block() const { return _block; }

	public:
		virtual void execute(EngineExecutionContext& ctx);

	protected:
		DataDescriptor getPortDataDescriptor(const Port* port, unsigned int index = 0) const;
		DataDescriptor getBestDataDescriptor(const DataDescriptor& dataDesc, const DataDescriptors& targetDescriptors) const;

		DataBlob* portData(EngineExecutionContext& ctx, const Port* port, bool convert = true, bool required = true) const;
		template<typename PropType>
		PropType* portProperty(const Port* port, PropertyID propertyID, bool required = true) const;

	protected:
		void throwProcessorException(QString what) const;

	private:
		const Port* resolveDataPort(const Port* port, bool required) const;

	protected:
		const Engine* _engine{nullptr};
		const Block* _block{nullptr};
	};
}

#include "ProcessorBase.impl.h"

#endif
