/******************************************************************************
 * File: DataBlob.h
 * Date: 19.2.2018
 *****************************************************************************/

#ifndef DATABLOB_H
#define DATABLOB_H

#include "DataDescriptor.h"
#include "cv/CVTypes.h"

#include <opencv2/core.hpp>

namespace grndr
{
	class DataBlob
	{
	public:
		DataBlob(const DataDescriptor& dataDesc, ColorSpace colorSpace = ColorSpace::RGB);
		template<typename DataType>
		DataBlob(const DataDescriptor& dataDesc, const DataType& data, ColorSpace colorSpace = ColorSpace::RGB);
		template<typename DataType>
		DataBlob(const DataDescriptor& dataDesc, DataType&& data, ColorSpace colorSpace = ColorSpace::RGB);
		DataBlob(const DataBlob& blob) = default;
		DataBlob(DataBlob&& blob) = default;

		DataBlob& operator =(const DataBlob& blob) = default;
		DataBlob& operator =(DataBlob&& blob) = default;

		DataBlob& operator =(const cv::Mat& data) { set(data); return *this; }
		template<typename DataType>
		DataBlob& operator =(const std::vector<DataType>& data) { set(data); return *this; }
		template<typename DataType>
		DataBlob& operator =(const DataType& data) { set(data); return *this; }

		operator cv::Mat() const { return getMatrix(); }
		template<typename DataType>
		operator std::vector<DataType>() const { return getVector<DataType>(); }
		template<typename DataType>
		operator DataType() const { return getScalar<DataType>(); }

	public:
		void set(const cv::Mat& data);
		void set(cv::Mat&& data);
		template<typename DataType>
		void set(const std::vector<DataType>& data);
		template<typename DataType>
		void set(const DataType& data);

		cv::Mat getMatrix() const { return _data; }
		template<typename DataType>
		std::vector<DataType> getVector() const;
		template<typename DataType>
		DataType getScalar() const;

		void convertTo(const DataDescriptor& dataDesc, bool normalize = false, double minValue = 0.0, double maxValue = 1.0);
		template<typename TargetType>
		void convertTo(const DataDescriptor& dataDesc);

		void clear() { _data.release(); }

	public:
		const DataDescriptor& dataDescriptor() { return _dataDescriptor; }
		ColorSpace getColorSpace() const { return _colorSpace; }
		void setColorSpace(ColorSpace colorSpace);

		cv::Mat& data() { return _data; }
		const cv::Mat& data() const { return _data; }

		bool empty() const { return _data.empty(); }

	private:
		void updateDataDescriptor();

	private:
		DataDescriptor _dataDescriptor;
		ColorSpace _colorSpace{ColorSpace::RGB};

		cv::Mat _data;
	};
}

#include "DataBlob.impl.h"

#endif
