/******************************************************************************
 * File: AdaptiveThresholdProcessor.cpp
 * Date: 21.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "AdaptiveThresholdProcessor.h"
#include "pipeline/properties/AdaptiveThresholdMethodProperty.h"
#include "pipeline/properties/AdaptiveThresholdTypeProperty.h"

#include <opencv2/imgproc.hpp>

AdaptiveThresholdProcessor::AdaptiveThresholdProcessor(const Block* block) : Processor(block)
{

}

void AdaptiveThresholdProcessor::execute(EngineExecutionContext& ctx)
{
	Processor::execute(ctx);

	if (auto dataBlob = portData(ctx, _block->inPort()))
	{
		cv::Mat processedImage;
		cv::adaptiveThreshold(dataBlob->getMatrix(), processedImage, *_block->targetValue(), *_block->thresholdMethod(), *_block->thresholdType(), *_block->blockSize(), *_block->constant());

		ctx.setContextEntry(_block->outPort(), DataBlob{getPortDataDescriptor(_block->outPort()), std::move(processedImage), dataBlob->getColorSpace()});
	}
}
