/******************************************************************************
 * File: AlphaBlendingProcessor.cpp
 * Date: 07.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "AlphaBlendingProcessor.h"
#include "cv/MatrixCompat.h"

#include <opencv2/imgproc.hpp>

AlphaBlendingProcessor::AlphaBlendingProcessor(const Block* block) : Processor(block)
{

}

void AlphaBlendingProcessor::execute(EngineExecutionContext& ctx)
{
	Processor::execute(ctx);

	auto dataBlob1 = portData(ctx, _block->src1Port());
	auto dataBlob2 = portData(ctx, _block->src2Port());

	if (dataBlob1 && dataBlob2)
	{
		// Both images must be of the same dimension and type
		if (!MatrixCompat::checkCompatibility({dataBlob1, dataBlob2}, MatrixCompat::CompatAttribute::All))
			throwProcessorException("The input images must be of the same size and format");

		cv::Mat processedImage;
		double alpha = *_block->alpha();

		cv::addWeighted(dataBlob1->getMatrix(), alpha, dataBlob2->getMatrix(), 1.0 - alpha, 0.0, processedImage);
		ctx.setContextEntry(_block->outPort(), DataBlob{getPortDataDescriptor(_block->outPort()), std::move(processedImage), dataBlob1->getColorSpace()});
	}
}
