/******************************************************************************
 * File: BinaryThresholdProcessor.cpp
 * Date: 21.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "BinaryThresholdProcessor.h"
#include "pipeline/properties/BinaryThresholdTypeProperty.h"
#include "pipeline/properties/BinaryThresholdModeProperty.h"

#include <opencv2/imgproc.hpp>

BinaryThresholdProcessor::BinaryThresholdProcessor(const Block* block) : Processor(block)
{

}

void BinaryThresholdProcessor::execute(EngineExecutionContext& ctx)
{
	Processor::execute(ctx);

	if (auto dataBlob = portData(ctx, _block->inPort()))
	{
		// Otsu and Triangle thresholding require a single-channel image
		if (*_block->thresholdMode() == BinaryThresholdMode::Otsu || *_block->thresholdMode() == BinaryThresholdMode::Triangle)
		{
			if (!dataBlob->dataDescriptor().isGrayscale())
				throwProcessorException("Otsu/Triangle threshold modes require a grayscale image as input");
		}

		cv::Mat processedImage;
		cv::threshold(dataBlob->getMatrix(), processedImage, *_block->threshold(), *_block->targetValue(), *_block->thresholdType()|*_block->thresholdMode());

		ctx.setContextEntry(_block->outPort(), DataBlob{getPortDataDescriptor(_block->outPort()), std::move(processedImage), dataBlob->getColorSpace()});
	}
}
