/******************************************************************************
 * File: ConvertToColorProcessor.cpp
 * Date: 12.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ConvertToColorProcessor.h"

#include <opencv2/imgproc.hpp>

ConvertToColorProcessor::ConvertToColorProcessor(const Block* block) : Processor(block)
{

}

void ConvertToColorProcessor::execute(EngineExecutionContext& ctx)
{
	Processor::execute(ctx);

	if (auto dataBlob = portData(ctx, _block->inPort()))
	{
		cv::Mat processedImage;

		if (dataBlob->dataDescriptor().isGrayscale())	// If the input image is in grayscale, convert it to RGB first
			cv::cvtColor(dataBlob->getMatrix(), processedImage, cv::COLOR_GRAY2BGR);
		else
			processedImage = dataBlob->getMatrix();

		DataBlob dataBlobNew{getPortDataDescriptor(_block->outPort()), std::move(processedImage), dataBlob->getColorSpace()};
		dataBlobNew.setColorSpace(*_block->colorSpace());

		ctx.setContextEntry(_block->outPort(), std::move(dataBlobNew));
	}
}
