/******************************************************************************
 * File: MergeChannelsProcessor.cpp
 * Date: 14.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "MergeChannelsProcessor.h"
#include "cv/MatrixCompat.h"

#include <opencv2/core.hpp>

MergeChannelsProcessor::MergeChannelsProcessor(const Block* block) : Processor(block)
{

}

void MergeChannelsProcessor::execute(EngineExecutionContext& ctx)
{
	Processor::execute(ctx);

	auto channel1Blob = portData(ctx, _block->channel1Port());
	auto channel2Blob = portData(ctx, _block->channel2Port());
	auto channel3Blob = portData(ctx, _block->channel3Port());

	if (channel1Blob && channel2Blob && channel3Blob)
	{
		// 3 1-channel images are required for merging
		if (!channel1Blob->dataDescriptor().isGrayscale() || !channel2Blob->dataDescriptor().isGrayscale() || !channel3Blob->dataDescriptor().isGrayscale())
			throwProcessorException("It is only possible to merge three graycolor images");

		// All channel data must be of the same dimensions
		if (!MatrixCompat::checkCompatibility({channel1Blob, channel2Blob, channel3Blob}, MatrixCompat::CompatAttribute::Dimensions))
			throwProcessorException("The channels to be merged must all be of the same size");

		cv::Mat imageChannels[] = {channel1Blob->getMatrix(), channel2Blob->getMatrix(), channel3Blob->getMatrix()};
		cv::Mat processedImage;
		cv::merge(imageChannels, 3, processedImage);

		ctx.setContextEntry(_block->outPort(), DataBlob{getPortDataDescriptor(_block->outPort()), std::move(processedImage), channel1Blob->getColorSpace()});
	}
}
