/******************************************************************************
 * File: OutputProcessor.cpp
 * Date: 21.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "OutputProcessor.h"
#include "pipeline/blocks/InputBlock.h"
#include "project/Label.h"
#include "core/GrinderApplication.h"

#include <opencv2/imgproc.hpp>

OutputProcessor::OutputProcessor(const Block* block) : Processor(block)
{

}

void OutputProcessor::execute(EngineExecutionContext& ctx)
{
	Processor::execute(ctx);

	if (auto dataBlob = portData(ctx, _block->inPort()))
	{
		// Finally, get the image build for this block and set its image data
		if (auto imageBuild = ctx.label()->imageBuildPool().imageBuild(_block, ctx.activeImageReference()))
		{
			// Before displaying the image, make sure that it is of an 8-bit unsigned int type
			cv::Mat processedImage;
			cv::convertScaleAbs(dataBlob->getMatrix(), processedImage, *_block->scaleFactor(), *_block->delta());

			imageBuild->setImageData(processedImage);

			if (ctx.getExecutionMode() == Engine::ExecutionMode::View)
				grinder()->imageEditorManager().showEditor(_block, imageBuild);
		}
	}
}
