/******************************************************************************
 * File: ReplaceColorProcessor.cpp
 * Date: 19.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ReplaceColorProcessor.h"
#include "cv/Pixel.h"
#include "util/CVUtils.h"

ReplaceColorProcessor::ReplaceColorProcessor(const Block* block) : Processor(block)
{

}

void ReplaceColorProcessor::execute(EngineExecutionContext& ctx)
{
	Processor::execute(ctx);

	if (auto dataBlob = portData(ctx, _block->inPort()))
	{
		cv::Mat processedImage = dataBlob->getMatrix().clone();
		PixelAccessor pixels{processedImage};
		QColor fromColor = *_block->fromColor();

		if (dataBlob->dataDescriptor().isGrayscale())
			fromColor = CVUtils::colorToGrayscale(fromColor);

		pixels.forEach([this, fromColor](PixelRef pixel, QPoint pos) {
			Q_UNUSED(pos);

			if (CVUtils::compareColors(pixel, fromColor, *_block->tolerance() / 100.0f))
				pixel = *_block->toColor();
		});

		ctx.setContextEntry(_block->outPort(), DataBlob{getPortDataDescriptor(_block->outPort()), std::move(processedImage), dataBlob->getColorSpace()});
	}
}
