/******************************************************************************
 * File: SplitChannelsProcessor.cpp
 * Date: 14.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "SplitChannelsProcessor.h"

#include <opencv2/core.hpp>

SplitChannelsProcessor::SplitChannelsProcessor(const Block* block) : Processor(block)
{

}

void SplitChannelsProcessor::execute(EngineExecutionContext& ctx)
{
	Processor::execute(ctx);

	if (auto dataBlob = portData(ctx, _block->inPort()))
	{
		// A 3-channel image is required for splitting
		if (!dataBlob->dataDescriptor().isColor())
			throwProcessorException("It is only possible to split color images or 3D points");

		cv::Mat imageChannels[3];
		cv::split(dataBlob->getMatrix(), imageChannels);

		ctx.setContextEntry(_block->channel1Port(), DataBlob{getPortDataDescriptor(_block->channel1Port()), std::move(imageChannels[0]), dataBlob->getColorSpace()});
		ctx.setContextEntry(_block->channel2Port(), DataBlob{getPortDataDescriptor(_block->channel2Port()), std::move(imageChannels[1]), dataBlob->getColorSpace()});
		ctx.setContextEntry(_block->channel3Port(), DataBlob{getPortDataDescriptor(_block->channel3Port()), std::move(imageChannels[2]), dataBlob->getColorSpace()});
	}
}
