/******************************************************************************
 * File: DraftItem.h
 * Date: 20.3.2018
 *****************************************************************************/

#ifndef DRAFTITEM_H
#define DRAFTITEM_H

#include "common/properties/PropertyObject.h"
#include "DraftItemType.h"
#include "properties/ImageTagsAllotmentProperty.h"
#include "LayerItem.h"

namespace grndr
{
	class Layer;

	class DraftItem : public LayerItem<PropertyObject>
	{
		Q_OBJECT

	public:
		static const char* Serialization_Value_Type;

	public:
		DraftItem(Layer* layer, DraftItemType type);

	public:
		virtual void initDraftItem();

	public:
		DraftItemType getType() const { return _type; }
		int getZOrder() const;

		auto primaryColor() { return dynamic_cast<ColorProperty*>(_primaryColor.get()); }
		auto primaryColor() const { return dynamic_cast<const ColorProperty*>(_primaryColor.get()); }
		auto position() { return dynamic_cast<PointProperty*>(_position.get()); }
		auto position() const { return dynamic_cast<const PointProperty*>(_position.get()); }
		auto hasDirection() { return dynamic_cast<BoolProperty*>(_hasDirection.get()); }
		auto hasDirection() const { return dynamic_cast<const BoolProperty*>(_hasDirection.get()); }
		auto direction() { return dynamic_cast<AngleProperty*>(_direction.get()); }
		auto direction() const { return dynamic_cast<const AngleProperty*>(_direction.get()); }
		auto imageTagsAllotment() { return dynamic_cast<ImageTagsAllotmentProperty*>(_imageTagsAllotment.get()); }
		auto imageTagsAllotment() const { return dynamic_cast<const ImageTagsAllotmentProperty*>(_imageTagsAllotment.get()); }

	public:
		virtual void setDefaultPropertyValues() { }
		virtual void setDragPropertyValues(QPoint initialPos, QPoint currentPos) { Q_UNUSED(initialPos); Q_UNUSED(currentPos); }

		virtual void normalizePropertyValues() { }

	public:
		virtual void serialize(SerializationContext& ctx) const override;
		virtual void deserialize(DeserializationContext& ctx) override;

	protected:
		virtual void createProperties() override;

	protected:
		DraftItemType _type{DraftItemType::Undefined};

		std::shared_ptr<PropertyBase> _primaryColor;
		std::shared_ptr<PropertyBase> _position;
		std::shared_ptr<PropertyBase> _hasDirection;
		std::shared_ptr<PropertyBase> _direction;
		std::shared_ptr<PropertyBase> _imageTagsAllotment;
	};
}

#endif
