/******************************************************************************
 * File: ImageBuildPool.h
 * Date: 12.3.2018
 *****************************************************************************/

#ifndef IMAGEBUILDPOOL_H
#define IMAGEBUILDPOOL_H

#include "ImageBuildVector.h"

namespace grndr
{
	class Label;
	class Block;

	class ImageBuildPool : public QObject
	{
		Q_OBJECT

	public:
		static const char* Serialization_Group_Item;

		static const char* Serialization_Value_Block;

	public:
		ImageBuildPool(Label* label);

	public:
		const ImageBuildVector* imageBuilds(const Block* block) const;
		void removeImageBuilds(const Block* block);
		void removeImageBuilds(const ImageReference* imageRef);

		std::shared_ptr<ImageBuild> imageBuild(const Block* block, const std::vector<const ImageReference*>& imageReferences);
		std::shared_ptr<ImageBuild> imageBuild(const Block* block, const ImageReference* activeImageReference);
		void removeImageBuild(const Block* block, const ImageReference* imageRef);

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(DeserializationContext& ctx);

	signals:
		void imageBuildCreated(const std::shared_ptr<ImageBuild>&);
		void imageBuildRemoved(const std::shared_ptr<ImageBuild>&);

	private:
		std::shared_ptr<ImageBuildVector> imageBuildVector(const Block* block);
		std::shared_ptr<ImageBuild> imageBuild(std::shared_ptr<ImageBuildVector>& builds, const Block* block, const std::vector<const ImageReference*>& imageReferences);

	private slots:
		void blockRemoved(const std::shared_ptr<Block>& block);
		void imageReferenceRemoved(const std::shared_ptr<ImageReference>& imageRef);

	private:
		std::map<const Block*, std::shared_ptr<ImageBuildVector>> _imageBuilds;
	};
}

#endif
