/******************************************************************************
 * File: ImageTag.cpp
 * Date: 17.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageTag.h"

const char* ImageTag::Serialization_Value_Name = "Name";
const char* ImageTag::Serialization_Value_Color = "Color";

ImageTag::ImageTag(ImageTags* imageTags, QString name, QColor color) :
	_imageTags{imageTags}, _name{name}, _color{color}
{
	if (!imageTags)
		throw std::invalid_argument{_EXCPT("imageTags may not be null")};
}

QColor ImageTag::getTextColor() const
{
	// Use either black or white as the text's color based on the perceived luminance
	auto luminance = 1.0 - (0.299 * _color.red() + 0.587 * _color.green() + 0.114 * _color.blue()) / 255.0;
	return luminance < 0.5 ? Qt::black : Qt::white;
}

void ImageTag::serialize(SerializationContext& ctx) const
{
	// Serialize values
	ctx.settings()(Serialization_Value_Name) = _name;
	ctx.settings()(Serialization_Value_Color) = _color.name();
}

void ImageTag::deserialize(DeserializationContext& ctx)
{
	// Deserialize values
	_color = QColor{ctx.settings()(Serialization_Value_Color).toString()};
}
