/******************************************************************************
 * File: ImageTagsAllotment.h
 * Date: 03.5.2018
 *****************************************************************************/

#ifndef IMAGETAGSALLOTMENT_H
#define IMAGETAGSALLOTMENT_H

#include <set>

#include "ImageTag.h"
#include "common/serialization/SerializationContext.h"
#include "common/serialization/DeserializationContext.h"

namespace grndr
{
	class ImageBuild;
	class ImageTags;

	class ImageTagsAllotment : public QObject
	{
		Q_OBJECT

	public:
		static const char* Serialization_Value_AllottedTags;

	public:
		ImageTagsAllotment& operator =(const ImageTagsAllotment& imageTagsAllotment);

	public:
		void setImageBuild(const ImageBuild* imageBuild);

	public:
		const ImageTags* inputImageTags() const { return _inputImageTags; }

	public:
		void allotTag(ImageTag* imageTag);
		void unallotTag(ImageTag* imageTag);
		void unallotAllTags() { _allottedTags.clear(); emit allotmentChanged(); }

		auto& allottedTags() const { return _allottedTags; }

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(DeserializationContext& ctx);

	signals:
		void allotmentChanged();
		void allotmentModified();

	private:
		void setInputImageTags(const ImageTags* imageTags);

	private slots:
		void inputImageTagsChanged(const ImageTags* imageTags) { setInputImageTags(imageTags); }

		void imageTagModified(const std::shared_ptr<ImageTag>& imageTag) { Q_UNUSED(imageTag); emit allotmentModified(); }
		void imageTagRemoved(const std::shared_ptr<ImageTag>& imageTag);

	private:		
		struct ImageTagPtrCompare
		{
			bool operator ()(const ImageTag* lhs, const ImageTag* rhs) const
			{
				return lhs->getName() < rhs->getName();
			}
		};

		const ImageBuild* _imageBuild{nullptr};

		const ImageTags* _inputImageTags{nullptr};
		std::set<ImageTag*, ImageTagPtrCompare> _allottedTags;
	};
}

#endif
