/******************************************************************************
 * File: ItemRenderer.h
 * Date: 21.3.2018
 *****************************************************************************/

#ifndef ITEMRENDERER_H
#define ITEMRENDERER_H

#include <QPainter>
#include <QPalette>
#include <memory>

namespace grndr
{
	class ImageEditorStyle;

	class ItemRenderer : public QObject
	{
		Q_OBJECT

	public:
		enum class RenderModeFlag : unsigned int
		{
			RelativePositions = 0x0000,
			AbsolutePositions = 0x0001,

			RenderToImage = AbsolutePositions,
			RenderToScene = RelativePositions,
		};

		Q_DECLARE_FLAGS(RenderMode, RenderModeFlag)

		enum class RenderFlag : unsigned int
		{
			NoFlag = 0x0000,
			Selected = 0x0001,
			Inactive = 0x0002,
			ShowDirections = 0x0004,
			ShowTags = 0x0008,
		};

		Q_DECLARE_FLAGS(RenderFlags, RenderFlag)

		struct RendererStyle
		{
			RendererStyle();

			QColor selectionColor{QPalette{}.highlight().color().lighter()};
			QColor selectionColorInactive{QPalette{}.color(QPalette::Inactive, QPalette::Highlight).darker()};
			float selectionMargin{6.0f};
			float selectionOpacity{0.6f};

			float directionArrowLength{40.0};
			QColor directionArrowColor{0, 255, 128};
			float directionArrowWidth{3.5f};
			float directionArrowOpacity{0.6f};

			QFont imageTagsFont{};
			float imageTagsMargin{15.0};
			QMarginsF imageTagsLabelMargin{5.0f, 2.5f, 5.0f, 2.5f};
			float imageTagsLabelRadius{4.0f};
			float imageTagsLabelBorderWidth{1.0f};
			QColor imageTagsLabelLineColor{235, 235, 235};
			float imageTagsLabelLineWidth{2.5f};
			float imageTagsOpacity{0.75f};
		};

	public:
		ItemRenderer(const RendererStyle& rendererStyle);

	public:
		virtual void render(QPainter* painter, RenderMode mode, RenderFlags flags) = 0;
		virtual QPainterPath shape(RenderFlags flags) const { Q_UNUSED(flags); return QPainterPath{}; }

	protected:
		virtual QPoint getItemPosition(RenderMode mode) const = 0;

	protected:
		RendererStyle _rendererStyle;
	};
}

Q_DECLARE_OPERATORS_FOR_FLAGS(grndr::ItemRenderer::RenderMode)
Q_DECLARE_OPERATORS_FOR_FLAGS(grndr::ItemRenderer::RenderFlags)

#endif
