/******************************************************************************
 * File: Layer.h
 * Date: 17.3.2018
 *****************************************************************************/

#ifndef LAYER_H
#define LAYER_H

#include "ImageBuildItem.h"
#include "DraftItemVector.h"
#include "LayerPixels.h"

namespace grndr
{
	class Layer : public ImageBuildItem
	{
		Q_OBJECT

	public:
		static const char* Serialization_Value_Type;
		static const char* Serialization_Value_Visible;
		static const char* Serialization_Value_Alpha;
		static const char* Serialization_Value_BackgroundImage;

		enum class Type
		{
			Standard,
			BackgroundImage,
		};

	public:
		Layer(ImageBuild* imageBuild, QString name = "", Type type = Type::Standard, const ImageReference* backgroundImage = nullptr);

	public:
		void initLayer();

	public:
		std::shared_ptr<DraftItem> createDraftItem(DraftItemType type);
		void removeDraftItem(const DraftItem* item);

	public:
		Type getType() const { return _type; }

		int getZOrder() const;

		bool isVisible() const { return _isVisible; }
		void setVisible(bool visible = true);
		unsigned int getAlpha() const { return _alpha; }
		void setAlpha(unsigned int alpha);

		bool isRemovable() const { return _type == Type::Standard; }

		const DraftItemVector& draftItems() const { return _draftItems; }
		LayerPixels& layerPixels() { return _layerPixels; }
		const LayerPixels& layerPixels() const { return _layerPixels; }

		const ImageReference* backgroundImage() const { return _backgroundImage; }
		void setBackgroundImage(const ImageReference* imageRef) { _backgroundImage = imageRef; }

	public:
		virtual void serialize(SerializationContext& ctx) const override;
		virtual void deserialize(DeserializationContext& ctx) override;

	signals:
		void layerShown();
		void layerHidden();
		void layerAlphaChanged(unsigned int);

		void draftItemCreated(const std::shared_ptr<DraftItem>&);
		void draftItemRemoved(const std::shared_ptr<DraftItem>&);

	private:
		Type _type{Type::Standard};

		bool _isVisible{true};
		unsigned int _alpha{100};

		DraftItemVector _draftItems;
		LayerPixels _layerPixels;

		const ImageReference* _backgroundImage{nullptr};
	};
}

#endif
