/******************************************************************************
 * File: BoxDraftItem.cpp
 * Date: 20.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "BoxDraftItem.h"
#include "image/renderers/BoxDraftItemRenderer.h"

const DraftItemType BoxDraftItem::type_value = DraftItemType::Box;

BoxDraftItem::BoxDraftItem(Layer* layer) : DraftItem(layer, type_value)
{

}

std::unique_ptr<ItemRenderer> BoxDraftItem::createRenderer(const ItemRenderer::RendererStyle& rendererStyle) const
{
	return std::make_unique<BoxDraftItemRenderer>(this, rendererStyle);
}

void BoxDraftItem::createProperties()
{
	DraftItem::createProperties();

	// Create specific properties
	_boxSize = createProperty<SizeProperty>(PropertyID::Size, "Size", QSize{50, 50});
	boxSize()->setDescription("The size of the box.");

	_lineWidth = createProperty<UIntProperty>(PropertyID::Width, "Line width", 5);
	lineWidth()->createConstraint<RangeConstraint>(1, 100);
	lineWidth()->setDescription("The width of the box lines.");

	_solidBox = createProperty<BoolProperty>(PropertyID::Solid, "Solid box", false);
	solidBox()->setDescription("Fill the entire box.");

	// Override some property defaults
	hasDirection()->setValue(true);
}

void BoxDraftItem::setDragPropertyValues(QPoint initialPos, QPoint currentPos)
{
	auto vec = currentPos - initialPos;
	QSize size{vec.x(), vec.y()};
	boxSize()->setValue(size);
}

void BoxDraftItem::normalizePropertyValues()
{
	// Adjust for negative box sizes
	QPoint pos = *position();
	QSize size = *boxSize();

	if (size.width() < 0)
		pos.setX(pos.x() + size.width());

	if (size.height() < 0)
		pos.setY(pos.y() + size.height());

	size.setWidth(std::abs(size.width()));
	size.setHeight(std::abs(size.height()));

	position()->setValue(pos);
	boxSize()->setValue(size);
}
