/******************************************************************************
 * File: LineDraftItem.cpp
 * Date: 20.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "LineDraftItem.h"
#include "image/renderers/LineDraftItemRenderer.h"

const DraftItemType LineDraftItem::type_value = DraftItemType::Line;

LineDraftItem::LineDraftItem(Layer* layer) : DraftItem(layer, type_value)
{

}

std::unique_ptr<ItemRenderer> LineDraftItem::createRenderer(const ItemRenderer::RendererStyle& rendererStyle) const
{
	return std::make_unique<LineDraftItemRenderer>(this, rendererStyle);
}

void LineDraftItem::setDragPropertyValues(QPoint initialPos, QPoint currentPos)
{
	Q_UNUSED(initialPos);

	endPosition()->setValue(currentPos);
}

void LineDraftItem::setDefaultPropertyValues()
{
	DraftItem::setDefaultPropertyValues();

	// Create a default line that extends 50px to the right
	endPosition()->setValue(position()->getValue() + QPoint{50, 0});
}

void LineDraftItem::createProperties()
{
	DraftItem::createProperties();

	// Create specific properties
	_endPosition = createProperty<PointProperty>(PropertyID::EndPosition, "End position", QPoint{0, 0});
	endPosition()->setDescription("The end position of the line.");

	_lineWidth = createProperty<UIntProperty>(PropertyID::Width, "Line width", 3);
	lineWidth()->createConstraint<RangeConstraint>(1, 100);
	lineWidth()->setDescription("The width of the line.");
}
