/******************************************************************************
 * File: LayerPixelsRenderer.cpp
 * Date: 18.5.2018
 *****************************************************************************/

#include "Grinder.h"
#include "LayerPixelsRenderer.h"
#include "image/LayerPixelsData.h"

LayerPixelsRenderer::LayerPixelsRenderer(const LayerPixelsData* pixelsData, const ItemRenderer::RendererStyle& rendererStyle) : ItemRenderer(rendererStyle),
	_pixelsData{pixelsData}
{
	if (!pixelsData)
		throw std::invalid_argument{_EXCPT("pixelsData may not be null")};

	// Update the pixels' shape whenever the data has been modified
	connect(pixelsData, &LayerPixelsData::dataModified, this, &LayerPixelsRenderer::updateShape);

	updateShape();
}

void LayerPixelsRenderer::render(QPainter* painter, ItemRenderer::RenderMode mode, ItemRenderer::RenderFlags flags)
{
	Q_UNUSED(flags);

	painter->save();

	auto itemPosition = getItemPosition(mode);

	// Draw each color individually
	for (auto color : _pixelsData->getColors())
	{
		painter->setPen(QPen{color, 1.0});

		// Collect all points belonging to the current color
		std::vector<QPoint> points;
		auto& rangeMap = _pixelsData->at(color);

		for (const auto& rangeMapEntry : rangeMap)
		{
			// Iterator is a tuple containing ((row <- y, rangeMap), rangeVector iterator, current range value <- x)
			auto xPos = std::get<2>(rangeMapEntry);
			auto yPos = std::get<0>(rangeMapEntry)->first;
			points.emplace_back(xPos + itemPosition.x(), yPos + itemPosition.y());
		}

		// Draw all points in one go
		painter->drawPoints(points.data(), points.size());
	}

	painter->restore();
}

QPainterPath LayerPixelsRenderer::shape(RenderFlags flags) const
{
	Q_UNUSED(flags);
	return _pixelsShape;
}

void LayerPixelsRenderer::updateShape()
{
	QRegion region;

	// Add each pixel as a region
	for (const auto& pixel : *_pixelsData)
	{
		// Iterator is a tuple containing (color, (row <- y, rangeMap), rangeVector iterator, current range value <- x)
		auto xPos = std::get<3>(pixel);
		auto yPos = std::get<1>(pixel)->first;
		region += QRect{xPos, yPos, 1, 1};
	}

	QPainterPath pixelsPath;
	pixelsPath.addRegion(region);
	_pixelsShape = pixelsPath;
}
