/******************************************************************************
 * File: BlockCatalog.h
 * Date: 29.1.2018
 *****************************************************************************/

#ifndef BLOCKCATALOG_H
#define BLOCKCATALOG_H

#include <map>
#include <set>
#include <functional>
#include <memory>

#include "BlockType.h"
#include "BlockCategory.h"

namespace grndr
{
	class Pipeline;
	class Block;

	class BlockCatalog
	{
	private:
		using block_creator_type = std::function<std::unique_ptr<Block>(Pipeline*, QString)>;

	private:
		BlockCatalog() { }

	public:
		static std::unique_ptr<Block> createBlock(Pipeline* pipeline, BlockType type, QString name = "");

		static void registerStandardBlocks();

	public:
		static std::set<BlockType> getTypes(BlockCategory category = BlockCategory::Undefined);
		static std::set<BlockCategory> getCategories();

		static BlockCategory getCategory(BlockType type);
		static QString getDescription(BlockType type);

	private:
		static void registerBlockType(BlockType type, BlockCategory category, block_creator_type creator, QString description);

	private:
		struct BlockDescriptor
		{
			BlockType type;
			BlockCategory category;			

			block_creator_type creator;

			QString description;
		};

		static std::map<BlockType, BlockDescriptor> s_descriptors;
	};
}

#endif
