/******************************************************************************
 * File: PipelineExceptions.h
 * Date: 15.1.2018
 *****************************************************************************/

#ifndef PIPELINEEXCEPTIONS_H
#define PIPELINEEXCEPTIONS_H

#include <QString>

#include "core/GrinderExceptions.h"

namespace grndr
{
	class Pipeline;
	class PipelineItem;
	class Block;
	class Port;

	class PipelineException : public GrinderException
	{
	public:
		PipelineException(const Pipeline* pipeline, QString what);

	public:
		const Pipeline* pipeline() const { return _pipeline; }

	protected:
		const Pipeline* _pipeline{nullptr};
	};

	class PipelineItemException : public PipelineException
	{
	public:
		PipelineItemException(const PipelineItem* item, QString what);

	public:
		const PipelineItem* pipelineItem() const { return _pipelineItem; }

	protected:
		const PipelineItem* _pipelineItem{nullptr};
	};

	class BlockException : public PipelineItemException
	{
	public:
		BlockException(const Block* block, QString what);

	public:
		const Block* block() const { return _block; }

	protected:
		const Block* _block{nullptr};
	};

	class PortException : public BlockException
	{
	public:
		PortException(const Port* port, QString what);

	public:
		const Port* port() const { return _port; }

	protected:
		const Port* _port{nullptr};
	};
}

#endif
