/******************************************************************************
 * File: Port.h
 * Date: 15.1.2018
 *****************************************************************************/

#ifndef PORT_H
#define PORT_H

#include "PipelineItem.h"
#include "PortType.h"
#include "ConnectionVector.h"
#include "BlockType.h"
#include "engine/data/DataDescriptor.h"

namespace grndr
{
	class Block;

	class Port : public PipelineItem
	{
		Q_OBJECT

	public:
		static const char* Serialization_Value_Type;
		static const char* Serialization_Value_Index;

	public:
		enum class Direction
		{
			Dry,	/* Usually indicates an invalid port */
			In,
			Out,
		};

	public:
		Port(Block* parent, PortType type, Direction dir, const DataDescriptors& dataDescriptors, QString name = "");

	public:
		void initPort();		

		void verifyConnection(const Port* dest) const;
		std::shared_ptr<Connection> connectTo(Port* dest);
		void disconnectFrom(const Port* dest);
		void disconnectFromAll();		
		void removeConnection(const Connection* connection);

	public:
		Block* block() { return _block; }
		const Block* block() const { return _block; }

		QString getFormattedName() const;
		PortType getType() const { return _type; }
		Direction getDirection() const { return _direction; }
		void setDirection(Direction dir) { _direction = dir; }
		bool isIn() const { return _direction == Direction::In; }
		bool isOut() const { return _direction == Direction::Out; }
		bool isConnected() const;

		const DataDescriptors& dataDescriptors() const { return _dataDescriptors; }

		const ConnectionVector& connections() const { return _connections; }
		ConnectionVector::pointer_vec_type getConnections(Direction dir, BlockType blockType = BlockType::Undefined) const;

	public:
		virtual void serialize(SerializationContext& ctx) const override;
		virtual void deserialize(DeserializationContext& ctx) override;

	signals:
		void portConnected(const Connection*);
		void portDisconnected(const Connection*);

	private:
		Block* _block{nullptr};

		PortType _type{PortType::Undefined};
		Direction _direction{Direction::Dry};
		DataDescriptors _dataDescriptors;

		ConnectionVector _connections;

	};
}

#endif
