/******************************************************************************
 * File: AdaptiveThresholdBlock.cpp
 * Date: 07.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "AdaptiveThresholdBlock.h"
#include "engine/processors/AdaptiveThresholdProcessor.h"
#include "cv/CVTypes.h"

const BlockType AdaptiveThresholdBlock::type_value = BlockType::AdaptiveThreshold;
const BlockCategory AdaptiveThresholdBlock::category_value = BlockCategory::Thresholding;

AdaptiveThresholdBlock::AdaptiveThresholdBlock(Pipeline* pipeline, QString name) : Block(pipeline, type_value, category_value, name)
{

}

std::unique_ptr<ProcessorBase> AdaptiveThresholdBlock::createProcessor() const
{
	return std::make_unique<AdaptiveThresholdProcessor>(this);
}

void AdaptiveThresholdBlock::createProperties()
{
	Block::createProperties();

	_thresholdMethod = createProperty<AdaptiveThresholdMethodProperty>(PropertyID::Method, "Method", static_cast<int>(AdaptiveThresholdMethod::Gaussian));
	thresholdMethod()->setDescription("The adaptive method.");

	_thresholdType = createProperty<AdaptiveThresholdTypeProperty>(PropertyID::Type, "Type", static_cast<int>(AdaptiveThresholdType::Binary));
	thresholdType()->setDescription("The threshold type.");

	_targetValue = createProperty<UIntProperty>(PropertyID::TargetValue, "Target value", 255);
	targetValue()->createConstraint<RangeConstraint>(0, 255);
	targetValue()->setDescription("The value to use if a pixel is above (below if inverted) the threshold value; ranges from 0 to 255.");

	_blockSize = createProperty<UIntProperty>(PropertyID::Size, "Block size", 3);
	blockSize()->createConstraint<RangeConstraint>(3, std::numeric_limits<unsigned int>::max());
	blockSize()->createConstraint<EvenOddConstraint>(EvenOddConstraint<unsigned int>::Type::Odd);
	blockSize()->setDescription("The size of the pixel neighborhood used to calculate the threshold value for a pixel. Must be an odd value (3, 5, 7, and so on).");

	_constant = createProperty<RealProperty>(PropertyID::Constant, "Constant", 0.0);
	constant()->setDescription("Constant subtracted from the (weighted) mean.");
}

void AdaptiveThresholdBlock::createPorts()
{
	DataDescriptors inPortDataDescs = {DataDescriptor::imageDescriptor(false, DataDescriptor::ValueType::UInt8)};
	_inPort = createPort(PortType::ImageIn, Port::Direction::In, inPortDataDescs, "In");

	DataDescriptors outPortDataDescs = {DataDescriptor::imageDescriptor(false, DataDescriptor::ValueType::UInt8)};
	_outPort = createPort(PortType::ImageOut, Port::Direction::Out, outPortDataDescs, "Out");
}
