/******************************************************************************
 * File: AdaptiveThresholdBlock.h
 * Date: 07.6.2018
 *****************************************************************************/

#ifndef ADAPTIVETHRESHOLDBLOCK_H
#define ADAPTIVETHRESHOLDBLOCK_H

#include "pipeline/Block.h"
#include "pipeline/properties/AdaptiveThresholdMethodProperty.h"
#include "pipeline/properties/AdaptiveThresholdTypeProperty.h"

namespace grndr
{
	class AdaptiveThresholdBlock : public Block
	{
		Q_OBJECT

	public:
		static const BlockType type_value;
		static const BlockCategory category_value;

	public:
		AdaptiveThresholdBlock(Pipeline* pipeline, QString name = "");

	public:
		virtual std::unique_ptr<ProcessorBase> createProcessor() const override;

	public:
		auto thresholdMethod() { return dynamic_cast<AdaptiveThresholdMethodProperty*>(_thresholdMethod.get()); }
		auto thresholdMethod() const { return dynamic_cast<const AdaptiveThresholdMethodProperty*>(_thresholdMethod.get()); }
		auto thresholdType() { return dynamic_cast<AdaptiveThresholdTypeProperty*>(_thresholdType.get()); }
		auto thresholdType() const { return dynamic_cast<const AdaptiveThresholdTypeProperty*>(_thresholdType.get()); }
		auto targetValue() { return dynamic_cast<UIntProperty*>(_targetValue.get()); }
		auto targetValue() const { return dynamic_cast<const UIntProperty*>(_targetValue.get()); }
		auto blockSize() { return dynamic_cast<UIntProperty*>(_blockSize.get()); }
		auto blockSize() const { return dynamic_cast<const UIntProperty*>(_blockSize.get()); }
		auto constant() { return dynamic_cast<RealProperty*>(_constant.get()); }
		auto constant() const { return dynamic_cast<const RealProperty*>(_constant.get()); }

		Port* inPort() { return _inPort.get(); }
		const Port* inPort() const { return _inPort.get(); }
		Port* outPort() { return _outPort.get(); }
		const Port* outPort() const { return _outPort.get(); }

	protected:
		virtual void createProperties() override;
		virtual void createPorts() override;

	private:
		std::shared_ptr<PropertyBase> _thresholdMethod;
		std::shared_ptr<PropertyBase> _thresholdType;
		std::shared_ptr<PropertyBase> _targetValue;
		std::shared_ptr<PropertyBase> _blockSize;
		std::shared_ptr<PropertyBase> _constant;

		std::shared_ptr<Port> _inPort;
		std::shared_ptr<Port> _outPort;
	};
}

#endif
