/******************************************************************************
 * File: AlphaBlendingBlock.cpp
 * Date: 07.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "AlphaBlendingBlock.h"
#include "engine/processors/AlphaBlendingProcessor.h"

const BlockType AlphaBlendingBlock::type_value = BlockType::AlphaBlending;
const BlockCategory AlphaBlendingBlock::category_value = BlockCategory::Blending;

AlphaBlendingBlock::AlphaBlendingBlock(Pipeline* pipeline, QString name) : Block(pipeline, type_value, category_value, name)
{

}

std::unique_ptr<ProcessorBase> AlphaBlendingBlock::createProcessor() const
{
	return std::make_unique<AlphaBlendingProcessor>(this);
}

void AlphaBlendingBlock::createProperties()
{
	Block::createProperties();

	_alpha = createProperty<RealProperty>(PropertyID::Alpha, "Alpha", 0.5);
	alpha()->createConstraint<RangeConstraint>(0.0, 1.0);
	alpha()->setDescription("The alpha blending factor (Out = In1*Alpha + In2*[1-Alpha]).");
}

void AlphaBlendingBlock::createPorts()
{
	DataDescriptors inPortDataDescs = {DataDescriptor::imageDescriptor(true, DataDescriptor::ValueType::Any), DataDescriptor::imageDescriptor(false, DataDescriptor::ValueType::Any)};
	_src1Port = createPort(PortType::ImageSrc1, Port::Direction::In, inPortDataDescs, "In 1");
	_src2Port = createPort(PortType::ImageSrc2, Port::Direction::In, inPortDataDescs, "In 2");

	DataDescriptors outPortDataDescs = {DataDescriptor::dynamicOutputDescriptor("Image")};
	_outPort = createPort(PortType::ImageOut, Port::Direction::Out, outPortDataDescs, "Out");
}
