/******************************************************************************
 * File: AlphaBlendingBlock.h
 * Date: 07.6.2018
 *****************************************************************************/

#ifndef ALPHABLENDINGBLOCK_H
#define ALPHABLENDINGBLOCK_H

#include "pipeline/Block.h"

namespace grndr
{
	class AlphaBlendingBlock : public Block
	{
		Q_OBJECT

	public:
		static const BlockType type_value;
		static const BlockCategory category_value;

	public:
		AlphaBlendingBlock(Pipeline* pipeline, QString name = "");

	public:
		virtual std::unique_ptr<ProcessorBase> createProcessor() const override;

	public:
		auto alpha() { return dynamic_cast<RealProperty*>(_alpha.get()); }
		auto alpha() const { return dynamic_cast<const RealProperty*>(_alpha.get()); }

		Port* src1Port() { return _src1Port.get(); }
		const Port* src1Port() const { return _src1Port.get(); }
		Port* src2Port() { return _src2Port.get(); }
		const Port* src2Port() const { return _src2Port.get(); }
		Port* outPort() { return _outPort.get(); }
		const Port* outPort() const { return _outPort.get(); }

	protected:
		virtual void createProperties() override;
		virtual void createPorts() override;

	private:
		std::shared_ptr<PropertyBase> _alpha;

		std::shared_ptr<Port> _src1Port;
		std::shared_ptr<Port> _src2Port;
		std::shared_ptr<Port> _outPort;
	};
}

#endif
