/******************************************************************************
 * File: BinaryThresholdBlock.cpp
 * Date: 20.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "BinaryThresholdBlock.h"
#include "engine/processors/BinaryThresholdProcessor.h"
#include "cv/CVTypes.h"

const BlockType BinaryThresholdBlock::type_value = BlockType::BinaryThreshold;
const BlockCategory BinaryThresholdBlock::category_value = BlockCategory::Thresholding;

BinaryThresholdBlock::BinaryThresholdBlock(Pipeline* pipeline, QString name) : Block(pipeline, type_value, category_value, name)
{

}

std::unique_ptr<ProcessorBase> BinaryThresholdBlock::createProcessor() const
{
	return std::make_unique<BinaryThresholdProcessor>(this);
}

void BinaryThresholdBlock::createProperties()
{
	Block::createProperties();

	_thresholdType = createProperty<BinaryThresholdTypeProperty>(PropertyID::Type, "Type", static_cast<int>(BinaryThresholdType::Binary));
	thresholdType()->setDescription("The threshold type.");

	_thresholdMode = createProperty<BinaryThresholdModeProperty>(PropertyID::Mode, "Threshold mode", static_cast<int>(BinaryThresholdMode::Manual));
	thresholdMode()->setDescription("The threshold mode (either manually or automatically via Otsu/Triangle).");

	_threshold = createProperty<UIntProperty>(PropertyID::Threshold, "Threshold", 128);
	threshold()->createConstraint<RangeConstraint>(0, 255);
	threshold()->setDescription("The threshold value to use; ranges from 0 to 255.");

	_targetValue = createProperty<UIntProperty>(PropertyID::TargetValue, "Target value", 255);
	targetValue()->createConstraint<RangeConstraint>(0, 255);
	targetValue()->setDescription("The value to use for binary thresholding if a pixel is above (below if inverted) the threshold value; ranges from 0 to 255.");
}

void BinaryThresholdBlock::createPorts()
{
	DataDescriptors inPortDataDescs = {DataDescriptor::imageDescriptor(true, DataDescriptor::ValueType::Any), DataDescriptor::imageDescriptor(false, DataDescriptor::ValueType::Any)};
	_inPort = createPort(PortType::ImageIn, Port::Direction::In, inPortDataDescs, "In");

	DataDescriptors outPortDataDescs = {DataDescriptor::dynamicOutputDescriptor("Image")};
	_outPort = createPort(PortType::ImageOut, Port::Direction::Out, outPortDataDescs, "Out");
}
