/******************************************************************************
 * File: BinaryThresholdBlock.h
 * Date: 20.2.2018
 *****************************************************************************/

#ifndef BINARYTHRESHOLDBLOCK_H
#define BINARYTHRESHOLDBLOCK_H

#include "pipeline/Block.h"
#include "pipeline/properties/BinaryThresholdTypeProperty.h"
#include "pipeline/properties/BinaryThresholdModeProperty.h"

namespace grndr
{
	class BinaryThresholdBlock : public Block
	{
		Q_OBJECT

	public:
		static const BlockType type_value;
		static const BlockCategory category_value;

	public:
		BinaryThresholdBlock(Pipeline* pipeline, QString name = "");

	public:
		virtual std::unique_ptr<ProcessorBase> createProcessor() const override;

	public:
		auto thresholdType() { return dynamic_cast<BinaryThresholdTypeProperty*>(_thresholdType.get()); }
		auto thresholdType() const { return dynamic_cast<const BinaryThresholdTypeProperty*>(_thresholdType.get()); }
		auto thresholdMode() { return dynamic_cast<BinaryThresholdModeProperty*>(_thresholdMode.get()); }
		auto thresholdMode() const { return dynamic_cast<const BinaryThresholdModeProperty*>(_thresholdMode.get()); }
		auto threshold() { return dynamic_cast<UIntProperty*>(_threshold.get()); }
		auto threshold() const { return dynamic_cast<const UIntProperty*>(_threshold.get()); }
		auto targetValue() { return dynamic_cast<UIntProperty*>(_targetValue.get()); }
		auto targetValue() const { return dynamic_cast<const UIntProperty*>(_targetValue.get()); }

		Port* inPort() { return _inPort.get(); }
		const Port* inPort() const { return _inPort.get(); }
		Port* outPort() { return _outPort.get(); }
		const Port* outPort() const { return _outPort.get(); }

	protected:
		virtual void createProperties() override;
		virtual void createPorts() override;

	private:
		std::shared_ptr<PropertyBase> _thresholdType;
		std::shared_ptr<PropertyBase> _thresholdMode;
		std::shared_ptr<PropertyBase> _threshold;
		std::shared_ptr<PropertyBase> _targetValue;

		std::shared_ptr<Port> _inPort;
		std::shared_ptr<Port> _outPort;
	};
}

#endif
