/******************************************************************************
 * File: ConvertToColorBlock.cpp
 * Date: 12.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ConvertToColorBlock.h"
#include "engine/processors/ConvertToColorProcessor.h"
#include "cv/CVTypes.h"

const BlockType ConvertToColorBlock::type_value = BlockType::ConvertToColor;
const BlockCategory ConvertToColorBlock::category_value = BlockCategory::Conversion;

ConvertToColorBlock::ConvertToColorBlock(Pipeline* pipeline, QString name) : Block(pipeline, type_value, category_value, name)
{

}

std::unique_ptr<ProcessorBase> ConvertToColorBlock::createProcessor() const
{
	return std::make_unique<ConvertToColorProcessor>(this);
}

void ConvertToColorBlock::createProperties()
{
	Block::createProperties();

	_colorSpace = createProperty<ColorSpaceProperty>(PropertyID::ColorSpace, "Color space", static_cast<int>(ColorSpace::RGB));
	colorSpace()->setDescription("The target color space.");
}

void ConvertToColorBlock::createPorts()
{
	DataDescriptors inPortDataDescs = {DataDescriptor::imageDescriptor(true, DataDescriptor::ValueType::Any), DataDescriptor::imageDescriptor(false, DataDescriptor::ValueType::Any)};
	_inPort = createPort(PortType::ImageIn, Port::Direction::In, inPortDataDescs, "In");

	DataDescriptors outPortDataDescs = {DataDescriptor::imageDescriptor(true, DataDescriptor::ValueType::Dynamic)};
	_outPort = createPort(PortType::ImageOut, Port::Direction::Out, outPortDataDescs, "Out");
}
