/******************************************************************************
 * File: NormalizeBlock.cpp
 * Date: 12.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "NormalizeBlock.h"
#include "engine/processors/NormalizeProcessor.h"

const BlockType NormalizeBlock::type_value = BlockType::Normalize;
const BlockCategory NormalizeBlock::category_value = BlockCategory::Conversion;

NormalizeBlock::NormalizeBlock(Pipeline* pipeline, QString name) : Block(pipeline, type_value, category_value, name)
{

}

std::unique_ptr<ProcessorBase> NormalizeBlock::createProcessor() const
{
	return std::make_unique<NormalizeProcessor>(this);
}

void NormalizeBlock::createProperties()
{
	Block::createProperties();

	_lowerRange = createProperty<UIntProperty>(PropertyID::Alpha, "Lower range", 0);
	lowerRange()->setDescription("The lower range value used for range normalization.");

	_upperRange = createProperty<UIntProperty>(PropertyID::Beta, "Upper range", 255);
	upperRange()->setDescription("The upper range value used for range normalization.");
}

void NormalizeBlock::createPorts()
{
	DataDescriptors inPortDataDescs = {DataDescriptor::imageDescriptor(true, DataDescriptor::ValueType::Any), DataDescriptor::imageDescriptor(false, DataDescriptor::ValueType::Any)};
	_inPort = createPort(PortType::ImageIn, Port::Direction::In, inPortDataDescs, "In");

	DataDescriptors outPortDataDescs = {DataDescriptor::dynamicOutputDescriptor("Image")};
	_outPort = createPort(PortType::ImageOut, Port::Direction::Out, outPortDataDescs, "Out");
}
