/******************************************************************************
 * File: OutputBlock.cpp
 * Date: 15.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "OutputBlock.h"
#include "engine/processors/OutputProcessor.h"

const BlockType OutputBlock::type_value = BlockType::Output;
const BlockCategory OutputBlock::category_value = BlockCategory::Output;

OutputBlock::OutputBlock(Pipeline* pipeline, QString name) : Block(pipeline, type_value, category_value, name)
{

}

std::unique_ptr<ProcessorBase> OutputBlock::createProcessor() const
{
	return std::make_unique<OutputProcessor>(this);
}

void OutputBlock::createProperties()
{
	Block::createProperties();

	_scaleFactor = createProperty<RealProperty>(PropertyID::Factor, "Scale factor", 1.0);
	scaleFactor()->setDescription("Scale factor for the image data.");

	_delta = createProperty<RealProperty>(PropertyID::Delta, "Delta", 0.0);
	delta()->setDescription("Delta added to the (scaled) image data.");
}

void OutputBlock::createPorts()
{
	DataDescriptors inPortDataDescs = {DataDescriptor::imageDescriptor(true, DataDescriptor::ValueType::Any), DataDescriptor::imageDescriptor(false, DataDescriptor::ValueType::Any)};
	_inPort = createPort(PortType::ImageIn, Port::Direction::In, inPortDataDescs, "In");

	_imageTagsPort = createPort(PortType::ImageTagsIn, Port::Direction::In, {}, "Tags");
}
