/******************************************************************************
 * File: OutputBlock.h
 * Date: 15.2.2018
 *****************************************************************************/

#ifndef OUTPUTBLOCK_H
#define OUTPUTBLOCK_H

#include "pipeline/Block.h"

namespace grndr
{
	class OutputBlock : public Block
	{
		Q_OBJECT

	public:
		static const BlockType type_value;
		static const BlockCategory category_value;

	public:
		OutputBlock(Pipeline* pipeline, QString name = "");

	public:
		virtual std::unique_ptr<ProcessorBase> createProcessor() const override;

	public:
		auto scaleFactor() { return dynamic_cast<RealProperty*>(_scaleFactor.get()); }
		auto scaleFactor() const { return dynamic_cast<const RealProperty*>(_scaleFactor.get()); }
		auto delta() { return dynamic_cast<RealProperty*>(_delta.get()); }
		auto delta() const { return dynamic_cast<const RealProperty*>(_delta.get()); }

		Port* inPort() { return _inPort.get(); }
		const Port* inPort() const { return _inPort.get(); }
		Port* imageTagsPort() { return _imageTagsPort.get(); }
		const Port* imageTagsPort() const { return _imageTagsPort.get(); }

	protected:
		virtual void createProperties() override;
		virtual void createPorts() override;

	private:
		std::shared_ptr<PropertyBase> _scaleFactor;
		std::shared_ptr<PropertyBase> _delta;

		std::shared_ptr<Port> _inPort;
		std::shared_ptr<Port> _imageTagsPort;
	};
}

#endif
