/******************************************************************************
 * File: ReplaceColorBlock.cpp
 * Date: 19.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ReplaceColorBlock.h"
#include "engine/processors/ReplaceColorProcessor.h"

const BlockType ReplaceColorBlock::type_value = BlockType::ReplaceColor;
const BlockCategory ReplaceColorBlock::category_value = BlockCategory::Conversion;

ReplaceColorBlock::ReplaceColorBlock(Pipeline* pipeline, QString name) : Block(pipeline, type_value, category_value, name)
{

}

std::unique_ptr<ProcessorBase> ReplaceColorBlock::createProcessor() const
{
	return std::make_unique<ReplaceColorProcessor>(this);
}

void ReplaceColorBlock::createProperties()
{
	Block::createProperties();

	_fromColor = createProperty<ColorProperty>(PropertyID::From, "From color", QColor{255, 255, 255});
	fromColor()->setDescription("The color to replace.");

	_toColor = createProperty<ColorProperty>(PropertyID::To, "To color", QColor{0, 0, 0});
	toColor()->setDescription("The new color.");

	_tolerance = createProperty<UIntProperty>(PropertyID::Tolerance, "Tolerance", 0);
	tolerance()->createConstraint<RangeConstraint>(0, 100);
	tolerance()->setDescription("The tolerance (in %) when comparing the image pixels against the color to replace.");
}

void ReplaceColorBlock::createPorts()
{
	DataDescriptors inPortDataDescs = {DataDescriptor::imageDescriptor(true), DataDescriptor::imageDescriptor(false)};
	_inPort = createPort(PortType::ImageIn, Port::Direction::In, inPortDataDescs, "In");

	DataDescriptors outPortDataDescs = {DataDescriptor::dynamicOutputDescriptor("Image")};
	_outPort = createPort(PortType::ImageOut, Port::Direction::Out, outPortDataDescs, "Out");
}
