/******************************************************************************
 * File: ImageReferenceVector.cpp
 * Date: 10.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageReferenceVector.h"

const char* ImageReferenceVector::Serialization_Group = "ImageReferences";
const char* ImageReferenceVector::Serialization_Element = "ImageReference";

ImageReferenceVector::pointer_type ImageReferenceVector::selectByFilePath(QString filePath) const
{
	return selectFirst([filePath](auto ref) { return *ref == filePath; });
}

ImageReferenceVector::pointer_type ImageReferenceVector::selectByFileName(QString fileName) const
{
	return selectFirst([fileName](auto ref) { return ref->getImageFileName().compare(fileName, Qt::CaseInsensitive) == 0; });
}

ImageReferenceVector::pointer_type ImageReferenceVector::getNeighbor(const ImageReference* imageReference, Neighbor neighbor) const
{
	auto imageRefs = sorted();

	// Get the next/previous image reference, wrapping around if necessary
	auto index = imageRefs.indexOf(imageReference);

	if (index != -1)
	{
		if (neighbor == Neighbor::Previous)
		{
			if (--index < 0)
				index = static_cast<int>(imageRefs.size()) - 1;
		}
		else if (neighbor == Neighbor::Next)
		{
			if (++index >= static_cast<int>(imageRefs.size()))
				index = 0;
		}

		return imageRefs[index];
	}
	else
		return nullptr;
}
