/******************************************************************************
 * File: ProjectSerializer.cpp
 * Date: 27.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ProjectSerializer.h"
#include "core/GrinderApplication.h"
#include "common/serialization/SerializationExceptions.h"
#include "common/serialization/SerializationContext.h"
#include "common/serialization/DeserializationContext.h"
#include "project/Project.h"
#include "util/FileUtils.h"

ProjectSerializer::ProjectSerializer(Project* project) :
	_project{project}
{
	if (!project)
		throw std::invalid_argument{_EXCPT("project may not be null")};
}

SettingsContainer ProjectSerializer::serializeProject() const
{
	if (_project)
	{
		SerializationContext ctx{Project::Serialization_Group, SerializationContext::Mode::ProjectSerialization};
		_project->serialize(ctx);
		return ctx.settings(true);
	}
	else
		throw SerializationException{_EXCPT("No project to serialize")};
}

void ProjectSerializer::deserializeProject(const SettingsContainer& settings, QDir projectDir)
{
	if (_project)
	{		
		DeserializationContext ctx{settings, SerializationContext::Mode::ProjectSerialization};

		// Add all image search paths to the context
		ctx.addImageSearchPaths(FileUtils::getDirectoryList(projectDir.absolutePath()));

		for (const auto& path : grinder()->settings().imageOptions().additionalSearchPaths)
			ctx.addImageSearchPaths(FileUtils::getDirectoryList(path.trimmed()));

		_project->deserialize(ctx);
	}
	else
		throw SerializationException{_EXCPT("No project to deserialize")};
}
