/******************************************************************************
 * File: DragInfo.cpp
 * Date: 05.5.2018
 *****************************************************************************/

#include "Grinder.h"
#include "DragInfo.h"

DragInfo::DragInfo(QObject* dragSource, QString mimeType, QString image, QSize imageSize) :
	_dragSource{dragSource}, _dragMimeType{mimeType}
{
	if (!dragSource)
		throw std::invalid_argument{_EXCPT("dragSource may not be null")};

	if (mimeType.isEmpty())
		throw std::invalid_argument{_EXCPT("mimeType may not be empty")};

	_dragImage = QImage{image}.scaled(imageSize, Qt::KeepAspectRatio);
}

void DragInfo::initiateDrag(QPoint pos, QVariant dragData)
{
	_dragInitiated = true;
	_dragStartPosition = pos;
	_dragData = dragData;
}

void DragInfo::processInitiatedDrag(QPoint pos)
{
	if (_dragInitiated)
	{
		if ((pos - _dragStartPosition).manhattanLength() >= QApplication::startDragDistance())
			drag();
	}
}

void DragInfo::drag()
{
	QMimeData* mimeData = new QMimeData{};
	mimeData->setData(_dragMimeType, _dragData.toByteArray());

	QDrag* drag = new QDrag{_dragSource};
	drag->setMimeData(mimeData);
	drag->setPixmap(QPixmap::fromImage(_dragImage));
	drag->exec(_dropAction, _dropAction);

	_dragInitiated = false;
}
