/******************************************************************************
 * File: StyleSheet.cpp
 * Date: 02.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "StyleSheet.h"
#include "core/GrinderApplication.h"

QString StyleSheet::loadStyleSheet(QString name)
{
	QFile file(":/css/" + name);

	if (file.open(QIODevice::ReadOnly|QIODevice::Text))
	{
		QTextStream	stream{&file};
		QString sheet = stream.readAll();

		replacePlaceholders(sheet);
		removeConditionalLines(sheet);

		return sheet;
	}
	else
		return "";
}

void StyleSheet::replacePlaceholders(QString& styleSheet)
{
	QColor clrBorder = QPalette{}.color(QPalette::Dark);
	QColor clrBase = QPalette{}.color(QPalette::Base);
	QColor clrLightBg = QPalette{}.color(QPalette::Button).lighter(104);
	QColor clrLightGray = QPalette{}.color(QPalette::Dark).lighter(120);
	QColor clrHighlight = QPalette{}.color(QPalette::Highlight);

	replaceColorPlaceholder(styleSheet, "%BORDER%", clrBorder);
	replaceColorPlaceholder(styleSheet, "%BASE%", clrBase);
	replaceColorPlaceholder(styleSheet, "%LIGHTBG%", clrLightBg);
	replaceColorPlaceholder(styleSheet, "%LIGHTGRAY%", clrLightGray);
	replaceColorPlaceholder(styleSheet, "%HIGHLIGHT%", clrHighlight);
}

void StyleSheet::replaceColorPlaceholder(QString& styleSheet, QString name, QColor color)
{
	styleSheet.replace(name, color.name());
}

void StyleSheet::removeConditionalLines(QString& styleSheet)
{
	// Remove OS-specific lines
	auto sheetLines = styleSheet.split("\n");

	for (auto i = sheetLines.count() - 1; i >= 0; --i)
	{
		QRegExp re{".*/\\*\\*(.*)\\*\\*/.*"};

		if (re.indexIn(sheetLines[i]) != -1)	// The line contains conditional statements
		{
			auto conditions = re.cap(1).split(',', QString::SkipEmptyParts);

			for (int j = 0; j < conditions.count(); ++j)
				conditions[j] = conditions[j].trimmed();

			if (!checkConditions(conditions))
				sheetLines.removeAt(i);
		}
	}

	styleSheet = sheetLines.join("\n");
}

bool StyleSheet::checkConditions(const QStringList& conditions)
{
	bool allConditionsMet = true;

	if (!conditions.isEmpty())
	{
		if (conditions.contains("!win32", Qt::CaseInsensitive) && GrinderApplication::operatingSystem() != GrinderApplication::OperatingSystem::Windows)
			allConditionsMet = false;

		if (conditions.contains("!linux", Qt::CaseInsensitive) && GrinderApplication::operatingSystem() != GrinderApplication::OperatingSystem::Linux)
			allConditionsMet = false;

		if (conditions.contains("!macosx", Qt::CaseInsensitive) && GrinderApplication::operatingSystem() != GrinderApplication::OperatingSystem::MacOSX)
			allConditionsMet = false;
	}

	return allConditionsMet;
}
