/******************************************************************************
 * File: OptionsDialog.cpp
 * Date: 02.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "OptionsDialog.h"
#include "ui_OptionsDialog.h"
#include "core/GrinderApplication.h"

OptionsDialog::OptionsDialog(QWidget *parent) : QDialog(parent, Qt::Dialog|Qt::WindowTitleHint|Qt::WindowCloseButtonHint),
	ui{new Ui::OptionsDialog}
{
	_generalOptions = grinder()->settings().generalOptions();
	_startupOptions = grinder()->settings().startupOptions();
	_imageOptions = grinder()->settings().imageOptions();
	_imageEditorOptions = grinder()->settings().imageEditorOptions();

	setupUi();
}

OptionsDialog::~OptionsDialog()
{
	delete ui;
}

void OptionsDialog::accept()
{
	applyGeneralOptions(true);
	applyStartupOptions(true);
	applyImageOptions(true);
	applyImageEditorOptions(true);

	grinder()->settings().generalOptions() = _generalOptions;
	grinder()->settings().startupOptions() = _startupOptions;
	grinder()->settings().imageOptions() = _imageOptions;
	grinder()->settings().imageEditorOptions() = _imageEditorOptions;
	grinder()->settings().saveSettings();

	QDialog::accept();
}

void grndr::OptionsDialog::setupUi()
{
	ui->setupUi(this);

	applyGeneralOptions(false);
	applyStartupOptions(false);
	applyImageOptions(false);
	applyImageEditorOptions(false);
}

void OptionsDialog::applyGeneralOptions(bool save)
{
	if (save)
		_generalOptions.askOnUnsavedChanges = ui->chkAskOnUnsavedChanges->isChecked();
	else
		ui->chkAskOnUnsavedChanges->setChecked(_generalOptions.askOnUnsavedChanges);
}

void OptionsDialog::applyStartupOptions(bool save)
{
	if (save)
		_startupOptions.loadLastProject = ui->chkLoadLastProject->isChecked();
	else
		ui->chkLoadLastProject->setChecked(_startupOptions.loadLastProject);
}

void OptionsDialog::applyImageOptions(bool save)
{
	if (save)
	{
		auto paths = ui->txtAddSearchPaths->text().replace(',', ';').split(";", QString::SkipEmptyParts);

		for (auto& path : paths)
			path = path.trimmed();

		_imageOptions.additionalSearchPaths = paths;
	}
	else
		ui->txtAddSearchPaths->setText(_imageOptions.additionalSearchPaths.join("; "));
}

void OptionsDialog::applyImageEditorOptions(bool save)
{
	if (save)
	{
		_imageEditorOptions.groupIntoTabs = ui->chkGroupIntoTabs->isChecked();
		_imageEditorOptions.startUndocked = ui->chkStartUndocked->isChecked();
		_imageEditorOptions.autoFitToWindow = ui->chkAutoFitToWindow->isChecked();
	}
	else
	{
		ui->chkGroupIntoTabs->setChecked(_imageEditorOptions.groupIntoTabs);
		ui->chkStartUndocked->setChecked(_imageEditorOptions.startUndocked);
		ui->chkAutoFitToWindow->setChecked(_imageEditorOptions.autoFitToWindow);
	}
}

void OptionsDialog::on_chkGroupIntoTabs_toggled(bool checked)
{
	ui->chkStartUndocked->setEnabled(!checked);

	if (checked)
		ui->chkStartUndocked->setChecked(false);
}

void OptionsDialog::on_chkStartUndocked_toggled(bool checked)
{
	ui->chkGroupIntoTabs->setEnabled(!checked);

	if (checked)
		ui->chkGroupIntoTabs->setChecked(false);
}
