/******************************************************************************
 * File: GraphConnectionNode.h
 * Date: 25.1.2018
 *****************************************************************************/

#ifndef GRAPHCONNECTIONNODE_H
#define GRAPHCONNECTIONNODE_H

#include <memory>

#include "GraphConnectionBase.h"

namespace grndr
{
	class GraphPortNode;
	class Connection;

	class GraphConnectionNode : public GraphConnectionBase
	{
		Q_OBJECT

	public:
		GraphConnectionNode(GraphScene* scene, const std::shared_ptr<Connection>& con, QGraphicsItem* parent = nullptr);

	public:		
		GraphPortNode* sourcePortNode() { return _sourcePortNode(); }
		const GraphPortNode* sourcePortNode() const { return _sourcePortNode(); }
		GraphPortNode* destPortNode() { return _destPortNode(); }
		const GraphPortNode* destPortNode() const { return _destPortNode(); }

		std::weak_ptr<Connection>& connection() { return _connection; }
		const std::weak_ptr<Connection>& connection() const { return _connection; }

	public:
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget) override;

	protected:
		virtual QPointF calcSourcePosition() const override;
		virtual QPointF calcDestPosition() const override;

	private:
		void updateToolTip();
		void connectPortSignals();

		QColor getPortColor(const GraphPortNode* node) const;

	private:
		GraphPortNode* _sourcePortNode() const;
		GraphPortNode* _destPortNode() const;

	private slots:
		void connectedBlockMoved(VisualNode* node);

	private:
		std::weak_ptr<Connection> _connection;
	};
}

#endif
