/******************************************************************************
 * File: GraphStyle.h
 * Date: 20.1.2018
 *****************************************************************************/

#ifndef GRAPHSTYLE_H
#define GRAPHSTYLE_H

#include <QColor>
#include <QFont>
#include <QSizeF>
#include <QMarginsF>
#include <QPalette>

#include "ui/visscene/VisualSceneStyle.h"
#include "pipeline/BlockCategory.h"

namespace grndr
{
	class GraphScene;

	class GraphStyle : public VisualSceneStyle
	{
	public:
		struct LayoutStyle
		{
			QSizeF layoutMargins{100.0, 50.0};
		};

		struct NodeStyle
		{
			QFont font{};

			QColor darkBackgroundColor{100, 100, 100};
			QColor lightBackgroundColor{200, 200, 200};
			QColor borderColor{0, 0, 0};
			float borderRadius{3.0f};

			QColor selectionColor{QPalette{}.highlight().color().lighter()};
			QColor selectionColorInactive{QPalette{}.color(QPalette::Inactive, QPalette::Highlight).darker()};
			float selectionMargin{6.0f};
			float selectionOpacity{0.6f};

			NodeStyle();
		};

		struct BlockNodeStyle : public NodeStyle
		{
			QSizeF minimumSize{150.0, 60.0};

			QFont nameFont{font};
			QFont typeFont{font};
			QColor textColor{0, 0, 0};
			QMarginsF textMargins{20.0, 3.0, 20.0, 3.0};

			float borderWidth{3.0};

			QMarginsF portMargins{6.0, 6.0, 6.0, 6.0};

			QColor getBlockCategoryColor(BlockCategory category) const;

			BlockNodeStyle();
		};

		struct PortNodeStyle : public NodeStyle
		{
			float outerSize{15.0};
			float innerSize {7.5};

			QFont nameFont{font};
			QColor textColor{0, 0, 0};
			QMarginsF textMargins{5.0, 0.0, 5.0, 0.0};

			float borderWidth{1.0};
		};

		struct ConnectionNodeStyle : public NodeStyle
		{
			float lineWidth{5.0f};

			float blueprintOpacity{0.7f};
			QColor blueprintPendingColor{50, 150, 255};
			QColor blueprintValidColor{50, 255, 150};
			QColor blueprintInvalidColor{255, 50, 50};			
		};

		struct ConnectionMessageStyle : public ConnectionNodeStyle
		{
			float messageOpacity{0.8f};
			QFont messageFont{font};
			QColor messageColor{255, 255, 255};
			QMarginsF messageMargins{7.5, 7.5, 7.5, 7.5};
		};

	public:
		GraphStyle();

	public:		
		const LayoutStyle& getLayoutStyle() const { return _layoutStyle; }

		const BlockNodeStyle& getBlockNodeStyle() const { return _blockNodeStyle; }
		const PortNodeStyle& getPortNodeStyle() const { return _portNodeStyle; }
		const ConnectionNodeStyle& getConnectionNodeStyle() const { return _connectionNodeStyle; }
		const ConnectionMessageStyle& getConnectionMessageStyle() const { return _connectionMessageStyle; }

	private:		
		LayoutStyle _layoutStyle;

		BlockNodeStyle _blockNodeStyle;
		PortNodeStyle _portNodeStyle;
		ConnectionNodeStyle _connectionNodeStyle;
		ConnectionMessageStyle _connectionMessageStyle;
	};
}

#endif
