/******************************************************************************
 * File: BaseImageTagsListWidget.cpp
 * Date: 18.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "BaseImageTagsListWidget.h"
#include "image/ImageTags.h"

BaseImageTagsListWidget::BaseImageTagsListWidget(QWidget* parent) : MetaWidget(parent)
{	
	// Listen for item selections in order to update the actions
	connect(this, &BaseImageTagsListWidget::itemSelectionChanged, this, &BaseImageTagsListWidget::updateActions);

	updateActions();
}

void BaseImageTagsListWidget::assignImageTags(ImageTags* imageTags)
{
	// Disconnect any previously connected signals from the image tags
	if (_imageTags)
		disconnect(_imageTags, nullptr, this, nullptr);

	_imageTags = imageTags;
	populateList(_imageTags ? _imageTags->tags() : ImageTagVector{}, false);

	if (_imageTags)
	{
		// Listen for added/removed tags
		connect(_imageTags, &ImageTags::tagAdded, this, &BaseImageTagsListWidget::imageTagAdded);
		connect(_imageTags, &ImageTags::tagRemoved, this, &BaseImageTagsListWidget::imageTagRemoved);
		connect(_imageTags, &ImageTags::tagModified, this, &BaseImageTagsListWidget::imageTagModified);
	}
}

void BaseImageTagsListWidget::imageTagAdded(const std::shared_ptr<ImageTag>& imageTag)
{
	addObject(imageTag.get());
}

void BaseImageTagsListWidget::imageTagRemoved(const std::shared_ptr<ImageTag>& imageTag)
{
	removeObject(imageTag.get());
}

void BaseImageTagsListWidget::imageTagModified(const std::shared_ptr<ImageTag>& imageTag)
{
	updateObject(imageTag.get());
}
