/******************************************************************************
 * File: DraftItemNode.h
 * Date: 21.3.2018
 *****************************************************************************/

#ifndef DRAFTITEMNODE_H
#define DRAFTITEMNODE_H

#include "LayerItemNode.h"
#include "ImageEditorStyle.h"
#include "InPlaceEditor.h"

namespace grndr
{
	class DraftItemNode : public LayerItemNode
	{
		Q_OBJECT

	public:
		DraftItemNode(ImageEditorScene* scene, const std::shared_ptr<DraftItem>& item, QGraphicsItem* parent = nullptr);

	public:
		void initDraftItemNode();

	public:
		std::weak_ptr<DraftItem>& draftItem() { return _draftItem; }
		const std::weak_ptr<DraftItem>& draftItem() const { return _draftItem; }

	public slots:
		virtual void updateNode() override;

		virtual void updateZOrder() override;

	protected:
		virtual std::unique_ptr<InPlaceEditor> createInPlaceEditor() { return std::unique_ptr<InPlaceEditor>{}; }

	protected:
		virtual QVariant itemChange(GraphicsItemChange change, const QVariant& value) override;

		virtual void dragEnterEvent(QGraphicsSceneDragDropEvent* event) override;
		virtual void dragMoveEvent(QGraphicsSceneDragDropEvent* event) override;
		virtual void dropEvent(QGraphicsSceneDragDropEvent* event) override;

	protected:
		virtual QRect getBoundingRect() const = 0;
		virtual void updateGeometry() override;

		virtual std::vector<QAction*> getNodeActions(QMenu& menu) const override { Q_UNUSED(menu); return {nullptr, _deleteAction}; }
		virtual std::vector<QAction*> getNodesActions(QMenu& menu) const override { Q_UNUSED(menu); return {nullptr, _deleteAction}; }

	protected slots:
		void deleteNode();

	protected:
		std::weak_ptr<DraftItem> _draftItem;

		const ImageEditorStyle::ItemNodeStyle& _draftItemStyle;

		std::unique_ptr<InPlaceEditor> _inPlaceEditor;

	private:
		QAction* _deleteAction;
	};
}

#endif
