/******************************************************************************
 * File: ImageEditorNodeFactory.cpp
 * Date: 21.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "DraftItemNodeFactory.h"
#include "ImageEditorScene.h"
#include "image/ImageExceptions.h"
#include "DraftItemNode.h"

#include "draftitems/LineDraftItemNode.h"
#include "draftitems/BoxDraftItemNode.h"
#include "draftitems/PixelsDraftItemNode.h"

DraftItemNodeFactory::DraftItemNodeFactory(ImageEditorScene* scene) : VisualNodeFactory(scene)
{
	registerStandardDraftItems();
}

DraftItemNode* DraftItemNodeFactory::createDefaultNode(const std::shared_ptr<DraftItem>& item) const
{
	// Each draft item must be explicitly registered, so there is no "default" node type
	Q_UNUSED(item);
	throw ImageEditorException{nullptr, _EXCPT("Tried to create a draft item node of an unknown type")};
}

void DraftItemNodeFactory::registerStandardDraftItems()
{
	REGISTER_NODEFACTORY_TYPE(LineDraftItemNode);
	REGISTER_NODEFACTORY_TYPE(BoxDraftItemNode);
	REGISTER_NODEFACTORY_TYPE(PixelsDraftItemNode);
}
