/******************************************************************************
 * File: EditableImageTagsListWidget.cpp
 * Date: 01.5.2018
 *****************************************************************************/

#include "Grinder.h"
#include "EditableImageTagsListWidget.h"
#include "image/ImageTags.h"
#include "ui/image/ImageTagDialog.h"
#include "util/UIUtils.h"
#include "util/StringUtils.h"
#include "res/Resources.h"

EditableImageTagsListWidget::EditableImageTagsListWidget(QWidget* parent) : BaseImageTagsListWidget(parent)
{
	// Create tags actions
	_newImageTagAction = UIUtils::createAction(this, "&New tag", FILE_ICON_ADD, SLOT(newImageTag()), "Add a new tag", "Ins", Qt::WindowShortcut);
	_editImageTagAction = UIUtils::createAction(this, "&Edit tag", FILE_ICON_EDIT, SLOT(editImageTag()), "Edit the selected tag", "F2", Qt::WindowShortcut);
	_removeImageTagAction = UIUtils::createAction(this, "&Remove tag", FILE_ICON_DELETE, SLOT(removeImageTag()), "Remove the selected tag", "Del", Qt::WindowShortcut);
	_removeAllImageTagsAction = UIUtils::createAction(this, "Remove all tags", "", SLOT(removeAllImageTags()), "Remove all tags");

	updateActions();
}

void EditableImageTagsListWidget::assignImageTags(ImageTags* imageTags)
{
	if (!imageTags)
		throw std::invalid_argument{_EXCPT("imageTags may not be null")};

	BaseImageTagsListWidget::assignImageTags(imageTags);
}

std::unique_ptr<QMenu> EditableImageTagsListWidget::createContextMenu() const
{
	auto menu = widget_type::createContextMenu();
	menu->setDefaultAction(_editImageTagAction);
	return menu;
}

std::vector<QAction*> EditableImageTagsListWidget::getActions(MetaWidget::AddActionsMode mode) const
{
	std::vector<QAction*> actions;

	actions.push_back(_editImageTagAction);
	actions.push_back(nullptr);
	actions.push_back(_newImageTagAction);
	actions.push_back(_removeImageTagAction);

	if (mode == AddActionsMode::ContextMenu)
	{
		actions.push_back(nullptr);
		actions.push_back(_removeAllImageTagsAction);
	}

	return actions;
}

void EditableImageTagsListWidget::mouseDoubleClickEvent(QMouseEvent* event)
{
	ImageTagsObjectListWidget::mouseDoubleClickEvent(event);
	editImageTag();
}

void EditableImageTagsListWidget::updateActions()
{
	bool imageTagSelected = (currentObjectItem() != nullptr);

	_editImageTagAction->setEnabled(imageTagSelected);
	_removeImageTagAction->setEnabled(imageTagSelected);
	_removeAllImageTagsAction->setEnabled(count() > 0);
}

void EditableImageTagsListWidget::newImageTag()
{
	QString newTagName = StringUtils::generateUniqueItemName(_imageTags->tags(), "New tag", &ImageTag::getName);
	auto newTag = std::make_shared<ImageTag>(_imageTags, newTagName, QColor{Qt::white});

	ImageTagDialog dlg{newTag.get(), true, this};

	if (dlg.exec() == QDialog::Accepted)
		_imageTags->addTag(newTag);
}

void EditableImageTagsListWidget::editImageTag()
{
	if (auto imageTagSelected = currentObjectItem())
	{
		ImageTagDialog dlg{imageTagSelected->object(), false, this};

		if (dlg.exec() == QDialog::Accepted)
			_imageTags->updateTag(imageTagSelected->object());
	}
}

void EditableImageTagsListWidget::removeImageTag()
{
	if (auto imageTagSelected = currentObjectItem())
		_imageTags->removeTag(imageTagSelected->object());
}

void EditableImageTagsListWidget::removeAllImageTags()
{
	_imageTags->removeAllTags();
}
