/******************************************************************************
 * File: ImageEditor.cpp
 * Date: 27.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageEditor.h"
#include "core/GrinderApplication.h"

const char* ImageEditor::Serialization_Value_Block = "Block";

ImageEditor::ImageEditor() :
	_editorController{this}, _editorEnvironment{&_editorController}, _editorTools{this}, _editorDockWidget{new ImageEditorDockWidget{}}, _editorWidget{new ImageEditorWidget{this, _editorDockWidget}}
{
	// Assign the editor widget to the dock widget
	_editorDockWidget->setWidget(_editorWidget);
}

void ImageEditor::setImageBuild(const std::shared_ptr<ImageBuild>& imageBuild)
{
	_editorWidget->setImageBuild(imageBuild);
	_editorDockWidget->updateDockName(imageBuild->block());
}

void ImageEditor::serialize(SerializationContext& ctx) const
{
	// Serialize environment
	ctx.beginGroup(ImageEditorEnvironment::Serialization_Group);
	_editorEnvironment.serialize(ctx);
	ctx.endGroup();

	// Serialize tools
	ctx.beginGroup(ImageEditorToolList::Serialization_Group, true);
	_editorTools.serialize(ctx);
	ctx.endGroup();

	// Serialize editor widget
	ctx.beginGroup(ImageEditorWidget::Serialization_Group);
	_editorWidget->serialize(ctx);
	ctx.endGroup();
}

void ImageEditor::deserialize(DeserializationContext& ctx)
{
	// Deserialize environment
	if (ctx.beginGroup(ImageEditorEnvironment::Serialization_Group))
	{
		_editorEnvironment.deserialize(ctx);
		ctx.endGroup();
	}

	// Deserialize tools
	if (ctx.beginGroup(ImageEditorToolList::Serialization_Group))
	{
		_editorTools.deserialize(ctx);
		ctx.endGroup();
	}

	// Deserialize editor widget
	if (ctx.beginGroup(ImageEditorWidget::Serialization_Group))
	{
		_editorWidget->deserialize(ctx);
		ctx.endGroup();
	}
}
