/******************************************************************************
 * File: ImageEditorDockWidget.cpp
 * Date: 13.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageEditorDockWidget.h"
#include "ImageEditorView.h"
#include "core/GrinderApplication.h"
#include "ui/StyleSheet.h"
#include "res/Resources.h"

ImageEditorDockWidget::ImageEditorDockWidget(QWidget *parent) : QDockWidget(parent)
{
	// Some basic setup
	setAttribute(Qt::WA_DeleteOnClose);
	setFont(GrinderApplication::boldFont(this));
	setFeatures(QDockWidget::AllDockWidgetFeatures);

	setStyleSheet(StyleSheet::loadStyleSheet(FILE_STYLESHEET_IMAGEEDITORDOCKWIDGET));

	connect(this, &QDockWidget::visibilityChanged, this, &ImageEditorDockWidget::dockShown);
}

void ImageEditorDockWidget::closeEvent(QCloseEvent* event)
{
	emit dockClosed(this);
	QDockWidget::closeEvent(event);
}

void ImageEditorDockWidget::dockShown(bool shown)
{
	if (shown)
	{
		if (auto editorView= findChild<ImageEditorView*>())
			editorView->setFocus();
	}
}

void ImageEditorDockWidget::updateDockName(const Block* block)
{
	setObjectName("ImageEditor@" + block->getFormattedName());

	// Include the names of all used image references in the dock title
	QStringList imageNames;

	for (const auto& imageRef : block->assembleInputImageReferences(grinder()->projectController().activeImageReference()))
		imageNames << imageRef->getImageFileName();

	setWindowTitle(block->getFormattedName() + (!imageNames.isEmpty() ? QString{" (%1)"}.arg(imageNames.join(", ")) : ""));
}
