/******************************************************************************
 * File: ImageEditorEnvironment.cpp
 * Date: 26.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageEditorEnvironment.h"

const char* ImageEditorEnvironment::Serialization_Group = "Environment";

const char* ImageEditorEnvironment::Serialization_Value_PrimaryColor = "PrimaryColor";
const char* ImageEditorEnvironment::Serialization_Value_ShowDirectionArrows = "ShowDirectionArrows";
const char* ImageEditorEnvironment::Serialization_Value_ShowTags = "ShowTags";

ImageEditorEnvironment::ImageEditorEnvironment(ImageEditorController* controller) :
	_editorController{controller}
{
	if (!controller)
		throw std::invalid_argument{_EXCPT("controller may not be null")};
}

void ImageEditorEnvironment::setPrimaryColor(QColor color)
{
	if (color != _primaryColor)
	{
		_primaryColor = color;
		emit primaryColorChanged(color);
	}
}

void ImageEditorEnvironment::setShowDirectionArrows(bool show)
{
	if (show != _showDirectionArrows)
	{
		_showDirectionArrows = show;
		emit showDirectionArrowsChanged(show);
	}
}

void ImageEditorEnvironment::setShowTags(bool show)
{
	if (show != _showTags)
	{
		_showTags = show;
		emit showTagsChanged(show);
	}
}

void ImageEditorEnvironment::serialize(SerializationContext& ctx) const
{
	// Serialize values
	ctx.settings()(Serialization_Value_PrimaryColor) = _primaryColor.name();
	ctx.settings()(Serialization_Value_ShowDirectionArrows) = _showDirectionArrows;
	ctx.settings()(Serialization_Value_ShowTags) = _showTags;
}

void ImageEditorEnvironment::deserialize(DeserializationContext& ctx)
{
	// Deserialize values
	setPrimaryColor(QColor{ctx.settings()(Serialization_Value_PrimaryColor).toString()});
	setShowDirectionArrows(ctx.settings()(Serialization_Value_ShowDirectionArrows, true).toBool());
	setShowTags(ctx.settings()(Serialization_Value_ShowTags, true).toBool());
}
