/******************************************************************************
 * File: ImageEditorManager.h
 * Date: 13.3.2018
 *****************************************************************************/

#ifndef IMAGEEDITORMANAGER_H
#define IMAGEEDITORMANAGER_H

#include <memory>

#include "ImageEditor.h"

namespace grndr
{
	class ProjectController;
	class PipelineManager;
	class Pipeline;
	class ImageBuild;

	class ImageEditorManager : public QObject
	{
		Q_OBJECT

	public:
		static const char* Serialization_Group;
		static const char* Serialization_Element;

	public:
		ImageEditorManager(Project* project, ProjectController* projectController, PipelineManager* pipelineManager);

	public:
		void showEditor(const Block* block, const std::shared_ptr<ImageBuild>& imageBuild);
		void closeEditor(const Block* block);

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(DeserializationContext& ctx);

	signals:
		void editorShown(ImageEditor*);
		void editorClosed(ImageEditor*);

	private:
		ImageEditor* createEditor(const Block* block);
		void removeEditor(const Block* block);

		ImageEditor* findEditor(const Block* block) const;
		SettingsContainer* findEditorSettings(const Block* block) const;

		void loadEditorSettings(const Block* block, ImageEditor* editor) const;
		void saveEditorSettings(const Block* block, const ImageEditor* editor);

	private slots:
		void dockClosed(ImageEditorDockWidget* dock);

		void pipelineRemoved(const std::shared_ptr<Pipeline>& pipeline);
		void blockRemoved(const std::shared_ptr<Block>& block);
		void imageReferenceRemoved(const std::shared_ptr<ImageReference>& imageReference);

		void projectSaving(QString projectFile);

		void updateDockNames();		

	private:
		std::map<const Block*, std::unique_ptr<ImageEditor>> _editors;
		std::map<const Block*, std::unique_ptr<SettingsContainer>> _editorSettings;
	};
}

#endif
