/******************************************************************************
 * File: ImageEditorScene.h
 * Date: 15.3.2018
 *****************************************************************************/

#ifndef IMAGEEDITORSCENE_H
#define IMAGEEDITORSCENE_H

#include "ui/visscene/VisualScene.h"
#include "ImageEditorView.h"
#include "ImageEditorComponent.h"
#include "DraftItemNodeFactory.h"
#include "ImageEditorTool.h"
#include "image/ImageBuild.h"

namespace grndr
{
	class ImageEditor;
	class LayerItemNode;
	class LayerBackgroundNode;
	class LayerPixelsNode;

	class ImageEditorScene : public VisualScene<ImageEditorView>, public ImageEditorComponent
	{
		Q_OBJECT

	public:
		ImageEditorScene(ImageEditor* imageEditor, ImageBuild* imageBuild, ImageEditorView* view);

	public:
		void buildScene();

		void refreshImage(bool centerOnImage = false);
		void fitViewToImage();

		void createDraftItemNode(const std::shared_ptr<DraftItem>& item);
		void removeDraftItemNode(const std::shared_ptr<DraftItem>& item);

		void createLayerBackgroundNode(const std::shared_ptr<Layer>& layer);
		void removeLayerBackgroundNode(const std::shared_ptr<Layer>& layer);
		void createLayerPixelsNode(const std::shared_ptr<Layer>& layer);		
		void removeLayerPixelsNode(const std::shared_ptr<Layer>& layer);

		void updateLayerItemsOrder();
		void updateLayerItemsVisibility(const Layer* layer);
		void updateLayerItemsAlpha(const Layer* layer);

		void removeNode(QGraphicsItem* node) { if (node) delete node; }

	public:
		template<typename ValType = LayerItemNode>
		std::vector<ValType*> findLayerItemNodes(const Layer* layer = nullptr) { return _findLayerItemNodes<ValType>(layer); }
		template<typename ValType = LayerItemNode>
		std::vector<const ValType*> findLayerItemNodes(const Layer* layer = nullptr) const { return _findLayerItemNodes<const ValType>(layer); }

		DraftItemNode* findDraftItemNode(const DraftItem* item) { return _findDraftItemNode(item); }
		const DraftItemNode* findDraftItemNode(const DraftItem* item) const { return _findDraftItemNode(item); }

		LayerBackgroundNode* findLayerBackgroundNode(const ImageReference* imageRef) { return _findLayerBackgroundNode(imageRef); }
		const LayerBackgroundNode* findLayerBackgroundNode(const ImageReference* imageRef) const { return _findLayerBackgroundNode(imageRef); }
		LayerPixelsNode* findLayerPixelsNode(const LayerPixels* layerPixels) { return _findLayerPixelsNode(layerPixels); }		
		const LayerPixelsNode* findLayerPixelsNode(const LayerPixels* layerPixels) const { return _findLayerPixelsNode(layerPixels); }

	public:
		ImageBuild* imageBuild() { return _imageBuild; }
		const ImageBuild* imageBuild() const { return _imageBuild; }

		const DraftItemNodeFactory& nodeFactory() const { return _nodeFactory; }

	protected:
		virtual void mousePressEvent(QGraphicsSceneMouseEvent* mouseEvent) override;
		virtual void mouseMoveEvent(QGraphicsSceneMouseEvent* mouseEvent) override;
		virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent* mouseEvent) override;

		virtual void keyPressEvent(QKeyEvent* keyEvent) override;
		virtual void keyReleaseEvent(QKeyEvent* keyEvent) override;

	private:
		void createBackgroundNode();

		ImageEditorTool::InputEventResult handleMouseEvent(QGraphicsSceneMouseEvent* mouseEvent, std::function<ImageEditorTool::InputEventResult(ImageEditorTool*, QGraphicsSceneMouseEvent*)> handler);
		ImageEditorTool::InputEventResult handleKeyEvent(QKeyEvent* keyEvent, std::function<ImageEditorTool::InputEventResult(ImageEditorTool*, QKeyEvent*)> handler);

	private:
		template<typename ValType>
		std::vector<ValType*> _findLayerItemNodes(const Layer* layer = nullptr) const;

		DraftItemNode* _findDraftItemNode(const DraftItem* item) const;

		LayerBackgroundNode* _findLayerBackgroundNode(const ImageReference* imageRef) const;
		LayerPixelsNode* _findLayerPixelsNode(const LayerPixels* layerPixels) const;

	private:
		ImageBuild* _imageBuild{nullptr};

		DraftItemNodeFactory _nodeFactory;

	private:
		QGraphicsPixmapItem* _imageNode{nullptr};
	};
}

#include "ImageEditorScene.impl.h"

#endif
