/******************************************************************************
 * File: ImageEditorTool.cpp
 * Date: 22.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageEditorTool.h"
#include "ImageEditor.h"

const char* ImageEditorTool::Serialization_Value_Type = "Type";

ImageEditorTool::ImageEditorTool(ImageEditor* imageEditor, QString name, QString icon, QString shortcut, QCursor cursor) : ImageEditorComponent(imageEditor),
	_name{name}, _icon{icon}, _shortcut{shortcut}, _cursor{cursor}
{
	if (!imageEditor)
		throw std::invalid_argument{_EXCPT("imageEditor may not be null")};
}

void ImageEditorTool::initImageEditorTool()
{
	createProperties();
}

void ImageEditorTool::toolActivated(ImageEditorTool* prevTool)
{
	Q_UNUSED(prevTool);

	if (_supportRightMouseButton && _imageEditor->controller().activeScene())
	{
		// If the tool supports the right mouse button, we need to disable the view's context menu
		_prevContextMenuPolicy = _imageEditor->controller().activeScene()->view()->contextMenuPolicy();
		_imageEditor->controller().activeScene()->view()->setContextMenuPolicy(Qt::NoContextMenu);
	}
}

void ImageEditorTool::toolDeactivated(ImageEditorTool* nextTool)
{
	Q_UNUSED(nextTool);

	if (_supportRightMouseButton && _imageEditor->controller().activeScene())
		_imageEditor->controller().activeScene()->view()->setContextMenuPolicy(_prevContextMenuPolicy);
}

void ImageEditorTool::serialize(SerializationContext& ctx) const
{
	PropertyObject::serialize(ctx);

	// Serialize values
	ctx.settings()(Serialization_Value_Type) = getToolType();
}

void ImageEditorTool::deserialize(DeserializationContext& ctx)
{
	PropertyObject::deserialize(ctx);
}
