/******************************************************************************
 * File: ImageEditorToolList.h
 * Date: 22.3.2018
 *****************************************************************************/

#ifndef IMAGEEDITORTOOLLIST_H
#define IMAGEEDITORTOOLLIST_H

#include "ImageEditorTool.h"
#include "ImageEditorComponent.h"

namespace grndr
{
	class ImageEditorToolList : public QObject, public ImageEditorComponent
	{
		Q_OBJECT

	public:
		static const char* Serialization_Group;
		static const char* Serialization_Element;

	public:
		ImageEditorToolList(ImageEditor* imageEditor);

	public:
		void assignUiComponents(QWidget* actionOwner);

	public:		
		ImageEditorTool* activeTool() { return _activeTool; }
		const ImageEditorTool* activeTool() const { return _activeTool; }
		void activateTool(QString toolType);
		void activateDefaultTool();

		std::vector<QAction*> getActions() const;

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(DeserializationContext& ctx);

	signals:
		void activeToolChanging(const ImageEditorTool*);
		void activeToolChanged(const ImageEditorTool*);

	private slots:
		void toolSelected();

	private:
		template<typename ToolType>
		std::unique_ptr<ToolType> createTool(ImageEditor* imageEditor);
		QAction* createAction(const ImageEditorTool* tool, QWidget* owner);

		void setActiveTool(QAction* toolAction);

	private:
		struct ToolEntry
		{
			std::unique_ptr<ImageEditorTool> tool;
			QAction* toolAction{nullptr};
		};

		ToolEntry* findTool(QString toolType);

	private:
		std::vector<ToolEntry> _tools;
		ImageEditorTool* _activeTool{nullptr};
	};
}

#include "ImageEditorToolList.impl.h"

#endif
