/******************************************************************************
 * File: ImageEditorWidget.h
 * Date: 13.3.2018
 *****************************************************************************/

#ifndef IMAGEEDITORWIDGET_H
#define IMAGEEDITORWIDGET_H

#include <QLabel>
#include <QMenu>
#include <memory>

#include "ImageEditorComponent.h"
#include "common/serialization/SerializationContext.h"
#include "common/serialization/DeserializationContext.h"

namespace Ui
{
	class ImageEditorWidget;
}

namespace grndr
{
	class ImageEditor;
	class ImageBuild;

	class ImageEditorWidget : public QWidget, public ImageEditorComponent
	{
		Q_OBJECT

	public:
		static const char* Serialization_Group;

		static const char* Serialization_Value_ColorPresets;

	public:
		ImageEditorWidget(ImageEditor* imageEditor, QWidget* parent = nullptr);
		virtual ~ImageEditorWidget();

	public:
		void setImageBuild(const std::shared_ptr<ImageBuild>& imageBuild);	

	public:
		void serialize(SerializationContext& ctx) const;
		void deserialize(DeserializationContext& ctx);

	protected:
		virtual void keyPressEvent(QKeyEvent* event) override;

	private:
		void setupUi();
		void setupImageControlBar();
		Ui::ImageEditorWidget *ui;

		void updatePrimaryColorLabel();

		QLabel* _imageSceneZoomLabel{nullptr};		

	private slots:		
		void on_splitterMid_splitterMoved(int pos, int index);
		void on_splitterLeft_splitterMoved(int pos, int index);
		void on_splitterLayersTags_splitterMoved(int pos, int index);

		void on_primaryColorWidget_customContextMenuRequested(const QPoint &pos);

	private slots:
		void updateActions();
		void updateSceneZoomLevel(qreal zoomLevel);

		void updatePrimaryColor();
		void primaryColorChanged(QColor color);
		void primaryColorSelected(QColor color, bool byUser);
		void colorPresetSelected(QColor color);
		void assignPrimaryColorToPreset();

		void copyImageBuild() const;
		void pasteImageBuild() const;
		void duplicateImageBuild() const;
		void duplicateImageBuildTo() const;		

		void editImageBuildTags();

	private:
		QAction* _newLayerAction{nullptr};

		QAction* _copyImageBuild{nullptr};
		QAction* _pasteImageBuild{nullptr};
		QAction* _duplicateImageBuild{nullptr};
		QAction* _duplicateImageBuildTo{nullptr};

		QAction* _editImageBuildTags{nullptr};

		QMenu _duplicateImageBuildMenu;
	};
}

#endif
