/******************************************************************************
 * File: ImageTagDialog.cpp
 * Date: 19.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageTagDialog.h"
#include "ui_ImageTagDialog.h"
#include "image/ImageTags.h"
#include "image/ImageExceptions.h"

ImageTagDialog::ImageTagDialog(ImageTag* imageTag, bool newTag, QWidget* parent) : QDialog(parent, Qt::Dialog|Qt::WindowTitleHint|Qt::WindowCloseButtonHint),
	ui{new Ui::ImageTagDialog}, _imageTag{imageTag}
{
	if (!imageTag)
		throw std::invalid_argument{_EXCPT("imageTag may not be null")};

	setupUi(newTag);
}

ImageTagDialog::~ImageTagDialog()
{
	delete ui;
}

void ImageTagDialog::accept()
{
	try {
		verifyImageTagProperties();
		applyImageTagProperties(true);

		QDialog::accept();
	} catch (ImageTagException& e) {
		QMessageBox::warning(this, "Tag error", GetExceptionMessage(e.what()));
	}
}

void ImageTagDialog::setupUi(bool newTag)
{
	ui->setupUi(this);

	setWindowTitle(newTag ? "Add tag" : "Edit tag");

	applyImageTagProperties(false);
}

void ImageTagDialog::applyImageTagProperties(bool save)
{
	if (save)
	{
		_imageTag->setName(ui->txtImageTagName->text().trimmed());
		_imageTag->setColor(ui->imageTagColorWidget->getColor());
	}
	else
	{
		ui->txtImageTagName->setText(_imageTag->getName());
		ui->imageTagColorWidget->setColor(_imageTag->getColor());
	}
}

void ImageTagDialog::on_txtImageTagName_textChanged(const QString& text)
{
	ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(!text.isEmpty());
}

void ImageTagDialog::verifyImageTagProperties() const
{
	QString tagName = ui->txtImageTagName->text().trimmed();

	if (auto imageTag = _imageTag->imageTags()->tags().selectByName(tagName))
	{
		if (imageTag.get() != _imageTag)
			throw ImageTagException{_imageTag, _EXCPT(QString{"A tag with the name '%1' already exists"}.arg(tagName))};
	}
}
