/******************************************************************************
 * File: InPlaceEditor.cpp
 * Date: 28.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "InPlaceEditor.h"
#include "DraftItemNode.h"
#include "ImageEditorScene.h"

InPlaceEditor::InPlaceEditor(ImageEditorScene* scene, DraftItemNode* draftItemNode) : ImageEditorNode(scene, draftItemNode),
	_draftItemNode{draftItemNode}
{
	if (!draftItemNode)
		throw std::invalid_argument{_EXCPT("draftItemNode may not be null")};

	setFlag(ItemHasNoContents);
	setFlag(ItemIsFocusable, false);
	setFlag(ItemIsSelectable, false);
}

void InPlaceEditor::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	Q_UNUSED(painter);
	Q_UNUSED(option);
	Q_UNUSED(widget);

	// Don't paint anything by default
}

QVariant InPlaceEditor::itemChange(QGraphicsItem::GraphicsItemChange change, const QVariant& value)
{
	if (change == QGraphicsItem::ItemVisibleHasChanged)
		updateEditor();

	return ImageEditorNode::itemChange(change, value);
}

InPlaceEditorDragHandle* InPlaceEditor::createDragHandle(InPlaceEditorDragHandle::AllowedDirections allowedDirections, QPointF position)
{
	auto dragHandle = new InPlaceEditorDragHandle{_scene, allowedDirections, this};
	dragHandle->setPos(position);

	// Listen for drag handle events
	connect(dragHandle, &InPlaceEditorDragHandle::handlePressed, this, &InPlaceEditor::handlePressed);
	connect(dragHandle, &InPlaceEditorDragHandle::handleMoved, this, &InPlaceEditor::handleMoved);
	connect(dragHandle, &InPlaceEditorDragHandle::handleReleased, this, &InPlaceEditor::handleReleased);

	return dragHandle;
}

void InPlaceEditor::dragHandleReleased(InPlaceEditorDragHandle* dragHandle)
{
	Q_UNUSED(dragHandle);

	// When a handle has been dragged, normalize the property values (e.g., to adjust for negative sizes)
	if (auto draftItem = _draftItemNode->draftItem().lock())	// Make sure that the underlying draft item still exists
		draftItem->normalizePropertyValues();
}

void InPlaceEditor::handlePressed()
{
	if (auto dragHandle = dynamic_cast<InPlaceEditorDragHandle*>(sender()))
		dragHandlePressed(dragHandle);
}

void InPlaceEditor::handleMoved(QPoint offset)
{
	if (auto dragHandle = dynamic_cast<InPlaceEditorDragHandle*>(sender()))
		dragHandleMoved(dragHandle, offset);
}

void InPlaceEditor::handleReleased()
{
	if (auto dragHandle = dynamic_cast<InPlaceEditorDragHandle*>(sender()))
		dragHandleReleased(dragHandle);
}
