/******************************************************************************
 * File: LayerBackgroundNode.cpp
 * Date: 15.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "LayerBackgroundNode.h"
#include "image/Layer.h"
#include "project/ImageReference.h"
#include "util/CVUtils.h"
#include "res/Resources.h"

LayerBackgroundNode::LayerBackgroundNode(ImageEditorScene* scene, Layer* layer, QGraphicsItem* parent) : LayerItemNode(scene, layer, parent),
	_imageNode{new QGraphicsPixmapItem{this}}
{
	setFlags(0);	// This item doesn't behave like an item at all

	// Set up the background node
	_imageNode->setPos(QPointF{0.0, 0.0});
	_imageNode->setShapeMode(QGraphicsPixmapItem::BoundingRectShape);
	_imageNode->setTransformationMode(Qt::SmoothTransformation);

	refreshImage();

	updateZOrder();
	updateVisibility();
	updateAlpha();

	updateNode();
}

void LayerBackgroundNode::refreshImage()
{
	bool resetPixmap = true;

	if (auto backgroundImage = _layer->backgroundImage())
	{
		try {
			auto imageData = backgroundImage->loadImage();
			auto image = CVUtils::matrixToPixmap(imageData);

			if (!image.isNull())
			{
				_imageNode->setPixmap(image);
				resetPixmap = false;
			}
		} catch (...) {
			// Just ignore any errors
		}
	}

	if (resetPixmap)
		_imageNode->setPixmap(QPixmap{FILE_ICON_EDITOR_NOIMAGE});	// If the image is invalid, show a question mark indicating an error

	updateNode();
}

void LayerBackgroundNode::updateZOrder()
{
	setZValue(_layer->getZOrder() - 0.5);	// Place the background just below the layer pixels
}

void LayerBackgroundNode::updateGeometry()
{
	_nodeRect = _nodeRectSelected = _imageNode->boundingRect();
	ImageEditorNode::updateGeometry();
}
