/******************************************************************************
 * File: LayersListWidget.h
 * Date: 17.3.2018
 *****************************************************************************/

#ifndef LAYERSLISTWIDGET_H
#define LAYERSLISTWIDGET_H

#include "ui/widgets/MetaWidget.h"
#include "ui/widgets/ActiveObjectListWidget.h"
#include "ImageEditorComponent.h"
#include "LayersListItem.h"

namespace grndr
{
	class Layer;
	class SliderValueWidget;

	using LayerObjectListWidget = ActiveObjectListWidget<Layer, LayersListItem>;

	class LayersListWidget : public MetaWidget<LayerObjectListWidget>, public ImageEditorComponent
	{
		Q_OBJECT

	public:
		LayersListWidget(QWidget* parent = nullptr);

	public:
		void assignUiComponents(ImageEditor* imageEditor);
		void setupUi(QAction* newLayerAction, QMenu* menu, ControlBar* controlBar);

	public:
		void swapLayers(int indexOld, int indexNew);

	protected:
		virtual std::vector<QAction*> getActions(AddActionsMode mode) const override;

	protected:
		virtual void switchToObjectItem(LayersListItem* item, bool selectItem = true) override;

	private slots:
		void switchLayer() { switchToObjectItem(currentObjectItem()); }
		void renameLayer() { editItem(currentItem()); }
		void newLayer();
		void moveLayerUp();
		void moveLayerDown();
		void convertPixelsToItems();
		void copyLayer();
		void pasteLayer();
		void cutLayer();
		void removeLayer();
		void removeAllLayers();

		void imageBuildSwitched(ImageBuild* imageBuild);

		void layerCheckChanged(QListWidgetItem* item) const;
		void layerOpacityChanged(int value) const;
		void layerSwitching(Layer* layer);
		void layerSwitched(Layer* layer);
		void layerRenamed(QWidget* editor) const;

		void selectedItemChanged();
		void updateActions();

	private:
		SliderValueWidget* _sliderValueWidget{nullptr};

		QAction* _renameLayerAction{nullptr};
		QAction* _newLayerAction{nullptr};
		QAction* _moveUpAction{nullptr};
		QAction* _moveDownAction{nullptr};
		QAction* _convertPixelsToItemsAction{nullptr};
		QAction* _copyLayer{nullptr};
		QAction* _pasteLayer{nullptr};
		QAction* _cutLayer{nullptr};
		QAction* _removeLayerAction{nullptr};
		QAction* _removeAllLayersAction{nullptr};
		QAction* _alphaAction{nullptr};
	};
}

#endif
