/******************************************************************************
 * File: BoxDraftItemNode.cpp
 * Date: 23.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "BoxDraftItemNode.h"
#include "ui/image/editors/RectangularInPlaceEditor.h"

const DraftItemType BoxDraftItemNode::type_value = DraftItemType::Box;

BoxDraftItemNode::BoxDraftItemNode(ImageEditorScene* scene, const std::shared_ptr<DraftItem>& item, QGraphicsItem* parent) : DraftItemNode(scene, item, parent)
{
	updateNode();
}

std::unique_ptr<grndr::InPlaceEditor> BoxDraftItemNode::createInPlaceEditor()
{
	if (auto draftItem = boxDraftItem())	// Make sure that the underlying draft item still exists
		return std::make_unique<RectangularInPlaceEditor>(_scene, this, draftItem->position(), draftItem->boxSize());
	else
		return nullptr;
}

QRect BoxDraftItemNode::getBoundingRect() const
{
	if (auto draftItem = boxDraftItem())	// Make sure that the underlying draft item still exists
	{
		int lineWidth = *draftItem->lineWidth();
		QSize size = *draftItem->boxSize() + QSize{lineWidth, lineWidth};

		return QRect{QPoint{std::lround(-lineWidth / 2.0), std::lround(-lineWidth / 2.0)}, size};
	}
	else
		return QRect{};
}
