/******************************************************************************
 * File: LineDraftItemNode.cpp
 * Date: 23.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "LineDraftItemNode.h"
#include "ui/image/editors/LinearInPlaceEditor.h"
#include "util/MathUtils.h"

const DraftItemType LineDraftItemNode::type_value = DraftItemType::Line;

LineDraftItemNode::LineDraftItemNode(ImageEditorScene* scene, const std::shared_ptr<DraftItem>& item, QGraphicsItem* parent) : DraftItemNode(scene, item, parent)
{
	updateNode();
}

std::unique_ptr<grndr::InPlaceEditor> LineDraftItemNode::createInPlaceEditor()
{
	if (auto draftItem = lineDraftItem())	// Make sure that the underlying draft item still exists
		return std::make_unique<LinearInPlaceEditor>(_scene, this, draftItem->position(), draftItem->endPosition());
	else
		return nullptr;
}

QRect LineDraftItemNode::getBoundingRect() const
{
	if (auto draftItem = lineDraftItem())	// Make sure that the underlying draft item still exists
	{
		QPoint position = *draftItem->position();
		QPoint endPosition = *draftItem->endPosition();
		auto width = std::abs(position.x() - endPosition.x());
		auto height = std::abs(position.y() - endPosition.y());

		return QRect{QPoint{0, 0}, QSize{width, height}};
	}
	else
		return QRect{};
}

QVariant LineDraftItemNode::itemChange(QGraphicsItem::GraphicsItemChange change, const QVariant& value)
{
	QVariant result;

	if (change == QGraphicsItem::ItemPositionHasChanged)
	{
		// Update the line draft item's end position when the item has been moved around
		if (auto draftItem = lineDraftItem())	// Make sure that the underlying draft item still exists
		{
			QPoint newPosition = value.toPointF().toPoint();
			QPoint oldPosition = *draftItem->position();
			auto movementDelta = newPosition - oldPosition;
			auto endPosition = *draftItem->endPosition() + movementDelta;

			draftItem->endPosition()->setValue(endPosition);
		}
	}

	return DraftItemNode::itemChange(change, value);
}
