/******************************************************************************
 * File: LinearInPlaceEditor.cpp
 * Date: 28.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "LinearInPlaceEditor.h"

LinearInPlaceEditor::LinearInPlaceEditor(ImageEditorScene* scene, DraftItemNode* draftItemNode, PointProperty* startPosition, PointProperty* endPosition) : InPlaceEditor(scene, draftItemNode),
	_startPosition{startPosition}, _endPosition{endPosition}
{
	if (!startPosition)
		throw std::invalid_argument{_EXCPT("startPosition may not be null")};

	if (!endPosition)
		throw std::invalid_argument{_EXCPT("endPosition may not be null")};

	_startDragHandle = createDragHandle(InPlaceEditorDragHandle::AllowedDirection::All);
	_endDragHandle = createDragHandle(InPlaceEditorDragHandle::AllowedDirection::All);
}

void LinearInPlaceEditor::updateEditor()
{
	updateDragHandlePositions();
}

void LinearInPlaceEditor::dragHandleMoved(InPlaceEditorDragHandle* dragHandle, QPoint offset)
{
	if (dragHandle == _startDragHandle)	// The start drag handle changes the start position of the line
		_startPosition->setValue(*_startPosition + offset);
	else if (dragHandle == _endDragHandle)	// The end drag handle changes the end position of the line
		_endPosition->setValue(*_endPosition + offset);
}

void LinearInPlaceEditor::updateDragHandlePositions()
{
	// Reposition the end drag handle; the start drag handle always stays at 0,0
	auto endPosition = *_endPosition - *_startPosition;
	_endDragHandle->setPos(endPosition);
}
