/******************************************************************************
 * File: EraserTool.cpp
 * Date: 21.5.2018
 *****************************************************************************/

#include "Grinder.h"
#include "EraserTool.h"
#include "ui/image/ImageEditor.h"
#include "util/ImageUtils.h"
#include "res/Resources.h"

const char* EraserTool::tool_type = "EraserTool";

EraserTool::EraserTool(ImageEditor* imageEditor) : PaintbrushTool(imageEditor, "Eraser", FILE_ICON_EDITOR_ERASER, "E", QCursor{QPixmap{FILE_CURSOR_EDITOR_ERASER}, 12, 12})
{
	_eraseByDefault = true;
}

void EraserTool::createProperties()
{
	PaintbrushTool::createProperties();

	// The eraser should have a default width equal to the cursor size
	penWidth()->setValue(_cursor.pixmap().width());
	penWidth()->setName("Eraser width");
	penWidth()->setDescription("The width of the eraser in (screen) pixels.");
}

VisualSceneInputHandler::InputEventResult EraserTool::rightMousePressed(const QGraphicsSceneMouseEvent* event)
{
	createRubberBand(event->scenePos());
	return InputEventResult::Process;
}

VisualSceneInputHandler::InputEventResult EraserTool::rightMouseMoved(const QGraphicsSceneMouseEvent* event)
{
	if (_rubberBandNode)
	{
		_rubberBandNode->move(event->scenePos());
		return InputEventResult::Process;
	}
	else
		return InputEventResult::Ignore;
}

VisualSceneInputHandler::InputEventResult EraserTool::rightMouseReleased(const QGraphicsSceneMouseEvent* event)
{
	Q_UNUSED(event);

	if (_rubberBandNode)
	{
		// Erase all pixels within the rubber band rectangle
		auto eraseRect = _rubberBandNode->getRect();
		std::vector<QPoint> points;

		for (int r = eraseRect.top(); r <= eraseRect.bottom(); ++r)
		{
			for (int c = eraseRect.left(); c <= eraseRect.right(); ++c)
				points.emplace_back(c, r);
		}

		_imageEditor->controller().paintPixels(nullptr, QColor{}, points);

		removeRubberBand();
		return InputEventResult::Process;
	}
	else
		return InputEventResult::Ignore;
}

void EraserTool::createRubberBand(QPointF pos)
{
	if (!_rubberBandNode)
	{
		if (auto scene = _imageEditor->controller().activeScene())
			_rubberBandNode = new RubberBandNode{scene, pos};
	}
}

void EraserTool::removeRubberBand()
{
	if (_rubberBandNode)
	{
		delete _rubberBandNode;
		_rubberBandNode = nullptr;
	}
}
