/******************************************************************************
 * File: ZoomTool.cpp
 * Date: 05.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ZoomTool.h"
#include "ui/image/ImageEditor.h"
#include "res/Resources.h"

const char* ZoomTool::tool_type = "ZoomTool";

ZoomTool::ZoomTool(ImageEditor* imageEditor) : ImageEditorTool(imageEditor, "Zoom", FILE_ICON_EDITOR_ZOOM, "Z", QCursor{QPixmap{FILE_CURSOR_EDITOR_ZOOM}, 7, 7})
{

}

VisualSceneInputHandler::InputEventResult ZoomTool::mousePressed(const QGraphicsSceneMouseEvent* event)
{
	createRubberBand(event->scenePos());
	return InputEventResult::Process;
}

VisualSceneInputHandler::InputEventResult ZoomTool::mouseMoved(const QGraphicsSceneMouseEvent* event)
{
	if (_rubberBandNode)
	{
		_rubberBandNode->move(event->scenePos());
		return InputEventResult::Process;
	}
	else
		return InputEventResult::Ignore;
}

VisualSceneInputHandler::InputEventResult ZoomTool::mouseReleased(const QGraphicsSceneMouseEvent* event)
{
	Q_UNUSED(event);

	if (_rubberBandNode)
	{
		// Zoom into the rubber band rectangle
		if (auto scene = _imageEditor->controller().activeScene())
			scene->view()->zoomToRect(_rubberBandNode->getRect());

		removeRubberBand();
		return InputEventResult::Process;
	}
	else
		return InputEventResult::Ignore;
}

void ZoomTool::createRubberBand(QPointF pos)
{
	if (!_rubberBandNode)
	{
		if (auto scene = _imageEditor->controller().activeScene())
			_rubberBandNode = new RubberBandNode{scene, pos};
	}
}

void ZoomTool::removeRubberBand()
{
	if (_rubberBandNode)
	{
		delete _rubberBandNode;
		_rubberBandNode = nullptr;

		if (auto scene = _imageEditor->controller().activeScene())
		{
			scene->view()->setDragMode(QGraphicsView::ScrollHandDrag);
			scene->view()->setDragMode(QGraphicsView::RubberBandDrag);
		}
	}
}
