/******************************************************************************
 * File: GrinderWindow.h
 * Date: 11.1.2018
 *****************************************************************************/

#ifndef GRINDERWINDOW_H
#define GRINDERWINDOW_H

#include <QMainWindow>
#include <memory>

#include "ui/image/ImageEditorManager.h"
#include "ui/graph/GraphWidget.h"

namespace Ui
{
	class GrinderWindow;
}

namespace grndr
{
	class Pipeline;

	class GrinderWindow : public QMainWindow
	{
		Q_OBJECT

	public:
		GrinderWindow(QWidget* parent = nullptr);
		~GrinderWindow();

	protected:
		virtual void closeEvent(QCloseEvent* event) override;

	private slots:
		void on_actNewProject_triggered();
		void on_actLoadProject_triggered();
		void on_actSaveProject_triggered();
		void on_actSaveProjectAs_triggered();

		void on_actExecuteLabel_triggered();
		void on_actExecuteNextImage_triggered();
		void on_actExecutePreviousImage_triggered();

		void on_actOptions_triggered();
		void on_actAbout_triggered();

	private slots:
		void pipelineSwitched(Pipeline* pipeline);
		void currentProjectFileChanged(QString fileName);

		void imageEditorShown(ImageEditor* editor);
		void imageEditorClosed(ImageEditor* editor);

		void updateCaption();
		void updateActions();
		void updateRecentProjects(QString fileName) { updateRecentProjects(fileName, true); }		

	private:
		bool loadProject(QString fileName);
		bool saveProject(QString fileName);
		bool loadProjectOnStartup();
		bool handleUnsavedProjectChanges();
		void updateRecentProjects(QString fileName, bool addEntry);

		void tabifyImageEditorDocks();

	private:
		void setupUi();
		Ui::GrinderWindow* ui;
	};
}

#endif
