/******************************************************************************
 * File: ImageReferencesListItem.cpp
 * Date: 10.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageReferencesListItem.h"
#include "util/StringUtils.h"
#include "res/Resources.h"

#include <opencv2/highgui.hpp>

ImageReferencesListItem::ImageReferencesListItem(ImageReference* imageRef) : ActiveObjectListItem(imageRef)
{
	setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);
	setIcon(QIcon{FILE_ICON_IMAGEREFERENCE});

	updateItem();
}

bool ImageReferencesListItem::operator <(const QListWidgetItem& other) const
{
	auto imageRef = dynamic_cast<const ImageReferencesListItem&>(other);
	return *_object < *imageRef._object;
}

void ImageReferencesListItem::viewImage() const
{
	// Show the image using OpenCV
	if (_object->isValid())
	{
		auto image = cv::imread(_object->getImageFilePath().toStdString());

		if (!image.empty())
			cv::imshow(_object->getImageFileName().toStdString(), image);
	}
}

void ImageReferencesListItem::updateItem()
{
	ActiveObjectListItem::updateItem();

	// Create the tooltip string
	auto imageInfo = _object->getImageInfo();
	QFileInfo fileInfo{_object->getImageFilePath()};
	QString toolTip = QString{"<nobr><b>%1</b></nobr><br><em>Location: </em>%6<br><em>File size: </em>%2<br><em>File date: </em>%3<br><em>Dimensions: </em>%4 x %5 pixels"}
			.arg(_object->getImageFileName())
			.arg(StringUtils::fileSizeToString(imageInfo.fileSize))
			.arg(imageInfo.fileDate.toString(Qt::SystemLocaleShortDate))
			.arg(imageInfo.imageSize.width()).arg(imageInfo.imageSize.height())
			.arg(fileInfo.path());

	setText(getName());
	setToolTip(toolTip);
}
