/******************************************************************************
 * File: ImageReferencesListWidget.h
 * Date: 10.2.2018
 *****************************************************************************/

#ifndef IMAGEREFERENCESLISTWIDGET_H
#define IMAGEREFERENCESLISTWIDGET_H

#include "ui/widgets/MetaWidget.h"
#include "ui/widgets/ActiveObjectListWidget.h"
#include "ImageReferencesListItem.h"

namespace grndr
{
	class ControlBar;
	class ImageReference;

	using ImageReferenceObjectListWidget = ActiveObjectListWidget<ImageReference, ImageReferencesListItem>;

	class ImageReferencesListWidget : public MetaWidget<ImageReferenceObjectListWidget>
	{
		Q_OBJECT

	public:
		ImageReferencesListWidget(QWidget* parent = nullptr);

	protected:
		virtual std::unique_ptr<QMenu> createContextMenu() const override;

		virtual std::vector<QAction*> getActions(AddActionsMode mode) const override;

	protected:
		virtual void switchToObjectItem(ImageReferencesListItem* item, bool selectItem = true) override;

	protected:
		virtual void dragEnterEvent(QDragEnterEvent* event) override;
		virtual void dragMoveEvent(QDragMoveEvent* event) override;
		virtual void dropEvent(QDropEvent* event) override;

	private slots:
		void switchImageReference() { switchToObjectItem(currentObjectItem()); }
		void nextImageReference() const;
		void previousImageReference() const;
		void viewImageReference();
		void addImageReferences();
		void addImageReferences(QStringList files);
		void removeImageReference();
		void removeAllImageReferences();

		void imageReferenceSwitched(ImageReference* imageRef);

		void updateActions();

	private:
		QAction* _switchImageReferenceAction{nullptr};
		QAction* _nextImageReferenceAction{nullptr};
		QAction* _previousImageReferenceAction{nullptr};
		QAction* _viewImageReferenceAction{nullptr};
		QAction* _addImageReferencesAction{nullptr};
		QAction* _removeImageReferenceAction{nullptr};
		QAction* _removeAllImageReferencesAction{nullptr};
	};
}

#endif
