/******************************************************************************
 * File: RecentProjectsMenu.cpp
 * Date: 01.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "RecentProjectsMenu.h"

void RecentProjectsMenu::setupUi(MRUStringList* recentProjects, std::function<void(QString)> loadProjectCallback)
{
	if (!recentProjects)
		throw std::invalid_argument{_EXCPT("recentProjects may not be null")};

	if (!loadProjectCallback)
		throw std::invalid_argument{_EXCPT("loadProjectCallback may not be null")};

	_recentProjects = recentProjects;
	_loadProjectCallback = loadProjectCallback;

	updateUi();
}

void RecentProjectsMenu::updateUi()
{
	clear();

	if (!_recentProjects->empty())
	{
		for (int i = 0; i < _recentProjects->size(); ++i)
		{
			auto fileName = _recentProjects->at(i);
			auto actionText = fileName;

			if (i < 9)
				actionText.insert(0, QString{"&%1 | "}.arg(i + 1));

			QAction* action = new QAction{actionText};
			action->setData(fileName);	// Save the file name to retrieve it when executing
			addAction(action);

			connect(action, &QAction::triggered, this, &RecentProjectsMenu::loadRecentProject);
		}
	}
	else
		addAction("No items to display")->setEnabled(false);
}

void RecentProjectsMenu::loadRecentProject()
{
	if (auto action = dynamic_cast<QAction*>(sender()))
	{
		QString fileName = action->data().toString();

		if (!fileName.isEmpty())
			_loadProjectCallback(fileName);
	}
}
