/******************************************************************************
 * File: BlockPropertyTreeItem.cpp
 * Date: 06.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "BlockPropertyTreeItem.h"
#include "pipeline/Block.h"
#include "res/Resources.h"

BlockPropertyTreeItem::BlockPropertyTreeItem(const std::shared_ptr<grndr::Block>& block) :
	_block{block}
{
	if (!block)
		throw std::invalid_argument{_EXCPT("block may not be null")};

	setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);

	QFont fontBold = font(0);
	fontBold.setBold(true);
	setFont(0, fontBold);

	setBackgroundColor(0, QPalette().color(QPalette::Button));

	updateItem();

	// Update this item if the pipeline item has been renamed; use the internal helper since BlockPropertyTreeItem can't be a QObject
	if (auto block = _block.lock())
		block->connect(block.get(), &PipelineItem::itemRenamed, _helper.get(), &_PropertyTreeItemHelper::updateItem);
}

void BlockPropertyTreeItem::updateItem()
{
	if (auto block = _block.lock())	// Make sure that the underlying block still exists
	{
		setText(0, block->getName());
		setToolTip(0, text(0));
		setIcon(0, QIcon{FILE_ICON_BLOCK});
	}
}
