/******************************************************************************
 * File: PropertyTreeWidget.h
 * Date: 06.3.2018
 *****************************************************************************/

#ifndef PROPERTYTREEWIDGET_H
#define PROPERTYTREEWIDGET_H

#include <QTreeWidget>
#include <QLabel>
#include <memory>

namespace grndr
{
	class Pipeline;
	class PipelineItem;
	class Block;
	class GraphBlockNode;
	class GraphConnectionNode;
	class ValuePropertyTreeItem;

	class PropertyTreeWidget : public QTreeWidget
	{
		Q_OBJECT

		friend class PropertyTreeItemDelegate;

	public:
		PropertyTreeWidget(QWidget *parent = nullptr);

	public:
		void setupUi(QLabel* propertyDescLabel);

		void clear() { QTreeWidget::clear(); updateActions(); updatePropertyDescription(); }

	protected:
		virtual void showEvent(QShowEvent* event) override;

		virtual void contextMenuEvent(QContextMenuEvent* event) override;
		virtual void mouseDoubleClickEvent(QMouseEvent* event) override;
		virtual void keyPressEvent(QKeyEvent* event) override;

	private:
		void addBlocks(const std::vector<std::shared_ptr<Block>>& blocks);
		void addProperties(const PipelineItem* pipelineItem, QTreeWidgetItem* parentItem);

	private slots:
		void itemSelectionChanged();
		void itemClicked(QTreeWidgetItem* item, int column);
		void editCurrentItem();

		void pipelineSwitching(Pipeline* pipeline);
		void pipelineSwitched(Pipeline* pipeline);

		void sceneSelectionChanged();

		void expandAllItems() { expandAllItems(true); }
		void collapseAllItems() { expandAllItems(false); }

		void updateActions();
		void updatePropertyDescription();

	private:
		ValuePropertyTreeItem* valuePropertyItem(const QModelIndex& index) const;
		ValuePropertyTreeItem* currentValuePropertyItem() const;

	private:
		void updateColumnWidths();

		void expandAllItems(bool expand);

		bool isEditing() { return state() == EditingState; }

	private:
		QAction* _editItem{nullptr};
		QAction* _expandAllAction{nullptr};
		QAction* _collapseAllAction{nullptr};				

		QLabel* _propertyDescLabel;
	};
}

#endif
