/******************************************************************************
 * File: ValuePropertyTreeItem.cpp
 * Date: 07.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ValuePropertyTreeItem.h"
#include "common/properties/PropertyBase.h"

ValuePropertyTreeItem::ValuePropertyTreeItem(const std::shared_ptr<PropertyBase>& property) :
	_property{property}
{
	if (!property)
		throw std::invalid_argument{_EXCPT("property may not be null")};

	setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable|Qt::ItemIsEditable);
	setTextColor(1, QColor{20, 105, 140});

	updateItem();

	// Update this item if the property value has changed; use the internal helper since PropertyTreeItem can't be a QObject
	if (auto property = _property.lock())
		property->connect(property.get(), &PropertyBase::valueChanged, _helper.get(), &_PropertyTreeItemHelper::updateItem);
}

QWidget* ValuePropertyTreeItem::createEditor(QWidget* parent) const
{
	if (auto property = _property.lock())	// Make sure that the underlying property still exists
		return property->createEditor(parent);
	else
		return nullptr;
}

void ValuePropertyTreeItem::updateItem()
{
	if (auto property = _property.lock())	// Make sure that the underlying property still exists
	{
		setText(0, property->getName());
		setToolTip(0, text(0));

		setText(1, property->toString());
		setToolTip(1, text(1));
	}
}
