/******************************************************************************
 * File: AnglePropertyEditor.cpp
 * Date: 26.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "AnglePropertyEditor.h"

AnglePropertyEditor::AnglePropertyEditor(AngleProperty* property, QWidget* parent) : PropertyEditor(property, parent),
	_angleDial{new QDial{}}, _angleEdit{new AutoFocusSpinBox{}}
{
	setFocusPolicy(Qt::NoFocus);

	_angleDial->setFixedWidth(50);
	_angleDial->setFixedHeight(50);
	_angleDial->setRange(0, 360);
	_angleDial->setWrapping(true);
	_angleDial->setNotchesVisible(true);
	_angleDial->setNotchTarget(45);
	_angleDial->setInvertedAppearance(true);
	_angleDial->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);

	_angleEdit->setRange(0, 360);
	_angleEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	// Create a horizontal layout and add all controls to it
	auto layout = new QHBoxLayout{};
	layout->setContentsMargins(0, 0, 0, 0);
	layout->addWidget(_angleDial);
	layout->addWidget(_angleEdit);
	layout->addWidget(new QLabel{"deg"});
	setLayout(layout);

	// Keep the dial and the text edit in sync; this also updates the property
	connect(_angleDial, &QDial::valueChanged, this, &AnglePropertyEditor::angleDialChanged);
	connect(_angleEdit, SIGNAL(valueChanged(int)), this, SLOT(angleValueChanged(int)));

	applyPropertyValue();
}

void AnglePropertyEditor::applyPropertyValue()
{
	_angleEdit->setValue(getPropertyValue().toInt());
}

void AnglePropertyEditor::angleDialChanged(int value)
{
	auto angle = (value + 270) % 360;

	if (angle != _angleEdit->value())
		_angleEdit->setValue(angle);
}

void AnglePropertyEditor::angleValueChanged(int value)
{
	auto angle = (value + 90) % 360;

	if (angle != _angleDial->value())
		_angleDial->setValue(angle);

	setPropertyValue(_angleEdit->text());
}
