/******************************************************************************
 * File: DualIntPropertyEditor.impl.h
 * Date: 23.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "DualIntPropertyEditor.h"

template<class PropertyType>
DualIntPropertyEditor<PropertyType>::DualIntPropertyEditor(PropertyType* property, QString name1, QString name2, Range<int> range1, Range<int> range2, QWidget* parent) : PropertyEditor<QWidget, PropertyType>(property, parent),
	_spinBox1{new AutoFocusSpinBox{}}, _spinBox2{new AutoFocusSpinBox{}}
{
	this->setFocusPolicy(Qt::NoFocus);

	_spinBox1->setRange(range1.first(), range1.second());
	_spinBox1->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	_spinBox2->setRange(range2.first(), range2.second());
	_spinBox2->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

	// Create a horizontal layout and add all controls to it
	auto layout = new QHBoxLayout{};
	layout->setContentsMargins(0, 0, 0, 0);
	layout->addWidget(new QLabel{name1});
	layout->addWidget(_spinBox1);
	layout->addWidget(new QLabel{name2});
	layout->addWidget(_spinBox2);
	this->setLayout(layout);

	// Update the property whenever editing has been finished (can't use SIGNAL/SLOT due to template, so we have to use QOverload instead)
	this->connect(_spinBox1, QOverload<int>::of(&QSpinBox::valueChanged), this, &DualIntPropertyEditor::valueChanged);
	this->connect(_spinBox2, QOverload<int>::of(&QSpinBox::valueChanged), this, &DualIntPropertyEditor::valueChanged);
}
