/******************************************************************************
 * File: EnumPropertyEditor.cpp
 * Date: 12.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "EnumPropertyEditor.h"

EnumPropertyEditor::EnumPropertyEditor(EnumPropertyBase* property, QWidget *parent) : PropertyEditor(property, parent)
{
	// Fill the enum values
	populateEnumValues();

	// Update the property whenever the current index has been changed
	connect(this, SIGNAL(currentIndexChanged(int)), this, SLOT(currentIndexChanged(int)));
}

void EnumPropertyEditor::applyPropertyValue()
{
	bool itemSelected = false;

	// Search for the corresponding enum value in the list
	for (int i = 0; i < count(); ++i)
	{
		auto enumValue = itemData(i).toInt();

		if (enumValue == _property->getValue())
		{
			setCurrentIndex(i);
			itemSelected = true;
			break;
		}
	}

	// If no item was selected, select the first one
	if (!itemSelected)
		setCurrentIndex(0);
}

void EnumPropertyEditor::populateEnumValues()
{
	clear();

	// Get all enum names and sort them
	std::vector<std::pair<int, QString>> enumNames;

	for (const auto& enumEntry : _property->enumValues())
		enumNames.emplace_back(enumEntry.first, enumEntry.second);

	std::sort(enumNames.begin(), enumNames.end(), [](auto entry1, auto entry2) { return entry1.second < entry2.second; });

	// Add all enum names
	for (const auto& enumName : enumNames)
		addItem(enumName.second, enumName.first);

	applyPropertyValue();
}

void EnumPropertyEditor::currentIndexChanged(int index)
{
	_property->setValue(itemData(index).toInt());
}
