/******************************************************************************
 * File: IntPropertyEditor.cpp
 * Date: 28.5.2018
 *****************************************************************************/

#include "Grinder.h"
#include "IntPropertyEditor.h"
#include "common/Range.h"
#include "common/properties/constraints/RangeConstraint.h"
#include "common/properties/constraints/EvenOddConstraint.h"

IntPropertyEditor::IntPropertyEditor(PropertyBase* property, const PropertyConstraints* constraints, QWidget* parent) : PropertyEditor(property, parent)
{
	setSingleStep(1);
	setRange(std::numeric_limits<int>::min(), std::numeric_limits<int>::max());

	if (constraints)
		applyPropertyConstraints(*constraints);

	applyPropertyValue();

	// Update the property whenever the value has changed
	connect(this, SIGNAL(valueChanged(int)), this, SLOT(valueChanged()));
}

void IntPropertyEditor::applyPropertyConstraints(const PropertyConstraints& constraints)
{
	auto rangeSetter = [this](auto min, auto max) {
		if (static_cast<unsigned int>(max) > static_cast<unsigned int>(std::numeric_limits<int>::max()))
			max = std::numeric_limits<int>::max();

		setRange(static_cast<int>(min), static_cast<int>(max));
	};

	for (auto& constraint : constraints)
	{
		// Apply range constraints
		if (auto rangeConstraint = dynamic_cast<RangeConstraint<int>*>(constraint.get()))
		{
			auto range = rangeConstraint->range();
			rangeSetter(range.first(), range.second());
		}
		else if (auto rangeConstraint = dynamic_cast<RangeConstraint<unsigned int>*>(constraint.get()))
		{
			auto range = rangeConstraint->range();
			rangeSetter(range.first(), range.second());
		}

		// Apply even/odd constraints
		if (dynamic_cast<EvenOddConstraint<int>*>(constraint.get()) || dynamic_cast<EvenOddConstraint<unsigned int>*>(constraint.get()))
			setSingleStep(2);
	}
}

void IntPropertyEditor::applyPropertyValue()
{
	setValue(getPropertyValue().toInt());
}

void IntPropertyEditor::valueChanged()
{
	setPropertyValue(cleanText());
}
