/******************************************************************************
 * File: RealPropertyEditor.cpp
 * Date: 07.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "RealPropertyEditor.h"
#include "common/Range.h"
#include "common/properties/constraints/RangeConstraint.h"
#include "common/properties/constraints/EvenOddConstraint.h"
#include "util/StringConv.h"

RealPropertyEditor::RealPropertyEditor(PropertyBase* property, const PropertyConstraints* constraints, QWidget* parent) : PropertyEditor(property, parent)
{
	setSingleStep(0.1);
	setRange(std::numeric_limits<double>::min(), std::numeric_limits<double>::max());

	if (constraints)
		applyPropertyConstraints(*constraints);

	applyPropertyValue();

	// Update the property whenever the value has changed
	connect(this, SIGNAL(valueChanged(double)), this, SLOT(valueChanged()));
}

void RealPropertyEditor::applyPropertyConstraints(const PropertyConstraints& constraints)
{
	for (auto& constraint : constraints)
	{
		// Apply range constraints
		if (auto rangeConstraint = dynamic_cast<RangeConstraint<double>*>(constraint.get()))
		{
			auto range = rangeConstraint->range();
			setRange(range.first(), range.second());
		}
	}
}

void RealPropertyEditor::applyPropertyValue()
{
	setValue(getPropertyValue().toDouble());
}

void RealPropertyEditor::valueChanged()
{
	setPropertyValue(StringConv::convertValue(value()));
}
