/******************************************************************************
 * File: RubberBandNode.cpp
 * Date: 05.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "RubberBandNode.h"

RubberBandNode::RubberBandNode(QGraphicsScene* scene, QPointF pos) : QObject(scene),
	_scene{scene}, _startPosition{pos}
{
	_rubberBand = new QRubberBand{QRubberBand::Rectangle};
	_rubberBand->move(pos.toPoint());
	_rubberBand->resize(0, 0);
	_rubberBand->show();

	_rubberBandNode = scene->addWidget(_rubberBand);
	_rubberBandNode->setZValue(std::numeric_limits<qreal>::max());
}

RubberBandNode::~RubberBandNode()
{
	if (_scene)
	{
		for (auto& view : _scene->views())
		{
			// Fix Qt's cursor hell
			auto dragMode = view->dragMode();
			view->setDragMode(QGraphicsView::ScrollHandDrag);
			view->setDragMode(dragMode);
		}
	}

	if (_rubberBandNode)
		delete _rubberBandNode;
}

void RubberBandNode::move(QPointF pos)
{
	auto delta = pos - _startPosition;
	QPointF topLeft{std::min(pos.x(), pos.x() - delta.x()), std::min(pos.y(), pos.y() - delta.y())};
	QRect rect{topLeft.toPoint(), QSizeF{std::abs(delta.x()), std::abs(delta.y())}.toSize()};
	_rubberBand->setGeometry(rect);
}
