/******************************************************************************
 * File: VisualNode.h
 * Date: 21.3.2018
 *****************************************************************************/

#ifndef VISUALNODE_H
#define VISUALNODE_H

#include <QGraphicsObject>
#include <QAction>

namespace grndr
{
	class VisualNode : public QGraphicsObject
	{
		Q_OBJECT

	public:
		VisualNode(QGraphicsScene* scene, QGraphicsItem* parent = nullptr);

	public:
		virtual QRectF boundingRect() const override { return boundingRect(isSelected()); }
		QRectF boundingRect(bool selectedRect) const;

	signals:
		void nodeSelected(VisualNode*);
		void nodeDeselected(VisualNode*);
		void nodeMoved(VisualNode*);
		void nodeGeometryUpdated(VisualNode*);

	protected:
		virtual QVariant itemChange(GraphicsItemChange change, const QVariant& value) override;
		virtual void contextMenuEvent(QGraphicsSceneContextMenuEvent* event) override;

	protected:
		virtual void updateGeometry() { emit nodeGeometryUpdated(this); }

		virtual std::vector<QAction*> getNodeActions(QMenu& menu) const { Q_UNUSED(menu); return {}; }
		virtual std::vector<QAction*> getNodesActions(QMenu& menu) const { Q_UNUSED(menu); return {}; }
		QAction* createNodeAction(QString name, QString icon = "", const char* callback = nullptr, QString help = "", QString shortcut = "");

		void showContextMenu(QPoint pos) const;

	protected:
		QGraphicsScene* _graphicsScene{nullptr};

		QRectF _nodeRect;
		QRectF _nodeRectSelected;
	};
}

#endif
