/******************************************************************************
 * File: VisualNodeFactory.h
 * Date: 21.3.2018
 *****************************************************************************/

#ifndef VISUALNODEFACTORY_H
#define VISUALNODEFACTORY_H

#include <memory>
#include <functional>
#include <map>

#define REGISTER_NODEFACTORY_TYPE(cls)	registerType(cls::type_value, [scene = _scene](const std::shared_ptr<object_type>& object) { return new cls(scene, object); })

namespace grndr
{
	template<typename NodeType, typename ObjType, typename KeyType, typename SceneType>
	class VisualNodeFactory
	{
	public:
		using node_type = NodeType;
		using object_type = ObjType;
		using key_type = KeyType;
		using scene_type = SceneType;

	private:
		using node_creator_type = std::function<node_type*(const std::shared_ptr<object_type>&)>;

	public:
		VisualNodeFactory(scene_type* scene);

	public:
		void registerType(key_type type, node_creator_type creator);
		void unregisterType(key_type type);

		node_type* createNode(const std::shared_ptr<object_type>& object) const;

	protected:
		virtual node_type* createDefaultNode(const std::shared_ptr<object_type>& object) const = 0;

	protected:
		scene_type* _scene{nullptr};

	private:
		std::map<key_type, node_creator_type> _nodeCreators;
	};
}

#include "VisualNodeFactory.impl.h"

#endif
