/******************************************************************************
 * File: CheckListWidget.impl.h
 * Date: 25.4.2018
 *****************************************************************************/

#include "Grinder.h"
#include "CheckListWidget.h"
#include "util/UIUtils.h"
#include "res/Resources.h"

template<typename ObjectType, typename ItemType>
CheckListWidget<ObjectType, ItemType>::CheckListWidget(QWidget* parent) : ObjectListWidget<ObjectType, ItemType>(parent)
{
	_unCheckSelectedItems = UIUtils::createAction(this, "&Check selected", FILE_ICON_CHECK, SLOT(toggleCheckSelectedItems()), "Check the selected items", "Space");
	_checkAllItems = UIUtils::createAction(this, "Check &all", FILE_ICON_CHECKALL, SLOT(checkAllItems()), "Check all items", "Ctrl+Shift+A");
	_uncheckAllItems = UIUtils::createAction(this, "Unchec&k all", "", SLOT(uncheckAllItems()), "Uncheck all items");

	_selectAllAction = UIUtils::createAction(this, "&Select all", FILE_ICON_SELECTALL, SLOT(selectAll()), "Select all items", "Ctrl+A");
	_deselectAllAction = UIUtils::createAction(this, "&Deselect all", "", SLOT(clearSelection()), "Deselect all items");
	_invertSelectionAction = UIUtils::createAction(this, "&Invert selection", "", SLOT(invertSelection()), "Invert the current selection", "Ctrl+I");
}

template<typename ObjectType, typename ItemType>
template<typename ContainerType>
void CheckListWidget<ObjectType, ItemType>::populateList(const ContainerType& container)
{
	ObjectListWidget<ObjectType, ItemType>::populateList(container);

	for (int i = 0; i < this->count(); ++i)
	{
		if (auto item = this->item(i))
		{
			item->setFlags(item->flags()|Qt::ItemIsUserCheckable);
			item->setCheckState(Qt::Unchecked);
		}
	}
}

template<typename ObjectType, typename ItemType>
std::vector<ObjectType*> CheckListWidget<ObjectType, ItemType>::getCheckedObjects() const
{
	std::vector<ObjectType*> checkedItems;

	for (int i = 0; i < this->count(); ++i)
	{
		if (auto item = dynamic_cast<ItemType*>(this->item(i)))
		{
			if (item->checkState() == Qt::Checked)
				checkedItems.push_back(item->object());
		}
	}

	return checkedItems;
}

template<typename ObjectType, typename ItemType>
void CheckListWidget<ObjectType, ItemType>::setCheckedObjects(const std::vector<ObjectType*>& checkedObjects)
{
	for (int i = 0; i < this->count(); ++i)
	{
		if (auto item = dynamic_cast<ItemType*>(this->item(i)))
		{
			if (std::find(checkedObjects.cbegin(), checkedObjects.cend(), item->object()) != checkedObjects.cend())
				item->setCheckState(Qt::Checked);
			else
				item->setCheckState(Qt::Unchecked);
		}
	}
}

template<typename ObjectType, typename ItemType>
void CheckListWidget<ObjectType, ItemType>::contextMenuEvent(QContextMenuEvent* event)
{
	// Update the text of the check/uncheck selected action
	auto selItems = this->selectedItems();

	if (!selItems.isEmpty())
	{
		bool checkItems = (selItems[0]->checkState() != Qt::Checked);

		if (checkItems)
		{
			_unCheckSelectedItems->setText("&Check selected");
			_unCheckSelectedItems->setStatusTip("Check the selected items");
		}
		else
		{
			_unCheckSelectedItems->setText("&Uncheck selected");
			_unCheckSelectedItems->setStatusTip("Uncheck the selected items");
		}
	}

	QMenu menu;

	menu.addAction(_unCheckSelectedItems);
	menu.addSeparator();
	menu.addAction(_checkAllItems);
	menu.addAction(_uncheckAllItems);
	menu.addSeparator();
	menu.addAction(_selectAllAction);
	menu.addAction(_deselectAllAction);
	menu.addAction(_invertSelectionAction);

	menu.exec(event->globalPos());
}
