/******************************************************************************
 * File: ColorWidget.h
 * Date: 24.3.2018
 *****************************************************************************/

#ifndef COLORWIDGET_H
#define COLORWIDGET_H

#include <QWidget>

namespace grndr
{
	class ColorWidget : public QWidget
	{
		Q_OBJECT

	public:
		enum class Flag
		{
			NoFlag = 0x0000,
			SmallBox = 0x0001,
			SelectColorOnDoubleClick = 0x0002,
		};

		Q_DECLARE_FLAGS(Flags, Flag)

	public:
		ColorWidget(QWidget* parent = nullptr);
		ColorWidget(Flags flags, QWidget* parent = nullptr);

	public:
		QColor getColor() const { return _color; }
		void setColor(QColor color) { _color = color; updateWidget(); emit colorChanged(color, false); }
		std::pair<bool, QColor> getColorComp() const { return std::make_pair(_showColorComp, _colorComp); }
		void setColorComp(bool show, QColor color = QColor{}) { _showColorComp = show; _colorComp = color; updateWidget(); }

		QString getExtraToolTip() const { return _extraToolTip; }
		void setExtraToolTip(QString toolTip) { _extraToolTip = toolTip; updateWidget(); }

	public:
		virtual QSize sizeHint() const override;

	signals:
		void colorChanged(QColor, bool);
		void colorClicked(QColor);

	protected:
		virtual void paintEvent(QPaintEvent* event) override;
		virtual void mouseReleaseEvent(QMouseEvent* event) override;
		virtual void mouseDoubleClickEvent(QMouseEvent* event) override;

	private:
		void selectColor();

		void updateWidget();

	private slots:
		void colorDialogColorChanged(const QColor& color);

	private:
		Flags _flags{Flag::NoFlag};

		QColor _color;

		bool _showColorComp{false};
		QColor _colorComp;

		QString _extraToolTip{""};
	};
}

Q_DECLARE_OPERATORS_FOR_FLAGS(grndr::ColorWidget::Flags)

#endif
