/******************************************************************************
 * File: ControlBar.cpp
 * Date: 07.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ControlBar.h"
#include "ui/StyleSheet.h"
#include "res/Resources.h"

ControlBar::ControlBar(QWidget *parent) : QFrame(parent),
	_layout{new QHBoxLayout{this}}
{
	_layout->setContentsMargins(2, 2, 2, 2);
	_layout->addStretch();

	setStyleSheet(StyleSheet::loadStyleSheet(FILE_STYLESHEET_CONTROLBAR));
}

void ControlBar::addWidget(QWidget* widget, Qt::Alignment alignment)
{
	if (alignment == 0)
		alignment = _defaultAlignment;

	if (alignment == Qt::AlignRight)
		_layout->addWidget(widget);
	else if (alignment == Qt::AlignLeft)
		_layout->insertWidget(_layout->count() - 1, widget);

	setMinimumSize(0, sizeHint().height());
}

void ControlBar::addAction(QAction* action, Qt::ToolButtonStyle buttonStyle, Qt::Alignment alignment, QMenu* popupMenu)
{
	if (buttonStyle == Qt::ToolButtonFollowStyle)
		buttonStyle = _defaultToolButtonStyle;

	if (action->data().isValid())	// Actions can specify a tool button style in their data field
		buttonStyle = static_cast<Qt::ToolButtonStyle>(action->data().toInt());

	auto toolButton = new QToolButton{};

	toolButton->setDefaultAction(action);
	toolButton->setToolButtonStyle(buttonStyle);
	toolButton->setAutoRaise(true);

	if (popupMenu)
	{
		toolButton->setMenu(popupMenu);
		toolButton->setPopupMode(QToolButton::MenuButtonPopup);
	}

	addWidget(toolButton, alignment);
}

void ControlBar::addSeparator(Qt::Alignment alignment)
{
	auto line = new QFrame{};

	line->setFrameShape(QFrame::VLine);
	line->setFrameShadow(QFrame::Plain);

	addWidget(line, alignment);
}
