/******************************************************************************
 * File: MetaWidget.impl.h
 * Date: 08.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "MetaWidget.h"

template<typename Base>
template<typename T>
int MetaWidget<Base>::setupActions(T* widget, AddActionsMode mode) const
{
	auto actions = getActions(mode);

	for (auto action : actions)
	{
		if (action)
			widget->addAction(action);
		else
			widget->addSeparator();
	}

	return actions.size();
}

template<typename Base>
template<typename MenuType, typename BarType>
void MetaWidget<Base>::setupUi(MenuType* menu, BarType* toolbar) const
{
	// Add actions to the provided widgets
	if (menu)
		setupActions(menu, AddActionsMode::MainMenu);

	if (toolbar)
		setupActions(toolbar, AddActionsMode::Toolbar);
}

template<typename Base>
std::unique_ptr<QMenu> MetaWidget<Base>::createContextMenu() const
{
	auto menu = std::make_unique<QMenu>();
	setupActions(menu.get(), AddActionsMode::ContextMenu);
	return menu;
}

template<typename Base>
void MetaWidget<Base>::contextMenuEvent(QContextMenuEvent* event)
{
	auto menu = createContextMenu();

	if (menu && !menu->isEmpty())
	{
		menu->exec(event->globalPos());
		event->accept();
	}
}
