/******************************************************************************
 * File: ObjectListWidget.h
 * Date: 10.2.2018
 *****************************************************************************/

#ifndef OBJECTLISTWIDGET_H
#define OBJECTLISTWIDGET_H

#include "BaseListWidget.h"
#include "ObjectListItem.h"

namespace grndr
{
	class ControlBar;
	class Label;

	template<typename ObjectType, typename ItemType>
	class ObjectListWidget : public BaseListWidget
	{
		static_assert(std::is_base_of<ObjectListItem<std::remove_const_t<ObjectType>>, ItemType>::value, "ItemType must be derived from ObjectListItem<ObjectType>");

	public:
		using object_type = ObjectType;
		using item_type = ItemType;

	public:
		using BaseListWidget::BaseListWidget;

	public:		
		virtual item_type* addObject(object_type* obj, bool autoSelect = true, bool insertAtFront = false);
		virtual void removeObject(const object_type* obj, bool autoSelect = true);

		object_type* currentObject() const;		

		void updateObject(const object_type* obj) const;
		void updateAllObjects() const;

		template<typename ContainerType>
		void populateList(const ContainerType& container, bool selectFirst = true, bool reverseOrder = false);

	protected:
		item_type* objectItem(int index) const;
		item_type* currentObjectItem() const { return objectItem(currentRow()); }
		virtual void setCurrentObjectItem(item_type* item);

		item_type* findObjectItem(const object_type* obj) const;
		int findObjectItemIndex(const item_type* item);
	};
}

#include "ObjectListWidget.impl.h"

#endif
