/******************************************************************************
 * File: ObjectListWidget.impl.h
 * Date: 10.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ObjectListWidget.h"

template<typename ObjectType, typename ItemType>
ItemType* ObjectListWidget<ObjectType, ItemType>::addObject(object_type* obj, bool autoSelect, bool insertAtFront)
{
	auto item = new ItemType{obj};

	if (insertAtFront)
		insertItem(0, item);
	else
		addItem(item);

	if (autoSelect)
		setCurrentItem(item);

	return item;
}

template<typename ObjectType, typename ItemType>
void ObjectListWidget<ObjectType, ItemType>::removeObject(const object_type* obj, bool autoSelect)
{
	int curRow = currentRow();

	if (auto item = findObjectItem(obj))
	{
		auto taken = takeItem(indexFromItem(item).row());
		delete taken;

		if (autoSelect)
			setCurrentRow(std::min(curRow, count() - 1));
	}
}

template<typename ObjectType, typename ItemType>
ObjectType* ObjectListWidget<ObjectType, ItemType>::currentObject() const
{
	auto item = currentObjectItem();

	if (item)
		return item->object();
	else
		return nullptr;
}

template<typename ObjectType, typename ItemType>
void ObjectListWidget<ObjectType, ItemType>::updateObject(const object_type* obj) const
{
	if (auto item = findObjectItem(obj))
		item->updateItem();
}

template<typename ObjectType, typename ItemType>
void ObjectListWidget<ObjectType, ItemType>::updateAllObjects() const
{
	for (int i = 0; i < count(); ++i)
	{
		if (auto item = objectItem(i))
			item->updateItem();
	}
}

template<typename ObjectType, typename ItemType>
template<typename ContainerType>
void ObjectListWidget<ObjectType, ItemType>::populateList(const ContainerType& container, bool selectFirst, bool reverseOrder)
{
	clear();

	bool firstItem = true;

	for (const auto& item : container)
	{
		addObject(item.get(), firstItem && selectFirst, reverseOrder);
		firstItem = false;
	}

	if (!firstItem && selectFirst)	// There's at least one item, so select the first one
		setCurrentObjectItem(objectItem(0));
	else
		setCurrentObjectItem(nullptr);
}

template<typename ObjectType, typename ItemType>
ItemType* ObjectListWidget<ObjectType, ItemType>::objectItem(int index) const
{
	if (index < 0 || index >= count())
		return nullptr;

	return dynamic_cast<item_type*>(item(index));
}

template<typename ObjectType, typename ItemType>
void ObjectListWidget<ObjectType, ItemType>::setCurrentObjectItem(item_type* item)
{
	setCurrentItem(item);
}

template<typename ObjectType, typename ItemType>
ItemType* ObjectListWidget<ObjectType, ItemType>::findObjectItem(const object_type* obj) const
{
	for (int i = 0; i < count(); ++i)
	{
		if (auto item = objectItem(i))
		{
			if (item->object() == obj)
				return item;
		}
	}

	return nullptr;
}

template<typename ObjectType, typename ItemType>
int ObjectListWidget<ObjectType, ItemType>::findObjectItemIndex(const item_type* item)
{
	for (int i = 0; i < count(); ++i)
	{
		if (objectItem(i) == item)
			return i;
	}

	return -1;
}
