/******************************************************************************
 * File: SliderValueWidget.cpp
 * Date: 15.6.2018
 *****************************************************************************/

#include "Grinder.h"
#include "SliderValueWidget.h"
#include "core/GrinderApplication.h"

SliderValueWidget::SliderValueWidget(QString label, int minValue, int maxValue, QString valueSuffix, QWidget* parent) : QWidget(parent),
	_layout{new QHBoxLayout{this}}
{
	_layout->setContentsMargins(4, 2, 4, 2);
	_layout->setSpacing(6);

	_slider = new QSlider{Qt::Horizontal};
	_slider->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
	_slider->setRange(minValue, maxValue);
	_slider->setTickInterval((maxValue - minValue) / 20);
	_slider->setTickPosition(QSlider::TicksBelow);

	_spinBox = new QSpinBox{};
	_spinBox->setRange(minValue, maxValue);
	_spinBox->setSuffix(valueSuffix);

	connect(_slider, &QSlider::valueChanged, this, &SliderValueWidget::sliderValueChanged);
	connect(_spinBox, QOverload<int>::of(&QSpinBox::valueChanged), this, &SliderValueWidget::spinBoxValueChanged);

	if (!label.isEmpty())
	{
		auto labelWidget = new QLabel{label};
		labelWidget->setFont(QApplication::font("QMenu"));
		_layout->addWidget(labelWidget);
	}

	_layout->addWidget(_slider);
	_layout->addWidget(_spinBox);
}

void SliderValueWidget::sliderValueChanged(int value)
{
	if (_spinBox->value() != value)
	{
		_spinBox->setValue(value);
		emit valueChanged(value);
	}
}

void SliderValueWidget::spinBoxValueChanged(int value)
{
	if (_slider->value() != value)
	{
		_slider->setValue(value);
		emit valueChanged(value);
	}
}
