/******************************************************************************
 * File: ImageUtils.cpp
 * Date: 10.2.2018
 *****************************************************************************/

#include "Grinder.h"
#include "ImageUtils.h"

QStringList ImageUtils::getSupportedFormats()
{
	// Return a list of formats supported by OpenCV
	return {"bmp", "dib", "jpeg", "jpg", "jpe", "jp2", "png", "webp", "pbm", "pgm", "ppm", "pxm", "pnm", "sr", "ras", "tiff", "tif", "exr", "hdr", "pic"};
}

QStringList ImageUtils::getSupportedFormatFilters()
{
	auto formats = getSupportedFormats();
	QStringList filters;

	for (auto format : formats)
		filters << "*." + format;

	return filters;
}

int sgn(int x){
  return (x > 0) ? 1 : (x < 0) ? -1 : 0;
}

std::vector<QPoint> ImageUtils::getLinePoints(int xstart, int ystart, int xend, int yend)
{
	std::vector<QPoint> points;

	// Midpoint-Line algorithm (based on Wikipedia)
	static auto sgn = [](auto x) { return (x > 0) ? 1 : (x < 0) ? -1 : 0; };

	int dx = xend - xstart;
	int dy = yend - ystart;
	int incx = sgn(dx);
	int incy = sgn(dy);

	if (dx < 0)
		dx = -dx;

	if (dy < 0)
		dy = -dy;

	int pdx, pdy, ddx, ddy, deltaslowdirection, deltafastdirection;

	if (dx > dy)
	{
		pdx = incx;
		pdy = 0;
		ddx = incx;
		ddy = incy;
		deltaslowdirection = dy;
		deltafastdirection = dx;
	}
	else
	{
		pdx = 0;
		pdy = incy;
		ddx = incx;
		ddy = incy;
		deltaslowdirection = dx;
		deltafastdirection = dy;
	}

	int x = xstart;
	int y = ystart;
	int err = deltafastdirection / 2;

	points.emplace_back(x, y);

	for (int t = 0; t < deltafastdirection; ++t)
	{
		err -= deltaslowdirection;

		if (err < 0)
		{
			err += deltafastdirection;
			x += ddx;
			y += ddy;
		}
		else
		{
			x += pdx;
			y += pdy;
		}

		points.emplace_back(x, y);
	}

	return points;
}
