/******************************************************************************
 * File: StringConv.h
 * Date: 12.1.2018
 *****************************************************************************/

#ifndef STRINGCONV_H
#define STRINGCONV_H

#include <QString>
#include <QPoint>
#include <QSize>
#include <QColor>

namespace grndr
{
	class StringConv final
	{
	public:
		template<typename ValueType, typename std::enable_if<!std::is_pointer<ValueType>{}, int>::type = 0>
		static QString convertValue(const ValueType& val)
		{
			// Default implementation for non-pointer types
			return QString{"%1"}.arg(val);
		}

		template<typename ValueType, typename std::enable_if<std::is_pointer<ValueType>{}, int>::type = 0>
		static QString convertValue(const ValueType& val)
		{
			// Pointer types do not have string representations
			Q_UNUSED(val);
			return "";
		}

		template<typename ValueType, typename std::enable_if<!std::is_pointer<ValueType>{}, int>::type = 0>
		static ValueType convertString(const QString& str, bool* ok = nullptr);

		template<typename ValueType, typename std::enable_if<std::is_pointer<ValueType>{}, int>::type = 0>
		static ValueType convertString(const QString& str, bool* ok = nullptr)
		{
			// Pointer types do not have string representations
			Q_UNUSED(str);

			if (ok)
				*ok = false;

			return nullptr;
		}

	private:
		StringConv() { }
	};

	template<> QString StringConv::convertValue<bool>(const bool& val);
	template<> QString StringConv::convertValue<QPoint>(const QPoint& val);
	template<> QString StringConv::convertValue<QPointF>(const QPointF& val);
	template<> QString StringConv::convertValue<QSize>(const QSize& val);
	template<> QString StringConv::convertValue<QColor>(const QColor& val);

	template<> QString StringConv::convertString<QString>(const QString& str, bool* ok);
	template<> short StringConv::convertString<short>(const QString& str, bool* ok);
	template<> ushort StringConv::convertString<ushort>(const QString& str, bool* ok);
	template<> int StringConv::convertString<int>(const QString& str, bool* ok);
	template<> uint StringConv::convertString<uint>(const QString& str, bool* ok);
	template<> long StringConv::convertString<long>(const QString& str, bool* ok);
	template<> ulong StringConv::convertString<ulong>(const QString& str, bool* ok);
	template<> qlonglong StringConv::convertString<qlonglong>(const QString& str, bool* ok);
	template<> qulonglong StringConv::convertString<qulonglong>(const QString& str, bool* ok);
	template<> float StringConv::convertString<float>(const QString& str, bool* ok);
	template<> double StringConv::convertString<double>(const QString& str, bool* ok);
	template<> bool StringConv::convertString<bool>(const QString& str, bool* ok);
	template<> QPoint StringConv::convertString<QPoint>(const QString& str, bool* ok);
	template<> QPointF StringConv::convertString<QPointF>(const QString& str, bool* ok);
	template<> QSize StringConv::convertString<QSize>(const QString& str, bool* ok);
	template<> QColor StringConv::convertString<QColor>(const QString& str, bool* ok);
}

#endif
