/******************************************************************************
 * File: StringUtils.cpp
 * Date: 12.1.2018
 *****************************************************************************/

#include "Grinder.h"
#include "StringUtils.h"

QString StringUtils::encodeEscapeCharacters(QString str, bool includeQuotes)
{
	str.replace("\\", "\\\\");
	str.replace("\n", "\\n");
	str.replace("\r", "\\r");
	str.replace("\t", "\\t");

	if (includeQuotes)
		str.replace("\"", "\\\"");

	return str;
}

QString StringUtils::decodeEscapeCharacters(const QString& str, bool includeQuotes)
{
	QString	strOut;
	bool isEscape = false;

	for (QChar c : str)
	{
		if (isEscape)
		{
			if (c == 'n')
				c = '\n';
			else if (c == 'r')
				c = '\r';
			else if (c == 't')
				c = '\t';
			else if (c == '\\')
				c = '\\';
			else if (c == '\"' && includeQuotes)
				c = '\"';

			strOut += c;
			isEscape = false;
		}
		else
		{
			if (c == '\\')
				isEscape = true;
			else
				strOut += c;
		}
	}

	return strOut;
}

QString StringUtils::fileSizeToString(qint64 fileSize)
{
	static const QString units[] = {"B", "KB", "MB", "GB"};
	double size = fileSize;
	int unitIndex = 0;

	while (size > 1024.0)
	{
		size /= 1024.0;
		unitIndex++;
	}

	return QString{"%1 %2"}.arg(size, 0, 'f', 2).arg(units[unitIndex]);
}
