/******************************************************************************
 * File: StringUtils.impl.h
 * Date: 02.3.2018
 *****************************************************************************/

#include "Grinder.h"
#include "StringUtils.h"

template<typename ContainerType>
QString StringUtils::generateUniqueItemName(const ContainerType& container, QString name, std::function<QString(const typename ContainerType::value_type&)> nameGetter, bool caseSensitive, bool addSpaces)
{
	QString newName;

	// Convert names like ThisIsMyName to This Is My Name
	if (addSpaces)
	{
		bool prevUpper = true;

		for (QChar c : name)
		{
			if (c.isUpper())
			{
				if (!prevUpper)
					newName += " ";

				prevUpper = true;
			}

			newName += c;
			prevUpper = c.isUpper();
		}
	}
	else
		newName = name;

	unsigned int index = 1;

	// Add increasing numbers to the name until we've found a unique one
	while (true)
	{
		QString indexedName = QString{"%1 #%2"}.arg(newName).arg(index++);

		if (std::none_of(std::cbegin(container), std::cend(container), [indexedName, caseSensitive, nameGetter](const typename ContainerType::value_type& value) {
			QString existingName = nameGetter(value);
			return existingName.compare(indexedName, caseSensitive ? Qt::CaseSensitive : Qt::CaseInsensitive) == 0;
		}))
		{
			newName = indexedName;
			break;
		}
	}

	return newName;
}
